package pfc.prototipo;

import java.awt.TextArea;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;

import org.apache.log4j.Logger;

/**
 * <p>Gestion de ficheros fsicos</p>
 * @author Angel
 *
 */
public class PFC_ManejoFicheros 
{
	//Trazas
	private final static Logger moLog = Logger.getLogger(PFC_ManejoFicheros.class);

	public PFC_ManejoFicheros(){}
	
	/**
	 * <p>Obtiene el contenido de un fichero</p>
	 * @param psFichero Ruta y nombre del fichero
	 * @return
	 */
	public StringBuffer cargarFichero(String psFichero)
	{
		BufferedReader loLector = null;
		StringBuffer loLineaTotal = new StringBuffer();
		String lsLinea = null; 
		
		try 
		{
			//Cargamos fichero
			loLector = new BufferedReader(new FileReader(psFichero));
		} catch (FileNotFoundException e) 
		{
			moLog.error("Problemas al cargar fichero<"+psFichero+">. "+e);
		}

		try 
		{
			lsLinea = loLector.readLine();
			while (lsLinea != null)
			{
				loLineaTotal.append(lsLinea);
				loLineaTotal.append(System.getProperty("line.separator")); 
				lsLinea = loLector.readLine();
			} 
			loLector.close();
			 
		} catch (IOException e) 
		{
			moLog.error("Problemas al leer fichero<"+psFichero+">. "+e);
		}
		
		return loLineaTotal;
		
	}//cargarFichero

	/**
	 * <p>Guarda en disco el documento indicado</p>
	 * @param psFichero Nombre del fichero
	 * @param poDatos Contenido del contenido
	 * @throws Exception
	 */
	public static void guardarFichero(String psFichero, TextArea poDatos) throws Exception
	{
		PrintWriter loEscribir = null;
		
		try 
		{
			moLog.info("Peticion de guardar un fichero. Nombre del fichero<"+psFichero+">");
			loEscribir = new PrintWriter(psFichero);
			loEscribir.print(poDatos.getText());
			loEscribir.close();		
			
		} catch (FileNotFoundException e) 
		{
			moLog.error("Problemas al guardar el fichero<"+psFichero+">:"+e.getMessage());
			throw new Exception("No se ha podido guardar el documento.");
		}
	}//guardarFichero
	
	/**
	 * <p>Borra del disco el documento indicado</p>
	 * @param psFichero Nombre del fichero
	 * @throws Exception
	 */
	public static void borrarFichero(String psFichero) throws Exception
	{
		File loFichero = null;
		
		try 
		{
			moLog.info("Peticion de borrar un fichero. Nombre del fichero<"+psFichero+">");
			loFichero = new File(psFichero);
			loFichero.delete();
			
		} catch (Exception e) 
		{
			moLog.error("Problemas al eliminar el fichero<"+psFichero+">:"+e.getMessage());
			throw new Exception("No se ha podido eliminar el documento.");
		}
	}//borrarFichero
	
	/**
	 * <p>Convierte fichero a un array de Bytes</p>
	 * @param psRuta Ruta completa donde esta el fichero
	 * @return byte[] con el contenido del fichero
	 * @throws Exception
	 */
	public static byte[] getFicheroByteArray(String psRuta) throws Exception
	{
		File loFichero = null;
		InputStream loIn = null;
		long llTam;
		byte[] lvbDatos = null; 
        int liOffset = 0;
        int liNumRead = 0;

		
		// Inicializacion de variables
		loFichero = new File(psRuta);
		loIn = new FileInputStream(loFichero);				
        // Tamanyo del fichero
        llTam = loFichero.length();
        

        // Se verifica que el tamanyo del fichero no exceda de lo permitido
        if (llTam > Integer.MAX_VALUE) 
        {
            moLog.info("Se intenta enviar un fichero que excede el tamanyo permitido. Nombre del fichero<"+psRuta+"> Tamanyo<"+llTam+">");
            throw new Exception("Se intenta enviar un fichero que excede el tamanyo permitido");
        }

        // Se crea el array
        lvbDatos = new byte[(int)llTam];


        while ( (liOffset < lvbDatos.length) && ( (liNumRead=loIn.read(lvbDatos, liOffset, lvbDatos.length-liOffset)) >= 0) ) 
        {

        	liOffset += liNumRead;

        }

        // Se verifica que se hayan podido leer todos los datos 
        if (liOffset < lvbDatos.length) 
        {
            throw new Exception("No se ha podido convertir el fichero");
        }

        loIn.close();
        
        return lvbDatos;		
	}// getFicheroByteArray
	
	/**
	 * <p>Crea un fichero a partir de un byte[]</p>
	 * @param psRuta Ruta con nombre del fichero
	 * @param pvbDatos  Contenido del fichero
	 * @throws Exception
	 */
	public static void getByteArrayFile(String psRuta, byte[] pvbDatos) throws Exception
	{
		moLog.info("Peticion para crear un fichero a partir de un byte[] con el nombre<"+psRuta+">");
		// Crea el fichero en disco con el contenido pasado en el arrayByte
	    FileOutputStream loOut = new FileOutputStream(psRuta); 
	    loOut.write(pvbDatos);
	    loOut.close();
 	}//getByteArrayFile
}
