package pfc.prototipo;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.log4j.Logger;
import pfc.prototipo.bbdd.PFC_BDGestion;

/**
 * <p>Panel encargado de gestionar parametros de configuracion de la aplicacin</p>
 * @author Angel
 *
 */
public class PFC_PanelConfigurar  extends JPanel
{
	//Trazas
	private final static Logger moLog = Logger.getLogger(PFC_PanelConfigurar.class);

	private JLabel moEtiquetaDirectorio = null;    
	private JTextField moIntroduceDirectorio = null;
	private JLabel moEtiquetaSuperPeerIP = null;    
	private JTextField moIntroduceSuperPeerIP = null;
	private JLabel moEtiquetaSuperPeerPuerto = null;    
	private JTextField moIntroduceSuperPeerPuerto = null;
	private JButton moBotonGuardar = null;

    //Conexion con la base de datos
    private PFC_BDGestion moGestionBD = null;

	public PFC_PanelConfigurar(PFC_BDGestion poGestionBD)
	{	
		super(null);
		
		moGestionBD = poGestionBD;
		
		//Se aaden elementos necesarios en el panel
		moEtiquetaDirectorio = new JLabel(PFC_Constantes.ETIQUETA_DIRECTORIO);
		moIntroduceDirectorio = new JTextField();
		moEtiquetaSuperPeerIP = new JLabel(PFC_Constantes.ETIQUETA_SUPERPEER_IP);
		moIntroduceSuperPeerIP = new JTextField();
		moEtiquetaSuperPeerPuerto = new JLabel(PFC_Constantes.ETIQUETA_SUPERPEER_PUERTO);
		moIntroduceSuperPeerPuerto = new JTextField();
		moBotonGuardar = new JButton(PFC_Constantes.BOTON_GUARDAR);
		
		//Se inicializan los TextField con la informacion pedida
		moIntroduceDirectorio.setText(PFC_Constantes.DIRECTORIO);
		moIntroduceSuperPeerIP.setText(PFC_Constantes.SUPERPEER_IP);
		moIntroduceSuperPeerPuerto.setText(PFC_Constantes.SUPERPEER_PUERTO);
		
		//Se establece posicion de cada elemento
		moEtiquetaDirectorio.setBounds( 10, 30, 130, 20 );
		moIntroduceDirectorio.setBounds( 140, 30, 500, 20 );
		moEtiquetaSuperPeerIP.setBounds( 10, 60, 130, 20 );
		moIntroduceSuperPeerIP.setBounds( 140, 60, 200, 20 );
		moEtiquetaSuperPeerPuerto.setBounds( 10, 90, 130, 20 );
		moIntroduceSuperPeerPuerto.setBounds( 140, 90, 200, 20 );
		
		moBotonGuardar.setBounds( 360, 120, 80, 25 );
		
		// Se aaden eventos a los botones
		moBotonGuardar.addActionListener(new PFC_ModificarAccion());

		//Se aaden elementos al panel		
		add(moEtiquetaDirectorio);
		add(moIntroduceDirectorio);
		add(moEtiquetaSuperPeerIP);
		add(moIntroduceSuperPeerIP);
		add(moEtiquetaSuperPeerPuerto);
		add(moIntroduceSuperPeerPuerto);
		add(moBotonGuardar);
		
	}
	
	/**
	 * <p>Modifica el directorio de trabajo</p> 
	 * @author angel
	 *
	 */
	class PFC_ModificarAccion implements ActionListener
	{
		/**
		 * <p><Accion que se dispara con el boton Guardar</p>
		 */
		public void actionPerformed(ActionEvent e) 
		{
			//Se hacen las comprobaciones del directorio introducido
    		String lsDirectorio = moIntroduceDirectorio.getText();
			//Se hacen las comprobaciones de la IP del SuperPeer introducido
    		String lsSuperPeerIP = moIntroduceSuperPeerIP.getText();
			//Se hacen las comprobaciones del puerto del SuperPeer introducido
    		String lsSuperPeerPuerto = moIntroduceSuperPeerPuerto.getText();
    		
    		
    		if ( (lsDirectorio == null) || (lsDirectorio.equals("")) )
    		{
        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_DIRECTORIO_NO_VALIDO);
        		moLog.info("Cambio directorio - Falta introducir directorio");
        		return;
    		}

    		if (!PFC_Constantes.validarIP(lsSuperPeerIP))
    		{
        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_SUPERPEERIP_NO_VALIDO);
        		moLog.info("Cambio SuperPeer IP - Falta introducir IP");
        		return;
    		}
    		
    		
    		if (!PFC_Constantes.validarPuerto(lsSuperPeerPuerto))
    		{
        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_SUPERPEERPUERTO_NO_VALIDO);
        		moLog.info("Cambio SuperPeer Puerto - Falta introducir Puerto");
        		return;
    		}
    		
    		// Solo en el caso de haberse modificado el directorio se modifica
    		if (!PFC_Constantes.DIRECTORIO.equals(lsDirectorio))
    		{
	    		//Se verifica si existe y en caso contrario se crear
	    		try 
	    		{
					PFC_Constantes.verificaDirectorio(lsDirectorio);
				} catch (Exception e1) 
				{
					moLog.error("No se ha podido modificar fisicamente el directorio<"+lsDirectorio+">");
		    		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_DIRECTORIO_NO_CAMBIADO);
		    		return;
				}
				
	    		try
	    		{
	    			// Modificamos parametro que nos indica la ruta del directorio
	    			moGestionBD.modificarDirectorio(lsDirectorio);    		
				} catch (Exception e1) 
				{
					moLog.error("No se ha podido modificar la configuracion directorio<"+lsDirectorio+">");
		    		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_DIRECTORIO_NO_CAMBIADO);
		    		return;
				}
	
	    		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_DIRECTORIO_OK_CAMBIADO);
				
	    		// Se actualiza variable de trabajo con el nuevo directorio
				PFC_Constantes.DIRECTORIO = lsDirectorio;
    		}//if (!PFC_Constantes.DIRECTORIO.equals(lsDirectorio))

    		// Solo en el caso de haberse modificado la IP del SuperPeer se modifica
    		if (!PFC_Constantes.SUPERPEER_IP.equals(lsSuperPeerIP))
    		{
	    		try
	    		{
	    			// Modificamos parametro que nos indica la IP del SuperPeer
	    			moGestionBD.modificarSuperPeerIP(lsSuperPeerIP);
	    			
	    			// Se marca que ha habido cambio para que cuando se vuelve arrancar la red, coja estos cambios
	    			moGestionBD.modificarCambio("S");
	    			PFC_Constantes.CAMBIO_CONFIGURACION = true;
	    			
				} catch (Exception e1) 
				{
					moLog.error("No se ha podido modificar la configuracion IP del SuperPeer<"+lsSuperPeerIP+">");
		    		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_SUPERPEERIP_NO_CAMBIADO);
		    		return;
				}
	
	    		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_SUPERPEERIP_OK_CAMBIADO);
				
	    		// Se actualiza variable de trabajo con la nueva IP del SuperPeer
				PFC_Constantes.SUPERPEER_IP = lsSuperPeerIP;
    		}//if (!PFC_Constantes.SUPERPEER_IP.equals(lsSuperPeerIP))
    		
    		// Solo en el caso de haberse modificado el Puerto del SuperPeer se modifica
    		if (!PFC_Constantes.SUPERPEER_PUERTO.equals(lsSuperPeerPuerto))
    		{
	    		try
	    		{
	    			// Modificamos parametro que nos indica el Puerto del SuperPeer
	    			moGestionBD.modificarSuperPeerPuerto(lsSuperPeerPuerto);

	    			// Se marca que ha habido cambio para que cuando se vuelve arrancar la red, coja estos cambios
	    			moGestionBD.modificarCambio("S");
	    			PFC_Constantes.CAMBIO_CONFIGURACION = true;

				} catch (Exception e1) 
				{
					moLog.error("No se ha podido modificar la configuracion Puerto del SuperPeer<"+lsSuperPeerPuerto+">");
		    		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_SUPERPEERPUERTO_NO_CAMBIADO);
		    		return;
				}
	
	    		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_SUPERPEERPUERTO_OK_CAMBIADO);
				
	    		// Se actualiza variable de trabajo con el nuevo Puerto del SuperPeer
				PFC_Constantes.SUPERPEER_PUERTO = lsSuperPeerPuerto;
    		}//if (!PFC_Constantes.SUPERPEER_PUERTO.equals(lsSuperPeerPuerto))		
		}
	}//class PFC_ModificarAccion
	
}
