package pfc.prototipo;

import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.sql.Date;
import java.util.Vector;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

import org.apache.log4j.Logger;

import pfc.prototipo.bbdd.PFC_BDGestion;
import pfc.prototipo.jxta.PFC_Mensaje;
import pfc.prototipo.jxta.peer.PFC_Peer;
import pfc.prototipo.rendimiento.PFC_Rendimiento;
import pfc.prototipo.xml.PFC_GestorXML;

/**
 * <p>Panel encargado de gestionar todo lo relacionado con los documentos.</p>
 * @author Angel
 *
 */
public class PFC_PanelListarFicheros extends JPanel 
{
	//Trazas
	private final static Logger moLog = Logger.getLogger(PFC_PanelListarFicheros.class);

	private TextArea moContenidoFichero = null;	
	private JTable moListaDocumento = null;
	private JLabel moEtiquetaLista = null;    
	private JLabel moEtiquetaEdicion = null;
	private JScrollPane moScrollLista = null; 
	private JTextField moIntroduceNombre = null;
    private JButton moBotonGuardar = null;
    private JButton moBotonEliminar = null;    
    
    //Conexion con la base de datos
    private PFC_BDGestion moGestionBD = null;
	
    //Indica si estamos en modo Nuevo documento
    private boolean mbModoNuevo = false;
    
    //Indica las columnas de la tabla
    private Vector moColumnas = null;

    //Contiene los documentos obtenidos de la base de datos
    private PFC_Documentos moDocumentos = null;

    // Gestion de la conexion al grupo de Peer
    private PFC_Peer moPeer = null; 

	// Objeto para calcular tiempos (rendimiento)
	private PFC_Rendimiento moRendimiento = null;
    
	public PFC_PanelListarFicheros(PFC_BDGestion poGestion, boolean pbModoNuevo, PFC_Peer poPeer)
	{	
		super(null);
		
		moGestionBD = poGestion;
		mbModoNuevo = pbModoNuevo;
		moPeer = poPeer;
		
		//Se aaden elementos necesarios en el panel
		moEtiquetaEdicion = new JLabel(PFC_Constantes.EDICION_LABEL);
		moIntroduceNombre = new JTextField();
		moContenidoFichero = new TextArea();		
		moEtiquetaLista = new JLabel(PFC_Constantes.LISTA_DOCUMENTOS);
		moBotonGuardar = new JButton(PFC_Constantes.BOTON_GUARDAR);		
		moBotonEliminar = new JButton(PFC_Constantes.BOTON_ELIMINAR);
		
		
		//Inicializacion de los nombres de columnas
		moColumnas = new Vector();
		moColumnas.add(PFC_Constantes.LISTA_COLUMNA_DOCUMENTO);
		moColumnas.add(PFC_Constantes.LISTA_COLUMNA_FECHA);
		moColumnas.add(PFC_Constantes.LISTA_COLUMNA_PROPIETARIO);
		
		//Se crea la tabla con los documentos existentes
		moListaDocumento = new JTable(obtenerListadoDocumentos(), moColumnas);
		establecerAnchoColumna();
		
		moListaDocumento.addMouseListener(new PFC_SeleccionAccion()); 
		moScrollLista = new JScrollPane(moListaDocumento);
		
		//Se aade accion del boton Guardar
		moBotonGuardar.addActionListener(new PFC_GuardarAccion());

		//Se aade accion del boton Eliminar
		moBotonEliminar.addActionListener(new PFC_EliminarAccion());
		
		
		//Se ajustan posiciones de los elementos
		moContenidoFichero.setBounds(4, 50, 550, 600);
		moEtiquetaLista.setBounds( 582, 30, 150, 20 );
		moScrollLista.setBounds(580, 50, 300, 500);		
		moBotonGuardar.setBounds(370, 660, 80, 25);
		moBotonEliminar.setBounds(470, 660, 80, 25);
		moEtiquetaEdicion.setBounds( 4, 660, 50, 20 );
		moIntroduceNombre.setBounds( 60, 660, 200, 20 );

		//En caso estar en modo nuevo, el boton eliminar se desactiva
		if (mbModoNuevo)
		{
			moLog.debug("Modo Nuevo - Se desactiva boton Eliminar");
			moBotonEliminar.setEnabled(false);
		}
		else
		{
			//Al entrar en modo Consulta, se desactivan los botones y la introduccion del nombre
			moIntroduceNombre.setEnabled(false);
			moBotonGuardar.setEnabled(false);
			moBotonEliminar.setEnabled(false);
		}
		
		//Se aaden elementos al panel		
		add(moEtiquetaEdicion);		
		add(moIntroduceNombre);
		add(moContenidoFichero);
		add(moEtiquetaLista);
		add(moScrollLista);				
		add(moBotonGuardar);
		add(moBotonEliminar);				

		// Se inicializa medidor del rendimiento
		moRendimiento = new PFC_Rendimiento();		
	}//PFC_PanelListarFicheros

	/**
	 * <p>Vuelca el contenido de un documento en la area de edicion</p>
	 * @param poContenido StringBuffer con el contenido
	 */
	public void pintaContenidoFichero(StringBuffer poContenido)
	{
		moContenidoFichero.setText(poContenido.toString());
	}//pintaContenidoFichero

	/**
	 * <p>Vuelca el contenido de un documento en la area de edicion</p>
	 * @param poContenido String con el contenido
	 */
	public void pintaContenidoFichero(String psContenido)
	{
		moContenidoFichero.setText(psContenido);
	}//pintaContenidoFichero
	
	/**
	 * <p>Obtiene los documentos almacenados en la base de datos</p>
	 * @return Vector de vectores con los documentos (unicamente las columnas a mostrar en la lista)
	 */
	private Vector obtenerListadoDocumentos()
	{
		moDocumentos = moGestionBD.obtenerDocumentos();

		return moDocumentos.getDatosLista();
	}//obtenerListadoDocumentos
	    
	public void establecerAnchoColumna()
	{
		//Se fija tamao de la columna Propietario
		moListaDocumento.getColumn(PFC_Constantes.LISTA_COLUMNA_PROPIETARIO).setMinWidth(15);
		moListaDocumento.getColumn(PFC_Constantes.LISTA_COLUMNA_PROPIETARIO).setMaxWidth(15);
		moListaDocumento.getColumn(PFC_Constantes.LISTA_COLUMNA_PROPIETARIO).setWidth(15);
		moListaDocumento.getColumn(PFC_Constantes.LISTA_COLUMNA_PROPIETARIO).setPreferredWidth(15);		
	}//establecerAnchoColumna
	
	/**
	 * <p>Accion del boton Guardar</p>
	 * @author angel
	 *
	 */
	class PFC_GuardarAccion implements ActionListener
	{
		/**
		 * <p>Accion que se dispara al pulsar el boton Guardar.</p>
		 */
		public void actionPerformed(ActionEvent e) 
		{
        	moLog.debug("Se inicia proceso de Guardado en Modo nuevo<"+mbModoNuevo+">");
        	Date loFecha = null;
        	
        	if (mbModoNuevo)
        	{        		
        		//Se inicializa tiempos de la operacion
        		moRendimiento.ini("Alta Documento");
        		//Al estar en modo nuevo se tiene que verificar que el usuario haya introducido el nombre del fichero
        		String lsNomDocumento = moIntroduceNombre.getText();
        		String lsNomCompleto = null;
        		
        		if ( (lsNomDocumento == null) || (lsNomDocumento.equals("")) )
        		{
	        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_NOMBRE_DOCUMENTO);
	        		moLog.info("Nuevo documento - Falta introducir nombre");
	        		return;
        		}
        		
        		// Se verifica que el nombre del documento no exceda de lo permitido
        		if (lsNomDocumento.length() > PFC_Constantes.DOCUMENTOS_LON_NOMBRE)
        		{
	        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_ERROR_LON_NOM_DOCUMENTO);
	        		moLog.info("Nuevo documento - Nombre muy grande");
	        		return;        			
        		}
        		
        		//Se verifica que la extension del fichero sea XML
        		if (lsNomDocumento.length() < 5)
        		{
	        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_NOMBRE_ERRONEO);
	        		moLog.info("Nuevo documento - Nombre muy pequeo");
	        		return;        			        			
        		}
        		String lsExtension = lsNomDocumento.substring((lsNomDocumento.length()-4));
        		if (!lsExtension.equalsIgnoreCase(PFC_Constantes.DOCUMENTO_EXTENSION))
        		{
	        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_NOMBRE_ERRONEO);
	        		moLog.info("Nuevo documento - Extensin erronea.");
	        		return;        			        			        			
        		}
        		
        		//Se verifica que hayan introducido el contenido del documento
        		String lsContenido = moContenidoFichero.getText();
        	
        		if ( (lsContenido == null) || (lsContenido.equals("")) )
        		{
	        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_CONTENIDO_DOCUMENTO);
	        		moLog.info("Nuevo documento - Documento sin contenido.");
	        		return;
        		}

        		// Se verifica que no existe documento con el mismo nombre en base de datos y si es asi, se guarda  
        		String lsIdDoc = PFC_Constantes.componeIdDoc(PFC_Constantes.NODO_ID, lsNomDocumento);
        		try
        		{
        			loFecha = new Date(System.currentTimeMillis());
	        		moGestionBD.guardarDocumento(lsIdDoc, PFC_Constantes.NODO_ID, lsNomDocumento, PFC_Constantes.DIRECTORIO, loFecha);
        		
				} catch (Exception e1) 
				{
					moLog.error("No se ha podido de dar de alta el documento en bbdd<"+PFC_Constantes.DIRECTORIO+lsNomDocumento+">");
	        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_ERROR_CREAR_DOCUMENTO);
	        		return;
				}
        		
        		//Se guarda documento en disco
        		try 
        		{
        			//El nombre del documento al guardarlo en disco se le concatena el identificador unico del nodo seguido _ y del nombre        			
        			lsNomCompleto = PFC_Constantes.componeNomDocFisico(PFC_Constantes.DIRECTORIO, PFC_Constantes.NODO_ID, lsNomDocumento);
        			PFC_GestorXML.crearXML(moContenidoFichero.getText(), lsNomCompleto);
				} catch (Exception e1) 
				{
					moLog.error("No se ha podido de dar de alta el documento fisicamente<"+PFC_Constantes.DIRECTORIO+lsNomCompleto+">");
					
					// Anteriormente se habia dado de alta el registro en base de datos. En este punto hay un registro de documento en base
					// de datos pero no se ha dado de alta el documento....se procede a la baja logica
					try 
					{
						moGestionBD.eliminarDocumento(lsIdDoc);
					} catch (Exception e2) 
					{
						moLog.error("Problemas al querer borrar de base de datos un documento.", e2);
					}
					
	        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_ERROR_CREAR_DOCUMENTO_FORMATO);
	        		return;
				}
        		
        						
				// Se limpia pantalla y se refresca listado
				moIntroduceNombre.setText("");
				moContenidoFichero.setText("");
				DefaultTableModel loModelo = (DefaultTableModel)moListaDocumento.getModel();								
				loModelo.setDataVector(obtenerListadoDocumentos(), moColumnas);
				establecerAnchoColumna();
				
				// Si se esta conectado a la red, se envia documento al resto del grupo
				if (PFC_Constantes.ESTOY_CONECTADO)
				{
	        		try 
	        		{					
						PFC_Mensaje loMensaje = new PFC_Mensaje(PFC_Constantes.NODO_ID, PFC_Mensaje.TIPO_NUEVO);
						// Se establece el nombre del document
						loMensaje.setNombreDocumento(lsNomDocumento);
						// Se establece el identificador del document
						loMensaje.setIdDocumento(PFC_Constantes.componeIdDoc(PFC_Constantes.NODO_ID, lsNomDocumento));		
						//Se establece la fecha del documento
						loMensaje.setFechaDocumento(loFecha);						
						//Se establece propietario
						loMensaje.setPropietario(PFC_Constantes.NODO_ID);
						// Se establece el contenido del documento
						loMensaje.setContenido(PFC_ManejoFicheros.getFicheroByteArray(lsNomCompleto));  
						
						// Se informa al Grupo de la alta del documento
						moPeer.enviarMensajeGrupo(loMensaje);
												
					} catch (Exception e1) 
					{
						moLog.error("No se ha podido informar al grupo del alta del documento<"+PFC_Constantes.DIRECTORIO+lsNomCompleto+">");
						
						// Al tratarse de un error grave que puede dejar el sistema inconsistente, se guarda el Alta en Notificaciones
						// y se desconecta aplicacion del grupo
						try 
						{
							moGestionBD.altaNotificacion(PFC_Constantes.componeIdDoc(PFC_Constantes.NODO_ID, lsNomDocumento), 
									PFC_Constantes.NOTIFICACION_ALTA, loFecha, null, null, null, null);
						} catch (Exception e2) 
						{
							moLog.error("Problemas al dar de alta una notificacion.", e2);
						}

						
		        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_ERROR_INFORMAR_GRUPO_ALTA);
		        				        		
		        		return;
		        		
					}
				}
				else
				{
	        		try 
	        		{					
						moLog.info("Alta de un nuevo documento sin estar conectado al grupo. Se crea notificacion");
						// En caso de no estar conectado, se tiene que dar de alta un registro en notificaciones para que cuando se
						// conecte sepa que tiene que informar que hay un documento nuevo
						moGestionBD.altaNotificacion(PFC_Constantes.componeIdDoc(PFC_Constantes.NODO_ID, lsNomDocumento), 
								PFC_Constantes.NOTIFICACION_ALTA, loFecha, null, null, null, null);
					} catch (Exception e1) 
					{
						moLog.error("No se ha podido dar de alta la notificacion para el documento<"+PFC_Constantes.DIRECTORIO+lsNomCompleto+">");
		        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_ERROR_ALTA_NOTIFICACION); 
		        		return;
					}
				}
				
				//Calculo del tiempo utilizado en dar de alta un documento.
				moRendimiento.tiempo();
        	}//if (mbModoNuevo)
        	
        	// Si esta en modo Modificar, lo que se tiene que hacer es guardar los cambios que haya podido realizar en el documento. Solo se puede modificar el contenido
        	if (!mbModoNuevo)
        	{
        		//Se inicializa tiempos de la operacion
        		moRendimiento.ini("Modificar Documento");        		
        		//Nombre del documento
        		String lsNomDocumento = moIntroduceNombre.getText();
        		//Se verifica que hayan introducido el contenido del documento
        		String lsContenido = moContenidoFichero.getText();
        		String lsNomCompleto = null;
        		String lsIdDocumento = null;
        		
        		if ( (lsContenido == null) || (lsContenido.equals("")) )
        		{
	        		moLog.info("Modificar documento - Documento sin contenido.");
	        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_CONTENIDO_DOCUMENTO);
	        		return;
        		}

        		//El nombre del documento al guardarlo en disco se le concatena el identificador unico del nodo seguido _ y del nombre        			
        		lsNomCompleto = PFC_Constantes.componeNomDocFisico(PFC_Constantes.DIRECTORIO, PFC_Constantes.NODO_ID, lsNomDocumento);
        		// Identificador del documento 
        		lsIdDocumento = PFC_Constantes.componeIdDoc(PFC_Constantes.NODO_ID, lsNomDocumento); 
        		
        		// Se buscan los cambios que han habido en el documento
        		Vector loMods = PFC_GestorXML.detectarDiferencias(lsNomCompleto, lsContenido);        
        		
        		// Si lo devuelto es null, significa que se ha querido cambiar el nodo raiz cuando no se permite.
        		if (loMods == null)
        		{
					moLog.info("No se permite modificar el nombre del nodo raiz.");
	        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_NO_MODIFICAR_NOMBRE_RAIZ);
	        		return;        			
        		}
        		
        		        		        	
        		// Una vez actualizado registro en base de datos, se guarda el contenido en disco
        		try 
        		{
					PFC_GestorXML.crearXML(moContenidoFichero.getText(), lsNomCompleto);
				} catch (Exception e1) 
				{
					moLog.error("No se ha podido modificar el documento<"+PFC_Constantes.DIRECTORIO+lsNomCompleto+">");
	        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_ERROR_MODIFICAR_DOCUMENTO);
	        		return;
				}
        		
       
        		// Se actualiza registro de la base de datos para indicar la nueva fecha de actualizacion
        		try
        		{
        			loFecha = new Date(System.currentTimeMillis());
        			moGestionBD.modificarDocumento(lsIdDocumento,PFC_Constantes.NODO_ID, loFecha);
        		
				} catch (Exception e1) 
				{
					moLog.error("No se ha podido de dar de alta el documento en bbdd<"+PFC_Constantes.DIRECTORIO+lsNomDocumento+">");
	        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_ERROR_CREAR_DOCUMENTO);
	        		return;
				}
        		
				// Se informa al usuario que el documento se ha modificado de forma correcta
        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_OK_MODIFICAR_DOCUMENTO);
				DefaultTableModel loModelo = (DefaultTableModel)moListaDocumento.getModel();								
				loModelo.setDataVector(obtenerListadoDocumentos(), moColumnas);
				establecerAnchoColumna();
				
				// Si se esta conectado a la red, se envian los cambios
				if (PFC_Constantes.ESTOY_CONECTADO)
				{
	        		try 
	        		{					
						PFC_Mensaje loMensaje = new PFC_Mensaje(PFC_Constantes.NODO_ID, PFC_Mensaje.TIPO_CAMBIO);
						// Se establece el nombre del document
						loMensaje.setNombreDocumento(lsNomDocumento);
						// Se establece el identificador del document
						loMensaje.setIdDocumento(lsIdDocumento);		
						//Se establece la fecha del documento
						loMensaje.setFechaDocumento(loFecha);						
						// Se establecen los cambios realizados
						loMensaje.setCambio(loMods); 
						
						// Se informa al Grupo de la alta del documento
						moPeer.enviarMensajeGrupo(loMensaje);
												
					} catch (Exception e1) 
					{
						moLog.error("No se ha podido informar al grupo de las modificaciones del documento<"+PFC_Constantes.DIRECTORIO+lsNomDocumento+">");
		        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_ERROR_INFORMAR_GRUPO_MODIFICAR);
		        		return;
					}
				}
				else
				{
	        		try 
	        		{					
						moLog.info("Modificaciones de un documento sin estar conectado al grupo. Se crea notificacion");
						
						// Se verifica si existe un cambio pendiente enviar que indique el alta del documento. En tal caso, estas modificaciones no se
						// guardarn porque se entiende que al enviar el alta se enva la ultima version del documento
						boolean lbExisteAlta = moGestionBD.existeAltaPendiente(lsIdDocumento);
						if (!lbExisteAlta)
						{
							// En caso de no estar conectado, se tienen que dar de alta tantos registros de notificaciones como 
							// cambios haya habido para que cuando se conecte sepa que tiene que informar de los cambios
			        		for(int i = 0; i < loMods.size(); i++)
			        		{        			
			        			String[] lvsDatos = (String[])loMods.get(i);
			        		
			        			// Se crea registro en base de datos con los cambios
								moGestionBD.altaNotificacion(lsIdDocumento,	PFC_Constantes.NOTIFICACION_MODIFICACION, loFecha, lvsDatos[0],
										lvsDatos[1], lvsDatos[2], lvsDatos[3]);
			        		}//while
						}
						else
						{
							moLog.debug("No se guardan en pendiente envios los cambios realizados del documento<"+lsIdDocumento+">");
						}
						
						
					} catch (Exception e1) 
					{
						moLog.error("No se ha podido dar de alta las notificaciones de cambios para el documento<"+PFC_Constantes.DIRECTORIO+lsNomCompleto+">");
		        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_ERROR_ALTA_NOTIFICACION);
		        		return;
					}
				}        		

				//Calculo del tiempo utilizado en dar de alta un documento.
				moRendimiento.tiempo();
        	}//if (!mbModoNuevo)
				
		
		}
	}//PFC_GuardarAccion

	class PFC_SeleccionAccion implements MouseListener
	{

		/**
		 * <p>Al pulsar sobre un registro del listado, se carga ese registro en el editor de texto.</p>
		 * <p>Tambin se verifica si el propietario es el nodo que esta trabajando para activar los botones de Guardar y Eliminar.
		 * En caso contrario los botones se desactivan.</p>
		 */
		public void mouseClicked(MouseEvent e) 
		{
	    	  int liRegistro = moListaDocumento.rowAtPoint(e.getPoint());
	    	  if (liRegistro > -1)
	    	  {
	    		  String lsNombreFisicoDocumento = moDocumentos.getNombreFisicoDocumento(liRegistro);
	    		  moLog.debug("Volcamos contenido del fichero en el editor. Nombre fisico del fichero<"+lsNombreFisicoDocumento+">");

	    		  try 
	    		  {
	    			  pintaContenidoFichero(PFC_GestorXML.cargarXML(lsNombreFisicoDocumento));	    			  
	    		  } catch (Exception e1) 
	    		  {
					moLog.error("No se ha podido cargar el documento <"+lsNombreFisicoDocumento+">");					
	        		JOptionPane.showMessageDialog((JTable)e.getSource(), PFC_Constantes.AVISO_ERROR_CARGA_FICHERO);
	        		return;	    			  
	    		  }
	    		  
	    		  //Pintamos el nombre del documento
	    		  moIntroduceNombre.setText(moDocumentos.getNombreDocumento(liRegistro));
	    		  moIntroduceNombre.setEditable(false);
	    		  
	    		  //En el caso de ser propietario del documento, habilitamos botones de Guardar y Eliminar
	    		  if (moDocumentos.getSiPropietario(liRegistro).equalsIgnoreCase("S"))
	    		  {
	    			  moLog.debug("Nodo propietario del documento. Se habilitan opciones de edicion");
	    			  moBotonGuardar.setEnabled(true);
	    			  moBotonEliminar.setEnabled(true);	    			
	    		  }
	    		  else
	    		  {
	    			  moLog.debug("Nodo NO propietario del documento. Se deshabilitan opciones de edicion");
	    			  moBotonGuardar.setEnabled(false);
	    			  moBotonEliminar.setEnabled(false);	    			  
	    		  }
	    		  
	    		  //Se establece que trabajamos en modo edicin
	    		  mbModoNuevo = false;
	    	  }
	    	  	    	  
		}//mouseClicked

		public void mouseEntered(MouseEvent arg0) {}
		public void mouseExited(MouseEvent arg0) {}
		public void mousePressed(MouseEvent arg0) {}
		public void mouseReleased(MouseEvent arg0) {}
		
	}//class PFC_SeleccionAccion

	/**
	 * <p>Accion del boton eliminar.</p>
	 * <p>da de baja un documento tanto en base de datos como en la unidad de disco fisica</p>
	 * @author Angel
	 *
	 */
	class PFC_EliminarAccion implements ActionListener
	{

		public void actionPerformed(ActionEvent e) 
		{
    		//Nombre del documento
    		String lsNomDocumento = moIntroduceNombre.getText();
    		String lsNomCompletoDoc = "";
    		String lsIdDocumento = null;
    		
    		int liConfirmado = JOptionPane.showConfirmDialog( (JButton)e.getSource(), PFC_Constantes.AVISO_CONFIRMAR_BORRAR_DOCUMENTO);
    		
			if (JOptionPane.OK_OPTION == liConfirmado)
			{
				moRendimiento.ini("Eliminar documento");		
	    		moLog.info("Confirmacion para borrar documento<"+lsNomDocumento+">");
	    		lsIdDocumento = PFC_Constantes.componeIdDoc(PFC_Constantes.NODO_ID, lsNomDocumento);
				// Se actualiza el registro en base de datos indicando que se ha dado de baja un documenot
	    		try
	    		{
	        		moGestionBD.eliminarDocumento(lsIdDocumento);
	    		
				} catch (Exception e1) 
				{
					moLog.error("No se ha podido eliminar el documento en bbdd<"+PFC_Constantes.DIRECTORIO+lsNomDocumento+">");
	        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_ERROR_ELIMINAR_DOCUMENTO);
	        		return;
				}
				
	    		// Una vez actualizado registro en base de datos, se guarda el contenido en disco
	    		try 
	    		{
	    			//El nombre del documento al guardarlo en disco se le concatena el identificador unico del nodo seguido _ y del nombre        			
	    			lsNomCompletoDoc = PFC_Constantes.componeNomDocFisico(PFC_Constantes.DIRECTORIO, PFC_Constantes.NODO_ID, lsNomDocumento);
					PFC_ManejoFicheros.borrarFichero(lsNomCompletoDoc);
				} catch (Exception e1) 
				{
					moLog.error("No se ha podido eliminar el documento<"+PFC_Constantes.DIRECTORIO+lsNomCompletoDoc+">");
	        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_ERROR_ELIMINAR_DOCUMENTO);
	        		return;
				}
    		
				// Se limpia pantalla y se refresca listado
				moIntroduceNombre.setText("");
				moContenidoFichero.setText("");
				moBotonGuardar.setEnabled(false);
   			  	moBotonEliminar.setEnabled(false);	    			
				DefaultTableModel loModelo = (DefaultTableModel)moListaDocumento.getModel();								
				loModelo.setDataVector(obtenerListadoDocumentos(), moColumnas);
				establecerAnchoColumna();
							
				// Si se esta conectado a la red, se informa de la baja
				if (PFC_Constantes.ESTOY_CONECTADO)
				{
	        		try 
	        		{					
						PFC_Mensaje loMensaje = new PFC_Mensaje(PFC_Constantes.NODO_ID, PFC_Mensaje.TIPO_BAJA);
						// Se establece el nombre del document
						loMensaje.setNombreDocumento(lsNomDocumento);
						// Se establece el identificador del document
						loMensaje.setIdDocumento(lsIdDocumento);						
						
						// Se informa al Grupo de la alta del documento
						moPeer.enviarMensajeGrupo(loMensaje);
												
					} catch (Exception e1) 
					{
						moLog.error("No se ha podido informar al grupo de la baja del documento<"+PFC_Constantes.DIRECTORIO+lsNomDocumento+">");
		        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_ERROR_INFORMAR_GRUPO_BAJA);
		        		return;
					}
					
				}
				else
				{
	        		try 
	        		{					
						moLog.info("Baja de un documento sin estar conectado al grupo. Se crea notificacion");
						
						// Se verifica si previo a esta baja existe una alta que no se ha enviado al grupo, en tal caso no se guarda el cambio de 
						// baja y se elimina el registro de alta pendiente de enviar
						boolean lbExisteAlta = moGestionBD.existeAltaPendiente(lsIdDocumento);
						if (!lbExisteAlta)
						{
							// En caso de no estar conectado, se tiene que dar de alta un registro en notificaciones para que cuando se
							// conecte sepa que tiene que informar que se ha dado de baja un documento
							moGestionBD.altaNotificacion(PFC_Constantes.componeIdDoc(PFC_Constantes.NODO_ID, lsNomDocumento), 
									PFC_Constantes.NOTIFICACION_BAJA, new Date(System.currentTimeMillis()), null, null, null, null);
						}
						else
						{
							moLog.debug("No se guardan en pendiente envios los cambios realizados del documento<"+lsIdDocumento+">");
							moGestionBD.eliminarCambios(lsIdDocumento);
						}
	
					} catch (Exception e1) 
					{
						moLog.error("No se ha podido dar de alta la notificacion para el documento<"+PFC_Constantes.DIRECTORIO+lsNomDocumento+">");
		        		JOptionPane.showMessageDialog(((JPanel)((JButton)e.getSource()).getParent()), PFC_Constantes.AVISO_ERROR_ALTA_NOTIFICACION);
		        		return;
					}					
				}

				//Calculo del tiempo utilizado en dar de alta un documento.
				moRendimiento.tiempo();				
			}//if (JOptionPane.OK_OPTION == liConfirmado)
			else
			{
				moLog.info("No se ha confirmacion el borrado del documento<"+lsNomDocumento+">");
			}
		}//actionPerformed

	}//class PFC_EliminarAccion
}

