package pfc.prototipo;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.*;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import pfc.prototipo.bbdd.PFC_BDGestion;
import pfc.prototipo.jxta.peer.PFC_Peer;


/**
 * <p>Panel principal punto de partida de un Nodo tipo Peer.</p>
 * <p>Genera la pantalla inicial donde se define marco de trabajo.</p>
 * 
 * @author Angel
 *
 */
public class PFC_PanelPrincipal extends JFrame 
{
	//Trazas
	private final static Logger moLog = Logger.getLogger(PFC_PanelPrincipal.class);
	
	
	//Gestion de la base de datos
	private PFC_BDGestion moGestionBD = null;
	
	//Ventana principal de la aplicacion
	//JFrame moMarco = null; 

	//Definicin de paneles
	private	PFC_PanelConectarConexion	moPanelGestion;	
	private	PFC_PanelListarFicheros	moPanelListar;
	private	PFC_PanelListarFicheros	moPanelNuevo;
	private	PFC_PanelConfigurar	moPanelConfigurar;
	private JPanel loPanelEstado; 
	//Indica que opcion de menu tenemos seleccionado.
	private String msOpcionMenu = "";
	
    // Gestion de la conexion al grupo de Peer
    private PFC_Peer moPeer = null; 

	
	/**
	 * <p>Inicializa la aplicacion creando/cargando configuracion de la misma.</p>
	 */
	public PFC_PanelPrincipal()
	{
		super(PFC_Constantes.NOMBRE_APLICACION + " : " + PFC_Constantes.NODO_NOMBRE);
		
		//Indica si existe configuracion
		boolean lbExisteConfig = false;
		
		try
		{
			//Se crea conexion con base de datos.
			moGestionBD = new PFC_BDGestion();
			
			//Se verifica si existe una configuracion creada. En caso de no existir, se genera de nuevo.
			//Si no existe es porque la aplicacion se utiliza por primera vez y es cuando se tiene que generar los identificadores unicos
			//del peer y de los canales de comunicacion necesarios
			lbExisteConfig = moGestionBD.existeTabla(PFC_Constantes.TABLA_CONFIGURACION);
			
			if (lbExisteConfig)
			{
				moLog.info("Se carga configuracion.");
				PFC_Constantes.cargarConfiguracion(moGestionBD);
			}
			else
			{
				moLog.info("Aplicacion en proceso de inicializacion. Se carga configuracion por primera vez");
				//La primera vez que se inicia, se establece que el directorio es el path actual mas el establecido por defecto
				PFC_Constantes.DIRECTORIO = new File(".").getCanonicalPath() + PFC_Constantes.DIRECTORIO;
				moLog.info("Se establece el directorio de trabajo inicial<"+PFC_Constantes.DIRECTORIO+">");
				PFC_Constantes.crearConfiguracion(moGestionBD, false);
			}
			
		}catch(Exception e)
		{
			moLog.error("No se ha podido iniciar la solucion:"+e.getLocalizedMessage());
			System.out.println("No se ha podido iniciar la solucion");
			if (moGestionBD != null)
			{
				moGestionBD.cerrarConexion();
			}
			System.exit(-1);
		}
		
	}//PFC_PanelPrincipal
	
	
	
	
	/**
	 * <p>Se inician los siguiente puntos:</p>
	 * <p>	. Sistema de trazas</p>
	 * <p>	. Marco principal de la aplicacin</p>
	 * <p></p>
	 * <p>nicamente hay un parmetro de entrada que indica el nmero de nodo que se est generando. Es un numrico.</p>
	 * @param args
	 */
	public static void main(String[] args) 
	{
		int liNumeroNodo = -1;
		
		//Lo primero que se realiza es inicialiar el sistema de trazas. Se carga fichero de propiedades donde se indica toda la gestin del sistema de trazas
		try
		{
			//Se carga configuracion de trazas
			PropertyConfigurator.configure("log4j.properties");

		}catch(Exception e)
		{
			System.out.println("ERROR: Problemas al inicializar el sistema de trazas. Verifique su instalacin para poder utilizar la solucin.");
			System.exit(-1);
		}
				

		//Se verifica que se tiene el pametro de entrada que indica el numero de nodo a crear
		if ( (args == null) || (args.length == 0) )
		{
			moLog.error("Parmetro que indica nmero de nodo no existe!. Introducir nmero de nodo.");
			System.out.println("Parmetro que indica nmero de nodo no existe!. Introducir nmero de nodo.");
			System.exit(-1);
		}
		
		//Se verifica que el parametro introducido sea un numerico
		try
		{
			liNumeroNodo =  new Integer(args[0]).intValue();
		}catch(Exception e)
		{
			//No se trata de un numero. Parametro introducido pero formato erroneo
			moLog.error("Formato del parmetro errneo. Debe ser un numrico entero.");
			System.out.println("Formato del parmetro errneo. Debe ser un numrico entero.");
			System.exit(-1);
		}
		
		//Se construye el nombre del nodo. Se compone de una parte fija y comun a cualquier nodo mas el nmero de nodo pasado por parmetro
		PFC_Constantes.NODO_NOMBRE = PFC_Constantes.NODO_NOMBRE_PREFIJO + liNumeroNodo;	
		
		moLog.debug("Se inicia el proceso de creacion del Panel Principal para Nodo<"+PFC_Constantes.NODO_NOMBRE+">");
		
		//Se inicializa panel principal		
		final PFC_PanelPrincipal loVentana = new PFC_PanelPrincipal();
				
		java.awt.EventQueue.invokeLater(new Runnable()  
		{
			public void run()
			{ 
				loVentana.generarPantallaPrincipal(); 
			}
		});
	}//main

	/**
	 * <p>Genera el marco principal de la aplicacion.</p>
	 */
	private void generarPantallaPrincipal() 
	{
        //Se inicializan parmetros de la pantalla principal
		JFrame.setDefaultLookAndFeelDecorated(true);
		this.setName(PFC_Constantes.NOMBRE_APLICACION);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setSize(PFC_Constantes.PANTALLA_ANCHO, PFC_Constantes.PANTALLA_ALTO);

		//Generacion del menu
		generarMenu();

		// Incluir un JPanel abajo como barra de estado donde se indicara si esta o no conectado al grupo
		loPanelEstado = new JPanel();		
		loPanelEstado.setBorder(BorderFactory.createEtchedBorder());
		loPanelEstado.setName(PFC_Constantes.PANEL_ESTADO_NOMBRE);
		
		JLabel loTexto = new JLabel(PFC_Constantes.ESTOY_CONECTADO ? PFC_Constantes.ESTADO_CONECTADO : PFC_Constantes.ESTADO_DESCONECTADO);		
		loTexto.setName(PFC_Constantes.LABEL_ESTADO_NOMBRE);	
		loPanelEstado.add(loTexto);
		
		JProgressBar loBarra = new JProgressBar(0, 100);	
		loBarra.setName(PFC_Constantes.PROGRESO_BARRA);
		loBarra.setStringPainted(true);				
		loBarra.setVisible(false);		
		loPanelEstado.add(loBarra);
		
		//Se aade el menu a la pantalla principal	    
	    this.getContentPane().add(loPanelEstado, BorderLayout.SOUTH);
		
		// Pantalla visible
	    this.setVisible(true);		
		// Pantalla centrada
	    this.setLocationRelativeTo(null);  

		//Se aade un listener para capturar el evento de cierre de pantalla y poder cerrar la conexion
	    this.addWindowListener(new WindowAdapter() {   
			public void windowClosing(WindowEvent e)
			{     				
				moLog.info("Se cierra la aplicacion!");
				if (moGestionBD != null)
				{
					moGestionBD.cerrarConexion();
				}            

				// En caso de estar conectado a la red, para comunicaciones
				pararComunicaciones();
				
				System.exit(0);   
			} 
		});
	
	}//generarPantallaPrincipal

	/**
	 * <p>Crea el menu de la aplicacion.</p>
	 */
	private void generarMenu()
	{
		//Menu
		JMenuBar loMenubar = null;
		//Opcion de menu Conexion
		JMenu loOpConexion = null;
		//Opcion de menu Fichero
		JMenu loOpFichero = null;
		//Opcion de menu Sistema
		JMenu loOpSistema = null;
		
		//Opcion Conectar del menu Conexion
		JMenuItem loOpGestionar;
		//Opcion Listar del menu Fichero
		JMenuItem loOpListar;
		//Opcion Nuevo del menu Fichero
		JMenuItem loOpNuevo;
		//Opcion Configuracion
		JMenuItem loOpConfig;
		//Opcion Salir
		JMenuItem loOpSalir;
		
		//Se crea opcion Listar del menu Fichero
		loOpListar = new JMenuItem(PFC_Constantes.MENU_FICHEROS_LISTAR);
		loOpListar.setMnemonic('L');
		loOpListar.setAccelerator(KeyStroke.getKeyStroke("control L"));
		//Se crea opcion Nuevo del menu Fichero		
		loOpNuevo = new JMenuItem(PFC_Constantes.MENU_FICHEROS_NUEVO);
		loOpNuevo.setMnemonic('N');        	
		loOpNuevo.setAccelerator(KeyStroke.getKeyStroke("control N"));
		//Se crea opcion Conectar del menu Conexion
		loOpGestionar = new JMenuItem(PFC_Constantes.MENU_CONEXION_GESTION);
		loOpGestionar.setMnemonic('M');
		loOpGestionar.setAccelerator(KeyStroke.getKeyStroke("control m"));
		//Se crea opcion Configuracion del menu Sistema
		loOpConfig = new JMenuItem(PFC_Constantes.MENU_SISTEMA_CONFIGURAR);
		loOpConfig.setMnemonic('C');
		loOpConfig.setAccelerator(KeyStroke.getKeyStroke("control O"));
		//Se crea opcion Salir del menu Sistema
		loOpSalir = new JMenuItem(PFC_Constantes.MENU_SISTEMA_SALIR);
		loOpSalir.setMnemonic('E');
		loOpSalir.setAccelerator(KeyStroke.getKeyStroke("control e"));

        //Se crea y genera menu de la aplicacion
        loMenubar = new JMenuBar();
        //Opcion Conexion
        loOpConexion = new JMenu(PFC_Constantes.MENU_CONEXION);
        loOpConexion.setMnemonic('C');
        loMenubar.add(loOpConexion);      
        loOpConexion.add(loOpGestionar);                
        
        //Opcion Ficheros
        loOpFichero = new JMenu(PFC_Constantes.MENU_FICHEROS);
        loOpFichero.setMnemonic('F');
	    loMenubar.add(loOpFichero);
	    loOpFichero.add(loOpListar);	    
	    loOpFichero.add(loOpNuevo);

        //Opcion Sistema
	    loOpSistema = new JMenu(PFC_Constantes.MENU_SISTEMA);
	    loOpSistema.setMnemonic('S');
	    loMenubar.add(loOpSistema);
	    loOpSistema.add(loOpConfig);	    
	    loOpSistema.add(loOpSalir);

	    //Se aade los eventos de las opciones de menu
	    loOpGestionar.addActionListener(new gestionarAction());	    
	    loOpListar.addActionListener(new listarFicherosAction());
	    loOpNuevo.addActionListener(new nuevoFicheroAction());
	    loOpConfig.addActionListener(new configurarAction());
	    loOpSalir.addActionListener(new salirAction());
	    
	    //Se aade el menu a la pantalla principal	    
	    this.getContentPane().add(loMenubar, BorderLayout.NORTH);
	}//generarMenu
	
    /**
     * <p>Carga panel de conexion</p>
     * @author angel
     *
     */
    class gestionarAction implements ActionListener 
    {
        public void actionPerformed(ActionEvent e) 
        {
    		//Se crea panel Listar con los elementos necesarios
        	limpiarPanel();
        	moPanelGestion = null;
        	moPanelGestion = new PFC_PanelConectarConexion(moGestionBD);    		    		
    		PFC_Constantes.getPrincialMenu(e).getContentPane().add(moPanelGestion);
    		moPanelGestion.updateUI();    
    		msOpcionMenu = "CONEXIONES";
        }
    }//class conectarAction
    

    /**
     * <p>Carga panel con Listado de documento y edicion de ficheros</p>
     * @author angel
     *
     */
    class listarFicherosAction implements ActionListener 
    {
        public void actionPerformed(ActionEvent e) 
        {
    		//Se crea panel Listar con los elementos necesarios
        	limpiarPanel();
        	moPanelListar = null;
        	moPanelListar = new PFC_PanelListarFicheros(moGestionBD, false, moPeer);    		    		
    		PFC_Constantes.getPrincialMenu(e).getContentPane().add(moPanelListar);
    		moPanelListar.updateUI();    
    		msOpcionMenu = "LISTAR";
        }
    }//cargarFicheroAction

    /**
     * <p>Carga panel en modo nuevo documento.</p>
     * @author angel
     *
     */
    class nuevoFicheroAction implements ActionListener 
    {
        public void actionPerformed(ActionEvent e) 
        {
    		//Se crea panel Nuevo con los elementos necesarios        	
        	limpiarPanel();
        	moPanelNuevo = null;
        	moPanelNuevo = new PFC_PanelListarFicheros(moGestionBD, true, moPeer);        	
    		PFC_Constantes.getPrincialMenu(e).getContentPane().add(moPanelNuevo);
    		moPanelNuevo.updateUI();
    		msOpcionMenu = "NUEVO";
        }
    }//cargarFicheroAction
    
    /**
     * <p>Carga panel de configuracion de la aplicacion</p>
     * @author angel
     *
     */
    class configurarAction implements ActionListener 
    {
        public void actionPerformed(ActionEvent e) 
        {
    		//Se crea panel Nuevo con los elementos necesarios
        	limpiarPanel();
        	moPanelConfigurar = null;
        	moPanelConfigurar = new PFC_PanelConfigurar(moGestionBD);    		
    		PFC_Constantes.getPrincialMenu(e).getContentPane().add(moPanelConfigurar);
    		moPanelConfigurar.updateUI();
    		msOpcionMenu = "CONFIGURAR";
        }
    }//configurarAction

    /**
     * <p>Se sale de la aplicacion</p>
     * @author angel
     *
     */
    class salirAction implements ActionListener 
    {
        public void actionPerformed(ActionEvent e) 
        {
        	System.exit(-1);
        }
    }//salirAction

    /**
     * <p>Se encarga de quitar el Panel activado de la ventana</p>
     */
    private void limpiarPanel()
    {
    	if (msOpcionMenu.equalsIgnoreCase("NUEVO"))
    	{
    		moLog.debug("Limpiado de Panel - Se quita Panel NUEVO");
    		this.remove(moPanelNuevo);
    		this.repaint();
    	}
    	if (msOpcionMenu.equalsIgnoreCase("LISTAR"))
    	{
    		moLog.debug("Limpiado de Panel - Se quita Panel LISTAR");
    		this.remove(moPanelListar);
    		this.repaint();
    	}
    	if (msOpcionMenu.equalsIgnoreCase("CONEXIONES"))
    	{
    		moLog.debug("Limpiado de Panel - Se quita Panel CONEXIONES");
    		this.remove(moPanelGestion);
    	}
    	if (msOpcionMenu.equalsIgnoreCase("CONFIGURAR"))
    	{
    		moLog.debug("Limpiado de Panel - Se quita Panel CONFIGURAR");
    		this.remove(moPanelConfigurar);
    	}
    	
    }//limpiarPanel()
    

    public PFC_Peer getComunicaciones()
    {
    	return moPeer;
    }
    public void setComunicaciones(PFC_Peer poPeer)
    {
    	moPeer = poPeer;
    }

    public void pararComunicaciones()
    {
    	if (moPeer != null)
    	{
    		moPeer.pararServicio();
    		moPeer = null;
    	}
    }

	/**
	 * <p>Cambia el literal del estado de la aplicacion. Conectado/Desconectado al Grupo</p>
	 * @param psEstado Literal con el estado
	 */
	public void cambiarNotificacionEstado(String psEstado)
	{
		System.out.println("HOLAA");
		((JLabel)loPanelEstado.getComponent(0)).setText(psEstado);
	}//cambiarNotificacionEstado

}
