package pfc.prototipo.bbdd;


import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;

import pfc.prototipo.PFC_Constantes;
import pfc.prototipo.PFC_Documentos;


/**
 * <p>Gestiona la conexion y el manejo con la base de datos</p>
 * @author Angel
 *
 */
public class PFC_BDGestion 
{
	//Trazas
	private final static Logger moLog = Logger.getLogger(PFC_BDGestion.class);

	// Objeto conexion con la base de datos de la aplicacion
	private Connection moConexion = null;
    
	/**
	 * <p>Crea una conexion con la base de datos de la aplicacion</p> 
	 * @throws Exception 
	 */
    public PFC_BDGestion() throws Exception
    {
        try	        
        {        
        	//Se instancia driver de la base de datos
        	Class.forName(PFC_Constantes.BD_DRIVER); 
            moLog.debug("Driver<"+PFC_Constantes.BD_DRIVER+"> de base de datos instanciado!");
        	        	
        } catch(java.lang.ClassNotFoundException e)     
        {
        	moLog.error("Problemas al instanciar el Driver de Base de datos. Driver<"+PFC_Constantes.BD_DRIVER+">");
        	throw new Exception("Problemas al instanciar el Driver de Base de datos. Driver<"+PFC_Constantes.BD_DRIVER+">:"+e.getMessage());
        }

        try 
        {
            // Se crea conexion con la base de datos de la aplicacion
        	moLog.debug("Se inicia proceso para crear conexion con la base de datos de la aplicacin. URL de la base de datos<"+PFC_Constantes.BD_URL+">");
        	moConexion = DriverManager.getConnection(PFC_Constantes.BD_URL);		 
            moLog.debug("Conexion con la base de datos de la aplicacin realizado de forma correcta!");
        }  catch (Throwable e)  
        {   
        	moLog.error("Problemas al crear la conexion con base de datos. URL BBDD<"+PFC_Constantes.BD_URL+">");
        	throw new Exception("Problemas al crear la conexion con base de datos. URL BBDD<"+PFC_Constantes.BD_URL+">:"+e.getMessage());        	
        }
    }//PFC_BDGestion
    
    /**
     * <p>Genera la configuracion inicial del sistema cuando se accede por primera vez</p>
     * @param psNomPeer Nombre del peer
     * @param psIdPeer Identificador del peer
     * @param psNomGrupo Nombre del grupo
     * @param psIdGrupo Identificador del grupo
     * @param psDirectorio Directorio donde se guardaran los documentos
     * @throws Exception
     */
    public void inicializaSistema(String psNomPeer, String psIdPeer, String psNomGrupo, String psIdPipeMulticast, String psDirectorio,
    		String psIpSP, String psPuertoSP, String psIdPipeUnicast, boolean pbSoySuperPeer) throws Exception
    {
    	//Query para crear la tabla
    	String lsTabla = null;    	
    	Statement loCreate = null;
    	PreparedStatement loInsert = null;
    	String lsInsert = null;
    	
    	//Se crea la tabla de configuracion del sistema
    	try 
    	{		
    		loCreate = moConexion.createStatement();
			//Sentencia para crear la tabla de Configuracion
    		lsTabla = "CREATE TABLE CONFIGURACION  "
            	+  "(CONFIG_PARAMETRO VARCHAR(20) NOT NULL" 
                +  "   CONSTRAINT PARAMETRO_PK PRIMARY KEY, " 
                +  " CONFIG_VALOR VARCHAR(300) NOT NULL) " ;
    		
            //Se crea la tabla
            loCreate.execute(lsTabla);
            
			moLog.debug("Tabla CONFIGURACION creada!");
            
		} catch (SQLException e) 
		{
			moLog.error("Problemas al crear la tabla de Configuracion:"+e.getMessage());
			try 
			{
				loCreate.close();
				loCreate = null;
			} catch (SQLException e1) 
			{
				moLog.error("Problemas al cerrar el Statement:"+e1.getMessage());			
			}					
			throw new Exception("No se he podido generar la configuracion!");
		}
		
		try
		{
			//Una vez creada la tabla, se insertan los valores de configuracion
			lsInsert = "INSERT INTO CONFIGURACION (CONFIG_PARAMETRO, CONFIG_VALOR) VALUES (?, ?)";
			
			loInsert = moConexion.prepareStatement(lsInsert);
			
			//Se inserta nombre del peer
			loInsert.setString(1, PFC_Constantes.CONFIGURACION_NOMBRE_PEER);
			loInsert.setString(2, psNomPeer);
			moLog.debug("Parametro<"+PFC_Constantes.CONFIGURACION_NOMBRE_PEER+"> creado <"+psNomPeer+">");
			loInsert.executeUpdate();
			loInsert.clearParameters();
			
			//Se inserta identificador del peer
			loInsert.setString(1, PFC_Constantes.CONFIGURACION_ID_PEER);
			loInsert.setString(2, psIdPeer);
			moLog.debug("Parametro<"+PFC_Constantes.CONFIGURACION_ID_PEER+"> creado <"+psIdPeer+">");
			loInsert.executeUpdate();
			loInsert.clearParameters();
			
			//Se inserta nombre del grupo
			loInsert.setString(1, PFC_Constantes.CONFIGURACION_NOMBRE_GRUPO);
			loInsert.setString(2, psNomGrupo);
			moLog.debug("Parametro<"+PFC_Constantes.CONFIGURACION_NOMBRE_GRUPO+"> creado <"+psNomGrupo+">");
			loInsert.executeUpdate();
			loInsert.clearParameters();
			
			//Se inserta identificador del pipe Multicast
			loInsert.setString(1, PFC_Constantes.CONFIGURACION_ID_PIPE_MULTICAST);
			loInsert.setString(2, psIdPipeMulticast);
			moLog.debug("Parametro<"+PFC_Constantes.CONFIGURACION_ID_PIPE_MULTICAST+"> creado <"+psIdPipeMulticast+">");
			loInsert.executeUpdate();
			loInsert.clearParameters();

			//Se inserta identificador del pipe Unicast
			loInsert.setString(1, PFC_Constantes.CONFIGURACION_ID_PIPE_UNICAST);
			loInsert.setString(2, psIdPipeUnicast);
			moLog.debug("Parametro<"+PFC_Constantes.CONFIGURACION_ID_PIPE_UNICAST+"> creado <"+psIdPipeUnicast+">");
			loInsert.executeUpdate();
			loInsert.clearParameters();

			//Se inserta la IP del SuperPeer
			loInsert.setString(1, PFC_Constantes.CONFIGURACION_SUPERPEER_IP);
			loInsert.setString(2, psIpSP);
			moLog.debug("Parametro<"+PFC_Constantes.CONFIGURACION_SUPERPEER_IP+"> creado <"+psIpSP+">");
			loInsert.executeUpdate();
			loInsert.clearParameters();
			
			//Se inserta el Puerto del SuperPeer
			loInsert.setString(1, PFC_Constantes.CONFIGURACION_SUPERPEER_PUERTO);
			loInsert.setString(2, psPuertoSP);
			moLog.debug("Parametro<"+PFC_Constantes.CONFIGURACION_SUPERPEER_PUERTO+"> creado <"+psPuertoSP+">");
			loInsert.executeUpdate();
			loInsert.clearParameters();
			
			//Se inserta directorio
			loInsert.setString(1, PFC_Constantes.CONFIGURACION_DIRECTORIO);
			loInsert.setString(2, psDirectorio);
			moLog.debug("Parametro<"+PFC_Constantes.CONFIGURACION_DIRECTORIO+"> creado <"+psDirectorio+">");
			loInsert.executeUpdate();
			loInsert.clearParameters();
			
			//Se inserta parametro que indica que no hay cambios aplicar
			loInsert.setString(1, PFC_Constantes.CONFIGURACION_CAMBIOS);
			loInsert.setString(2, "N");
			moLog.debug("Parametro<"+PFC_Constantes.CONFIGURACION_CAMBIOS+"> creado <N>");
			loInsert.executeUpdate();
			loInsert.clearParameters();
			
		}catch (Exception e)
		{
			moLog.error("Problemas al insertar la configuracion:"+e.getMessage());
			throw new Exception("No se he podido generar la configuracion!");
		}
		finally
		{
			loInsert.close();
			loInsert = null;			
		}				
		
    	//Se crea la tabla documentos
    	try 
    	{		
    		loCreate = moConexion.createStatement();
    		
			moLog.info("Peticion de crear tabla Documentos para Peer");				
    		lsTabla = "CREATE TABLE DOCUMENTOS  "
            	+  "(DOC_IDDOCUMENTO VARCHAR(111) NOT NULL,"                  
                +  " DOC_PROPIETARIO VARCHAR(80) NOT NULL," 
                +  " DOC_NOMBRE VARCHAR(35) NOT NULL," 
                +  " DOC_RUTA VARCHAR(300) NOT NULL," 
                +  " DOC_FECHA TIMESTAMP NOT NULL,"
                +  " PRIMARY KEY(DOC_IDDOCUMENTO)) " ;    			

	    		//Se crea la tabla
            loCreate.execute(lsTabla);
            
			moLog.debug("Tabla DOCUMENTOS creada!");
			
			//Sentencia para crear la tabla de DOC_CAMBIOS para el SuperPeer
			moLog.info("Peticion de crear tabla DOC_CAMBIOS");
			
    		lsTabla = "CREATE TABLE DOC_CAMBIOS  "
            	+  "(CAM_ID VARCHAR(63) NOT NULL,"                  
                +  " CAM_IDDOC VARCHAR(111) NOT NULL,"
                +  " CAM_TIPO VARCHAR(1) NOT NULL," 
                +  " CAM_FECHA TIMESTAMP NOT NULL,"
                +  " CAM_NODO VARCHAR(50),"
                +  " CAM_SUBTIPO VARCHAR(1),"
                +  " CAM_CONTENIDO VARCHAR(1000),"
                +  " CAM_ORDEN INTEGER,"
                +  " PRIMARY KEY(CAM_ID))"; 

    		//Se crea la tabla
            loCreate.execute(lsTabla);

			moLog.debug("Tabla DOC_CAMBIOS creada!");
            
		} catch (SQLException e) 
		{
			moLog.error("Problemas al crear las tablas de documentos y cambios:"+e.getMessage());
			try 
			{
				loCreate.close();
				loCreate = null;
			} catch (SQLException e1) 
			{
				moLog.error("Problemas al cerrar el Statement:"+e1.getMessage());			
			}					
			throw new Exception("No se he podido generar la configuracion!");
		}
		
		
    }//inicializaSistema
    
    /**
     * <p>Obtiene la configuracion y la cargas en las variables de trabajo</p>
     * @throws Exception
     */
    public void cargaConfiguracion() throws Exception
    {
    	//Sentencia a ejecutar
    	PreparedStatement loConsulta = null;
    	ResultSet loResultado = null;
    	String lsConsulta = null;
    	
		try
		{
			//Una vez creada la tabla, se insertan los valores de configuracion
			lsConsulta = "SELECT CONFIG_VALOR FROM CONFIGURACION WHERE CONFIG_PARAMETRO = ?";
			
			loConsulta = moConexion.prepareStatement(lsConsulta);

			//Se consulta nombre del peer
			loConsulta.setString(1, PFC_Constantes.CONFIGURACION_NOMBRE_PEER);
			loResultado = loConsulta.executeQuery();
			
			try
			{
				loResultado.next();
				PFC_Constantes.NODO_NOMBRE = loResultado.getString("CONFIG_VALOR");
				moLog.debug("Carga de configuracion. Nombre del nodo cargado<"+PFC_Constantes.NODO_NOMBRE+">");
			}catch(Exception e)
			{
				moLog.error("Problemas al obtener de configuracion el nombre del Nodo:"+e.getMessage());
				throw new Exception("Problemas al obtener de configuracion el nombre del Nodo");
			}
			
			//Se consulta identificador del peer
			loConsulta.setString(1, PFC_Constantes.CONFIGURACION_ID_PEER);
			loResultado = loConsulta.executeQuery();
			
			try
			{
				loResultado.next();
				PFC_Constantes.NODO_ID = loResultado.getString("CONFIG_VALOR");
				moLog.debug("Carga de configuracion. ID del nodo cargado<"+PFC_Constantes.NODO_ID+">");
			}catch(Exception e)
			{
				moLog.error("Problemas al obtener de configuracion el identificador del Nodo:"+e.getMessage());
				throw new Exception("Problemas al obtener de configuracion el identificador del Nodo");
			}
			
			//Se consulta nombre del grupo
			loConsulta.setString(1, PFC_Constantes.CONFIGURACION_NOMBRE_GRUPO);
			loResultado = loConsulta.executeQuery();
			
			try
			{
				loResultado.next();
				PFC_Constantes.GRUPO_TRABAJO_NOMBRE = loResultado.getString("CONFIG_VALOR");
				moLog.debug("Carga de configuracion. Nombre del grupo cargado<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			}catch(Exception e)
			{
				moLog.error("Problemas al obtener de configuracion el nombre del grupo:"+e.getMessage());
				throw new Exception("Problemas al obtener de configuracion el nombre del Grupo");
			}
			
			//Se consulta identificador del Pipe Multicast
			loConsulta.setString(1, PFC_Constantes.CONFIGURACION_ID_PIPE_MULTICAST);
			loResultado = loConsulta.executeQuery();
			
			try
			{
				loResultado.next();
				PFC_Constantes.PIPE_MULTICAST_ID = loResultado.getString("CONFIG_VALOR");
				moLog.debug("Carga de configuracion. ID del Pipe Multicast cargado<"+PFC_Constantes.PIPE_MULTICAST_ID+">");
			}catch(Exception e)
			{
				moLog.error("Problemas al obtener de configuracion el ID del Pipe Multicast:"+e.getMessage());
				throw new Exception("Problemas al obtener de configuracion el ID del Pipe Multicast");
			}
			
			//Se consulta identificador del Pipe Unicast
			loConsulta.setString(1, PFC_Constantes.CONFIGURACION_ID_PIPE_UNICAST);
			loResultado = loConsulta.executeQuery();
			
			try
			{
				loResultado.next();
				PFC_Constantes.PIPE_UNICAST_ID = loResultado.getString("CONFIG_VALOR");
				moLog.debug("Carga de configuracion. ID del Pipe Unicast cargado<"+PFC_Constantes.PIPE_UNICAST_ID+">");
			}catch(Exception e)
			{
				moLog.error("Problemas al obtener de configuracion el ID del Pipe Unicast:"+e.getMessage());
				throw new Exception("Problemas al obtener de configuracion el ID del Pipe Unicast");
			}
			
			//Se consulta IP del SuperPeer
			loConsulta.setString(1, PFC_Constantes.CONFIGURACION_SUPERPEER_IP);
			loResultado = loConsulta.executeQuery();
			
			try
			{
				loResultado.next();
				PFC_Constantes.SUPERPEER_IP = loResultado.getString("CONFIG_VALOR");
				moLog.debug("Carga de configuracion. IP del SuperPeer cargado<"+PFC_Constantes.SUPERPEER_IP+">");
			}catch(Exception e)
			{
				moLog.error("Problemas al obtener de configuracion la IP del SuperPeer:"+e.getMessage());
				throw new Exception("Problemas al obtener de configuracion la IP del SuperPeer");
			}

			//Se consulta Puerto del SuperPeer
			loConsulta.setString(1, PFC_Constantes.CONFIGURACION_SUPERPEER_PUERTO);
			loResultado = loConsulta.executeQuery();
			
			try
			{
				loResultado.next();
				PFC_Constantes.SUPERPEER_PUERTO = loResultado.getString("CONFIG_VALOR");
				moLog.debug("Carga de configuracion. Puerto del SuperPeer cargado<"+PFC_Constantes.SUPERPEER_PUERTO+">");
			}catch(Exception e)
			{
				moLog.error("Problemas al obtener de configuracion la Puerto del SuperPeer:"+e.getMessage());
				throw new Exception("Problemas al obtener de configuracion la Puerto del SuperPeer");
			}
			
			//Se consulta directorio
			loConsulta.setString(1, PFC_Constantes.CONFIGURACION_DIRECTORIO);
			loResultado = loConsulta.executeQuery();
			
			try
			{
				loResultado.next();
				PFC_Constantes.DIRECTORIO = loResultado.getString("CONFIG_VALOR");
				moLog.debug("Carga de configuracion. Directorio cargado<"+PFC_Constantes.DIRECTORIO+">");
			}catch(Exception e)
			{
				moLog.error("Problemas al obtener de configuracion el Directorio:"+e.getMessage());
				throw new Exception("Problemas al obtener de configuracion el Directorio");
			}

			//Se consulta si hay cambios aplicar
			loConsulta.setString(1, PFC_Constantes.CONFIGURACION_CAMBIOS);
			loResultado = loConsulta.executeQuery();
			
			try
			{
				loResultado.next();
				String lsResultado = loResultado.getString("CONFIG_VALOR");
				PFC_Constantes.CAMBIO_CONFIGURACION = (lsResultado == null || lsResultado.equals("N") ) ? false : true;
				moLog.debug("Carga de configuracion. cambios<"+PFC_Constantes.CAMBIO_CONFIGURACION+">");
			}catch(Exception e)
			{
				moLog.error("Problemas al obtener de configuracion el parametro cambios:"+e.getMessage());
				throw new Exception("Problemas al obtener de configuracion el parametro cambios");
			}
		}catch (Exception e)
		{
			moLog.error("Problemas al cargar la configuracion:"+e.getMessage());
			throw new Exception("No se he podido cargar la configuracion!");
		}
		finally
		{
			loResultado.close();
			loResultado = null;
			loConsulta.close();
			loConsulta = null;			
		}				

    	
    }//cargaConfiguracion
    
    
    /**
     * <p>Verifica si una determinada tabla ya existe en la base de datos</p>
     * @param psNomTabla
     * @return
     */
    public boolean existeTabla(String psNomTabla)
    {
    	DatabaseMetaData loMetadata = null;
    	String[] lvsTipo = { "TABLE" };
    	ResultSet loResultado = null;
    	boolean lbSalir = false;
    	String lsTabla = null;
    	
    	try 
    	{
			loMetadata = moConexion.getMetaData();
			
			loResultado = loMetadata.getTables(null, null, null, lvsTipo);
			
			//Recorremos tablas existentes en busca del objeto de la busqueda
			while ( (!lbSalir) && (loResultado.next()) )
			{
				lsTabla = loResultado.getString("TABLE_NAME");
				if ( (lsTabla != null) && (lsTabla.equalsIgnoreCase(psNomTabla)) )
				{
					lbSalir = true;
					moLog.debug("Tabla<"+psNomTabla+"> SI que existe en base de datos.");
				}
			}
			
			if (!lbSalir)
			{
				moLog.debug("Tabla<"+psNomTabla+"> NO existe en base de datos.");
			}
			
			
		} catch (SQLException e) 
		{
			moLog.error("Problemas al verificar si existe tabla.", e);
		}
    	finally
    	{
    		try {
				loResultado.close();
				loResultado = null;
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el resultset:", e);				
			}    		
    	}

    	return lbSalir;
    	
    }//existeTabla
    
    /**
     * <p>Obtiene todos los documentos existentes en la base de datos</p>
     * <p>Se crea un objeto tipo PFC_Documentos que contiene todo el listado de documentos existentes.</p>
     * @return
     */
    public PFC_Documentos obtenerDocumentos()
    {
    	//Sentencia a ejecutar
    	PreparedStatement loConsulta = null;
    	ResultSet loResultado = null;
    	String lsConsulta = null;
    	PFC_Documentos moDocumentos = null;
    	
		try
		{			
			//Una vez creada la tabla, se insertan los valores de configuracion
			lsConsulta = "SELECT DOC_IDDOCUMENTO, DOC_PROPIETARIO, DOC_NOMBRE, DOC_RUTA, DOC_FECHA FROM DOCUMENTOS ORDER BY DOC_FECHA DESC";
												  
			loConsulta = moConexion.prepareStatement(lsConsulta);

			//Se consultaN los documentos			
			loResultado = loConsulta.executeQuery();
						
			moDocumentos = new PFC_Documentos();
			while (loResultado.next())
			{
				moDocumentos.addFila();
				
				//Se obtiene el identificador
				moDocumentos.addElemento(loResultado.getString("DOC_IDDOCUMENTO"));

				//Se obtiene el propietario
				moDocumentos.addElemento(loResultado.getString("DOC_PROPIETARIO"));
				
				//Se obtiene el nombre
				moDocumentos.addElemento(loResultado.getString("DOC_NOMBRE"));

				//Se obtiene la ruta
				moDocumentos.addElemento(loResultado.getString("DOC_RUTA"));
				
				//Se obtiene la fecha actualizado
				moDocumentos.addElemento(loResultado.getTimestamp("DOC_FECHA"));
				
								
				moDocumentos.addRegistro();
			}//while

		}catch (Exception e)
		{
			moLog.error("Problemas al obtener los documentos:"+e.getMessage());
		}
		finally
		{
			try 
			{
				loResultado.close();
				loResultado = null;
				loConsulta.close();
				loConsulta = null;			
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}				

		return moDocumentos;
		
    }//obtenerDocumentos

    /**
     * <p>Da de alta un documento en la tabla documentos.</p>
     * <p>Previamente verifica que no exista dicho nombre/ruta de documento. En tal caso propaga error</p>
     * @param psIdDoc Identificador del documento
     * @param psPropietario Identificador del propietario
     * @param psNombre Nombre del documento
     * @param psRuta Ruta del fichero
     * @param psFecha Fecha alta
     * @return
     * @throws Exception 
     */
    public void guardarDocumento(String psIdDoc, String psPropietario, String psNombre, String psRuta,
    		Date poFecha) throws Exception
    {
    	//Sentencia a ejecutar
    	PreparedStatement loConsulta = null;
    	PreparedStatement loInsert = null;
    	ResultSet loResultado = null;
    	String lsConsulta = null;
    	String lsInsert = null;
    	
		try
		{		
			moLog.debug("Guardar Documento - Peticion nuevo documento. IdDoc<"+psIdDoc+"> Propietario<"+psPropietario+">"+
					" Nombre<"+psNombre+"> Ruta<"+psRuta+"> Fecha<"+poFecha+">");
			
			// Se verifica si existe un documento con el mismo nombre y 
			lsConsulta = "SELECT DOC_IDDOCUMENTO FROM DOCUMENTOS WHERE DOC_NOMBRE = ? AND DOC_PROPIETARIO = ?";
												  
			loConsulta = moConexion.prepareStatement(lsConsulta);
			
			// Se consulta existencia del documento
			loConsulta.setString(1, psNombre);
			loConsulta.setString(2, psPropietario);
			loResultado = loConsulta.executeQuery();

			
			if (loResultado.next())
			{
				// En caso de existir, indicamos el error
				moLog.debug("Guardar Documento - Documento con nombre<"+psNombre+"> YA existe en BBDD!");
				throw new Exception("Documento<"+psNombre+"> YA existe en base de datos.");
			}

			//Se da de alta un nuevo documento en la base de datos
			lsInsert = "insert into documentos (doc_iddocumento,doc_propietario,doc_nombre,doc_ruta,doc_fecha)"+
				" values(?, ?, ?, ?, ?)";
			
			loInsert = moConexion.prepareStatement(lsInsert);
			
			//Se inserta nombre del peer
			loInsert.setString(1, psIdDoc);
			loInsert.setString(2, psPropietario);
			loInsert.setString(3, psNombre);
			loInsert.setString(4, psRuta);
			//loInsert.setDate(5, poFecha);
			loInsert.setTimestamp(5, new Timestamp(poFecha.getTime()));
			
			loInsert.executeUpdate();
			
			
		}catch (Exception e)
		{
			moLog.error("Problemas al dar de alta el documento:"+e.getMessage());
			throw e;
		}
		finally
		{
			try 
			{
				loResultado.close();
				loResultado = null;
				loConsulta.close();
				loConsulta = null;
				loInsert.close();
				loInsert = null;
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}						
		
    }//guardarDocumento

    /**
     * <p>Da de alta un documento en la tabla documentos para SuperPeer</p>
     * <p>SuperPeer lo que guardara sera censo de documentos con su propietario. No guarda copia local del documentos</p>
     * @param psIdDoc Identificador del documento
     * @param psPropietario Identificador del propietario
     * @param psNombre Nombre del documento
     * @param psFecha Fecha alta
     * @param psRuta Ruta
     * @return
     * @throws Exception 
     */
    public void guardarDocumentoSP(String psIdDoc, String psPropietario, String psNombre, Date poFecha, String psRuta) throws Exception
    {
    	//Sentencia a ejecutar
    	PreparedStatement loInsert = null;
    	String lsInsert = null;
    	
		try
		{		
			moLog.debug("Guardar Documento - Peticion nuevo documento en SuperPeer. IdDoc<"+psIdDoc+"> Propietario<"+psPropietario+">"+
					" Nombre<"+psNombre+"> Fecha<"+poFecha+"> Ruta<"+psRuta+">");
			
			//Se da de alta un nuevo documento en la base de datos
			lsInsert = "insert into documentos (doc_iddocumento,doc_propietario,doc_nombre,doc_fecha, doc_ruta) values(?, ?, ?, ?, ?)";
			
			loInsert = moConexion.prepareStatement(lsInsert);
			
			//Se inserta nombre del peer
			loInsert.setString(1, psIdDoc);
			loInsert.setString(2, psPropietario);
			loInsert.setString(3, psNombre);
			loInsert.setTimestamp(4, new Timestamp(poFecha.getTime()));
			loInsert.setString(5, psRuta);
			
			loInsert.executeUpdate();
			
			
		}catch (Exception e)
		{
			moLog.error("Problemas al dar de alta el documento para SuperPeer:"+e.getMessage());
			throw e;
		}
		finally
		{
			try 
			{
				loInsert.close();
				loInsert = null;
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}						
		
    }//guardarDocumentoSP    
    /**
     * <p>Modificacion informacion de un documento.</p>
     * <p>Los datos del documento que se pueden modificar son la Fecha y si se notificado el cambio</p>
     * @param psIdDoc Identificador del documento
     * @param psPropietario Identificador del propietario
     * @param psFecha Fecha modificacion
     * @return
     * @throws Exception 
     */
    public void modificarDocumento(String psIdDoc, String psPropietario, Date poFecha) throws Exception
    {
    	//Sentencia a ejecutar    	
    	PreparedStatement loUpdate = null;    	    	
    	String lsUpdate = null;
    	
    	
		try
		{		
			moLog.debug("Modificar Documento - Peticion modificar documento. IdDoc<"+psIdDoc+"> Propietario<"+psPropietario+">"+
					" Fecha<"+poFecha+">");
			

			//Se da de alta un nuevo documento en la base de datos
			lsUpdate = "update documentos set doc_fecha = ? where doc_iddocumento = ?";
			
			
			loUpdate = moConexion.prepareStatement(lsUpdate);
			
			loUpdate.setTimestamp(1, new Timestamp(poFecha.getTime()));
			loUpdate.setString(2, psIdDoc);
			
			loUpdate.executeUpdate();
			
			
		}catch (Exception e)
		{
			moLog.error("Problemas al modificar el documento:"+e.getMessage());
			throw e;
		}
		finally
		{
			try 
			{
				loUpdate.close();
				loUpdate = null;
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}						
		
    }//modificarDocumento

    /**
     * <p>Da de alta un cambio en la tabla DOC_CAMBIOS</p>
     * @param psIdDoc
     * @param psTipo
     * @param poFecha
     * @param psNodo
     * @param psSubtipo
     * @param psContenido
     * @param psOrden
     * @throws Exception
     */
    public void altaNotificacion(String psIdDoc, String psTipo, Date poFecha, String psNodo, String psSubtipo, 
    		String psContenido, String psOrden) throws Exception
    {
    	//Sentencia a ejecutar
    	PreparedStatement loInsert = null;
    	String lsInsert = null;
    	String lsNotId = "";
    	
		try
		{		
			// En el caso de realizarse una modificacion, el identificador tendra concatenado el nodo
			if (psTipo.equals("M"))
				lsNotId = psNodo;
			
			lsNotId += System.currentTimeMillis();
			moLog.debug("Alta nuevo cambio. IdDoc<"+psIdDoc+"> Tipo<"+psTipo+"> NotId<"+lsNotId+"> Fecha<"+poFecha+">"+
					"Nodo<"+psNodo+"> Subtipo<"+psSubtipo+"> Orden<"+psOrden+">");
			
			//Se da de alta un nuevo documento en la base de datos
			lsInsert = "insert into doc_cambios (cam_id, cam_iddoc, cam_tipo, cam_fecha, cam_nodo, cam_subtipo,"+
				"cam_contenido, cam_orden) values(?, ?, ?, ?, ?, ?, ?, ?)";
			
			loInsert = moConexion.prepareStatement(lsInsert);
			
			//Se inserta nombre del peer
			loInsert.setString(1, lsNotId);
			loInsert.setString(2, psIdDoc);
			loInsert.setString(3, psTipo);
			loInsert.setTimestamp(4, new Timestamp(poFecha.getTime()));
			loInsert.setString(5, psNodo);
			loInsert.setString(6, psSubtipo);
			loInsert.setString(7, psContenido);
			loInsert.setInt(8, (psOrden != null ? new Integer(psOrden) : 0));
			
			loInsert.executeUpdate();
			
			
		}catch (Exception e)
		{
			moLog.error("Problemas al dar de alta una notificacion:"+e);
			throw e;
		}
		finally
		{
			try 
			{
				loInsert.close();
				loInsert = null;
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}						
		
    }//altaNotificacion    
    

    /**
     * <p>Elimina el documento.</p>
     * <p>Elimina el regsitro definitivamente</p>
     * @param psIdDoc Identificador del documento
     * @return
     * @throws Exception 
     */
    public void eliminarDocumento(String psIdDoc) throws Exception
    {
    	//Sentencia a ejecutar    	
    	PreparedStatement loDelete = null;    	    	
    	String lsDelete = null;
    	
    	
		try
		{		
			moLog.debug("Eliminar Documento - Peticion de eliminar documento. IdDoc<"+psIdDoc+">");
			

			//Se da de alta un nuevo documento en la base de datos
			lsDelete = "delete from documentos where doc_iddocumento = ? ";						
			loDelete = moConexion.prepareStatement(lsDelete);
			
			loDelete.setString(1, psIdDoc);			
			
			loDelete.executeUpdate();
			
			
		}catch (Exception e)
		{
			moLog.error("Problemas al eliminar el documento:"+e.getMessage());
			throw e;
		}
		finally
		{
			try 
			{
				loDelete.close();
				loDelete = null;
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}						
		
    }//eliminarDocumento

    /**
     * <p>Elimina cualquier cambio asociado a un documento.</p>
     * <p>En caso de psIdDoc==null, se eliminan todos los cambios</p>
     * @param psIdDoc Identificador del documento
     * @return
     * @throws Exception 
     */
    public void eliminarCambios(String psIdDoc) throws Exception
    {
    	//Sentencia a ejecutar    	
    	PreparedStatement loDelete = null;    	    	
    	String lsDelete = null;
    	
    	
		try
		{		
			moLog.debug("Eliminar Documento - Peticion de eliminar cambios. IdDoc<"+psIdDoc+">");
			

			//Se da de alta un nuevo documento en la base de datos
			if (psIdDoc != null)
				lsDelete = "delete from doc_cambios where cam_iddoc = ? ";
			else
				lsDelete = "delete from doc_cambios";
			loDelete = moConexion.prepareStatement(lsDelete);
			
			if (psIdDoc != null)				
				loDelete.setString(1, psIdDoc);			
			
			loDelete.executeUpdate();
			
			
		}catch (Exception e)
		{
			moLog.error("Problemas al eliminar cambios:"+e.getMessage());
			throw e;
		}
		finally
		{
			try 
			{
				loDelete.close();
				loDelete = null;
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}						
		
    }//eliminarCambios

    
    /**
     * <p>Modificacion informacion de configuracion. Cambio del directorio</p>
     * @param psDirectorio Nuevo directorio
     * @return
     * @throws Exception 
     */
    public void modificarDirectorio(String psDirectorio) throws Exception
    {
    	//Sentencia a ejecutar    	
    	PreparedStatement loUpdate = null;    	    	
    	String lsUpdate = null;
    	
    	
		try
		{		
			moLog.debug("Modificar Configuracion - Nuevo directorio. Directorio<"+psDirectorio+">");
			
			
			//Se modifica el parametro 
			lsUpdate = "update configuracion set CONFIG_VALOR = ? where CONFIG_PARAMETRO = ?";
						
			loUpdate = moConexion.prepareStatement(lsUpdate);
			
			loUpdate.setString(1, psDirectorio);
			loUpdate.setString(2, PFC_Constantes.CONFIGURACION_DIRECTORIO);
			
			loUpdate.executeUpdate();
			
			
		}catch (Exception e)
		{
			moLog.error("Problemas al modificar el directorio:"+e.getMessage());
			throw e;
		}
		finally
		{
			try 
			{
				loUpdate.close();
				loUpdate = null;
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}						
		
    }//modificarDirectorio
   
    /**
     * <p>Modificacion informacion de configuracion. Cambio de la IP del SuperPeer.</p>
     * @param psSuperPeerIP Nueva IP del SuperPeer
     * @return
     * @throws Exception 
     */
    public void modificarSuperPeerIP(String psSuperPeerIP) throws Exception
    {
    	//Sentencia a ejecutar    	
    	PreparedStatement loUpdate = null;    	    	
    	String lsUpdate = null;
    	
    	
		try
		{		
			moLog.debug("Modificar Configuracion - Nueva IP del SuperPeer. IP<"+psSuperPeerIP+">");
			
			
			//Se modifica el parametro 
			lsUpdate = "update configuracion set CONFIG_VALOR = ? where CONFIG_PARAMETRO = ?";
						
			loUpdate = moConexion.prepareStatement(lsUpdate);
			
			loUpdate.setString(1, psSuperPeerIP);
			loUpdate.setString(2, PFC_Constantes.CONFIGURACION_SUPERPEER_IP);
			
			loUpdate.executeUpdate();
			
			
		}catch (Exception e)
		{
			moLog.error("Problemas al modificar la IP del SuperPeer:"+e.getMessage());
			throw e;
		}
		finally
		{
			try 
			{
				loUpdate.close();
				loUpdate = null;
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}						
		
    }//modificarSuperPeerIP

    /**
     * <p>Modificacion informacion de configuracion. Cambio del Puerto del SuperPeer</p>
     * @param psSuperPeerPuerto Nuevo Puerto del SuperPeer
     * @return
     * @throws Exception 
     */
    public void modificarSuperPeerPuerto(String psSuperPeerPuerto) throws Exception
    {
    	//Sentencia a ejecutar    	
    	PreparedStatement loUpdate = null;    	    	
    	String lsUpdate = null;
    	
    	
		try
		{		
			moLog.debug("Modificar Configuracion - Nuevo Puerto del SuperPeer. Puerto<"+psSuperPeerPuerto+">");
			
			
			//Se modifica el parametro 
			lsUpdate = "update configuracion set CONFIG_VALOR = ? where CONFIG_PARAMETRO = ?";
						
			loUpdate = moConexion.prepareStatement(lsUpdate);
			
			loUpdate.setString(1, psSuperPeerPuerto);
			loUpdate.setString(2, PFC_Constantes.CONFIGURACION_SUPERPEER_PUERTO);
			
			loUpdate.executeUpdate();
			
			
		}catch (Exception e)
		{
			moLog.error("Problemas al modificar el Puerto del SuperPeer:"+e.getMessage());
			throw e;
		}
		finally
		{
			try 
			{
				loUpdate.close();
				loUpdate = null;
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}						
		
    }//modificarSuperPeerPuerto    
    
    /**
     * <p>Modificacion informacion de configuracion. Cambio del parametro que indica que hay cambios en la configuracion</p>
     * @param psCambio S/N indicando si hay o no que aplicar cambios
     * @return
     * @throws Exception 
     */
    public void modificarCambio(String psCambio) throws Exception
    {
    	//Sentencia a ejecutar    	
    	PreparedStatement loUpdate = null;    	    	
    	String lsUpdate = null;
    	
    	
		try
		{		
			moLog.debug("Modificar Configuracion - Cambio en la configuracion<"+psCambio+">");
			
			
			//Se modifica el parametro 
			lsUpdate = "update configuracion set CONFIG_VALOR = ? where CONFIG_PARAMETRO = ?";
						
			loUpdate = moConexion.prepareStatement(lsUpdate);
			
			loUpdate.setString(1, psCambio);
			loUpdate.setString(2, PFC_Constantes.CONFIGURACION_CAMBIOS);
			
			loUpdate.executeUpdate();
			
			
		}catch (Exception e)
		{
			moLog.error("Problemas al modificar el parametro cambio:"+e.getMessage());
			throw e;
		}
		finally
		{
			try 
			{
				loUpdate.close();
				loUpdate = null;
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}						
		
    }//modificarCambio    

    
    /**
     * <p>Obtiene todos los documentos que no son de la propiedad del nodo indicado por parametro</p>
     * <p>Se crea un objeto tipo PFC_Mensaje listo para poder ser enviado al SP</p>
     * @param psPropietario id unico del Peer que no queremos obtener los documentos
     * @return
     */
    public Hashtable obtenerMisNoDocumentos(String psPropietario)
    {
    	//Sentencia a ejecutar
    	PreparedStatement loConsulta = null;
    	ResultSet loResultado = null;
    	String lsConsulta = null;    	
    	Hashtable loDocumentos = null;
    	String[] lvsDatos = null;
    	String lsKey = null;
    	
		try
		{						
			moLog.info("Peticion para obtene documentos que no sean del <"+psPropietario+">");
			//Una vez creada la tabla, se insertan los valores de configuracion
			lsConsulta = "SELECT DOC_IDDOCUMENTO, DOC_PROPIETARIO, DOC_NOMBRE, DOC_FECHA FROM DOCUMENTOS WHERE DOC_PROPIETARIO <> ?";
												  
			loConsulta = moConexion.prepareStatement(lsConsulta);
			
			// Buscamos todos los documentos que no sean propiedad del nodo peticionario
			loConsulta.setString(1, psPropietario);
			
			//Se consultaN los documentos			
			loResultado = loConsulta.executeQuery();
						
			loDocumentos = new Hashtable();
			while (loResultado.next())
			{
				lvsDatos = new String[3];
								
				//Se obtiene el identificador
				lsKey = loResultado.getString("DOC_IDDOCUMENTO");
				
				//Se obtiene el propietario				
				lvsDatos[0] = loResultado.getString("DOC_PROPIETARIO"); 
				
				//Se obtiene el nombre
				lvsDatos[1] = loResultado.getString("DOC_NOMBRE");
				
				//Se obtiene la fecha actualizado
				lvsDatos[2] = ""+loResultado.getTimestamp("DOC_FECHA");
								
				loDocumentos.put(lsKey, lvsDatos);
			}//while

			moLog.info("Documentos que no sean del <"+psPropietario+">=<"+(loDocumentos != null ? loDocumentos.size() : 0)+">");
			
		}catch (Exception e)
		{
			moLog.error("Problemas al obtener los documentos NO propiedad del nodo:"+e.getMessage());
		}
		finally
		{
			try 
			{
				loResultado.close();
				loResultado = null;
				loConsulta.close();
				loConsulta = null;			
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}				

		return loDocumentos;
		
    }//obtenerMisNoDocumentos
    
    /**
     * <p>Obtiene los cambios que se han realizado en un determinado documento a partir de una fecha</p>
     * @param psIdDoc
     * @param psTipo
     * @param poFecha
     * @return
     */
    public Vector obtenerCambiosDocumento(String psIdDoc, String psTipo, Date poFecha)
    {
    	//Sentencia a ejecutar
    	PreparedStatement loConsulta = null;
    	ResultSet loResultado = null;
    	String lsConsulta = null;    	
    	Vector loCambios = null;
    	String[] lvsDatos = null;

    	
		try
		{						
			moLog.info("Peticion para obtener cambios sobre el documento<"+psIdDoc+"> a partir de fecha<"+new Timestamp(poFecha.getTime())+"> y del tipo<"+psTipo+">");
			//Una vez creada la tabla, se insertan los valores de configuracion
			lsConsulta = "SELECT CAM_NODO, CAM_SUBTIPO, CAM_CONTENIDO FROM DOC_CAMBIOS WHERE CAM_IDDOC = ? AND CAM_TIPO = ? AND CAM_FECHA > ? ORDER BY CAM_FECHA, CAM_ORDEN";
			
			loConsulta = moConexion.prepareStatement(lsConsulta);
			
			// Buscamos todos los documentos que no sean propiedad del nodo peticionario
			loConsulta.setString(1, psIdDoc);			
			loConsulta.setString(2, psTipo);
			loConsulta.setTimestamp(3, new Timestamp(poFecha.getTime()));
			
			//Se consultaN los documentos			
			loResultado = loConsulta.executeQuery();
						
			loCambios = new Vector();
			while (loResultado.next())
			{
				lvsDatos = new String[3];
								
				//Se obtiene el identificador de nodo
				lvsDatos[0] = loResultado.getString("CAM_NODO");
				
				//Se obtiene el tipo del cambio				
				lvsDatos[1] = loResultado.getString("CAM_SUBTIPO"); 
				
				//Se obtiene el contenido
				lvsDatos[2] = loResultado.getString("CAM_CONTENIDO");
				
								
				loCambios.add(lvsDatos);
			}//while

			moLog.info("Numero de cambios <"+(loCambios != null ? loCambios.size() : 0)+"> del Documento<"+psIdDoc+">");
			
		}catch (Exception e)
		{
			moLog.error("Problemas al obtener los cambios del documento:"+e);
		}
		finally
		{
			try 
			{
				loResultado.close();
				loResultado = null;
				loConsulta.close();
				loConsulta = null;			
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}				

		return loCambios;
		
    }//obtenerCambiosDocumento

    /**
     * <p>Obtiene los cambios que se han realizado en cualquier documento</p>
     * @param psIdDoc
     * @param psTipo
     * @param poFecha
     * @return
     */
    public Hashtable obtenerCambiosDocs()
    {
    	//Sentencia a ejecutar
    	PreparedStatement loConsulta = null;
    	ResultSet loResultado = null;
    	String lsConsulta = null;    	
    	Vector loCambios = null;
    	Hashtable loDocs = null;
    	String[] lvsDatos = null;    	
    	String lsIdDoc = null;
    	String lsIdTemp = null;
    	boolean lbPrimer = true;
    	
		try
		{						
			moLog.info("Peticion para obtener cambios realizados en cualquier documento");
			//Cambios en los documentos
			lsConsulta = "SELECT CAM_IDDOC, CAM_NODO, CAM_SUBTIPO, CAM_CONTENIDO, CAM_FECHA, CAM_ORDEN FROM DOC_CAMBIOS WHERE CAM_TIPO = 'M' ORDER BY CAM_IDDOC, CAM_FECHA, CAM_ORDEN";
			
			loConsulta = moConexion.prepareStatement(lsConsulta);
						
			//Se consultaN los documentos			
			loResultado = loConsulta.executeQuery();
						
			loDocs = new Hashtable();
			loCambios = new Vector();
			while (loResultado.next())
			{				
				lvsDatos = new String[5];
				
				// Se obtiene id del documento
				lsIdDoc = loResultado.getString("CAM_IDDOC");
				
				if (lbPrimer)
				{
					lsIdTemp = lsIdDoc;
					lbPrimer = false;
				}

				//Se obtiene el identificador de nodo
				lvsDatos[0] = loResultado.getString("CAM_NODO");
				
				//Se obtiene el tipo del cambio				
				lvsDatos[1] = loResultado.getString("CAM_SUBTIPO"); 
				
				//Se obtiene el contenido
				lvsDatos[2] = loResultado.getString("CAM_CONTENIDO");
				
				//Se obtiene la fecha
				lvsDatos[3] = ""+loResultado.getTimestamp("CAM_FECHA");
				
				//Se obtiene el orden del cambio
				lvsDatos[4] = ""+loResultado.getInt("CAM_ORDEN");
				
				if (!lsIdTemp.equals(lsIdDoc)) 
				{
					loDocs.put(lsIdTemp, loCambios);
					lsIdTemp = lsIdDoc;					
					loCambios = new Vector();					
				}
				
				loCambios.add(lvsDatos);
			}//while
			
			//Se aade el ultimo doc en caso de que se haya obtenido algo
			if (!lbPrimer)
				loDocs.put(lsIdDoc, loCambios);
			
			moLog.info("Numero de cambios realizados en documentos <"+(loCambios != null ? loCambios.size() : 0)+">");
			
		}catch (Exception e)
		{
			moLog.error("Problemas al obtener los cambios de los documentos:"+e.getMessage());
		}
		finally
		{
			try 
			{
				loResultado.close();
				loResultado = null;
				loConsulta.close();
				loConsulta = null;			
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}				

		return loDocs;
		
    }//obtenerCambiosDocs
    
    /**
     * <p>Obtiene los cambios tipo Alta (A) realizados sobre los documentos mientras un peer ha estado desconectado</p>
     * @return
     */
    public Vector obtenerCambiosAlta()
    {
    	//Sentencia a ejecutar
    	PreparedStatement loConsulta = null;
    	ResultSet loResultado = null;
    	String lsConsulta = null;    	
    	Vector loCambios = null;
    	String[] lvsDatos = null;
    	
		try
		{						
			moLog.info("Peticion para obtener cambios tipo Alta realizados sobre los documentos.");
			//Consulta
			lsConsulta = "SELECT DOC.DOC_IDDOCUMENTO, DOC.DOC_NOMBRE, DOC.DOC_RUTA, DOC.DOC_FECHA FROM DOCUMENTOS DOC, DOC_CAMBIOS CAM WHERE CAM.CAM_IDDOC = DOC.DOC_IDDOCUMENTO AND CAM.CAM_TIPO='A'";
												  
			loConsulta = moConexion.prepareStatement(lsConsulta);
			
			//Se consultaN los documentos			
			loResultado = loConsulta.executeQuery();
						
			loCambios = new Vector();
			while (loResultado.next())
			{
				lvsDatos = new String[4];
								
				//Se obtiene el identificador del documento
				lvsDatos[0] = loResultado.getString("DOC_IDDOCUMENTO");
				
				//Se obtiene el nombre del documento				
				lvsDatos[1] = loResultado.getString("DOC_NOMBRE"); 
				
				//Se obtiene la ruta
				lvsDatos[2] = loResultado.getString("DOC_RUTA");
				
				// Se obtiene la fecha
				lvsDatos[3] = ""+loResultado.getTimestamp("DOC_FECHA");
								
				loCambios.add(lvsDatos);
			}//while

			moLog.info("Numero de altas <"+(loCambios != null ? loCambios.size() : 0)+">");
			
		}catch (Exception e)
		{
			moLog.error("Problemas al obtener cambios tipo Alta en los documentos:"+e.getMessage());
		}
		finally
		{
			try 
			{
				loResultado.close();
				loResultado = null;
				loConsulta.close();
				loConsulta = null;			
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}				

		return loCambios;
		
    }//obtenerCambiosAlta    
    
    /**
     * <p>Obtiene los cambios tipo Baja (B) realizados sobre los documentos mientras un peer ha estado desconectado</p>
     * @return
     */
    public Vector obtenerCambiosBaja()
    {
    	//Sentencia a ejecutar
    	PreparedStatement loConsulta = null;
    	ResultSet loResultado = null;
    	String lsConsulta = null;    	
    	Vector loCambios = null;
    	String[] lvsDatos = null;
    	
		try
		{						
			moLog.info("Peticion para obtener cambios tipo Baja realizados sobre los documentos.");
			//Consulta
			lsConsulta = "SELECT CAM_IDDOC FROM DOC_CAMBIOS WHERE CAM_TIPO='B'";
												  
			loConsulta = moConexion.prepareStatement(lsConsulta);
			
			//Se consultaN los documentos			
			loResultado = loConsulta.executeQuery();
						
			loCambios = new Vector();
			while (loResultado.next())
			{
				lvsDatos = new String[1];
								
				//Se obtiene el identificador del documento
				lvsDatos[0] = loResultado.getString("CAM_IDDOC");
				
								
				loCambios.add(lvsDatos);
			}//while

			moLog.info("Numero de bajas <"+(loCambios != null ? loCambios.size() : 0)+">");
			
		}catch (Exception e)
		{
			moLog.error("Problemas al obtener cambios tipo Baja en los documentos:"+e.getMessage());
		}
		finally
		{
			try 
			{
				loResultado.close();
				loResultado = null;
				loConsulta.close();
				loConsulta = null;			
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}				

		return loCambios;
		
    }//obtenerCambiosBaja        
    
    /**
     * <p>Obtiene el path y ruta de un documento y fecha</p>
     * @param psIdDoc Identificador del documento
     * @return String[]  donde String[0]= path y nombre del documento, String[1]=fecha
     */
    public Object[] obtenerDocumento(String psIdDoc)
    {
    	//Sentencia a ejecutar
    	PreparedStatement loConsulta = null;
    	ResultSet loResultado = null;
    	String lsConsulta = null;    	
    	Object[] lvsDocumento = null;
    	
		try
		{						
			moLog.info("Peticion para obtene el path completo y nombre de un documento con id <"+psIdDoc+">");
			//Una vez creada la tabla, se insertan los valores de configuracion
			lsConsulta = "SELECT DOC_RUTA, DOC_NOMBRE, DOC_FECHA, DOC_PROPIETARIO FROM DOCUMENTOS WHERE DOC_IDDOCUMENTO = ?";
												  
			loConsulta = moConexion.prepareStatement(lsConsulta);
			
			// Buscamos el path completo del documento
			loConsulta.setString(1, psIdDoc);
			
			//Se consultaN los documentos			
			loResultado = loConsulta.executeQuery();
						
			if (loResultado.next())
			{
				lvsDocumento = new Object[4];
				//Se obtiene la ruta del documento
				lvsDocumento[0] = loResultado.getString("DOC_RUTA");
				// Se obtiene el nombre del documento
				lvsDocumento[1] = loResultado.getString("DOC_NOMBRE");
				//Se obtiene la fecha
				lvsDocumento[2] = loResultado.getTimestamp("DOC_FECHA").getTime();
				// Se obtiene el propietario
				lvsDocumento[3] = loResultado.getString("DOC_PROPIETARIO");
								
			}//if

			moLog.info("Nombre y ruta del documento con id<"+psIdDoc+"> es <"+lvsDocumento[0]+lvsDocumento[1]+"> con fecha<"+lvsDocumento[2]+">");
			
		}catch (Exception e)
		{
			moLog.error("Problemas al obtener el path completo de un documento:"+e.getMessage());
		}
		finally
		{
			try 
			{
				loResultado.close();
				loResultado = null;
				loConsulta.close();
				loConsulta = null;			
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}				

		return lvsDocumento;
		
    }//obtenerDocumento    

    
    /**
     * <p>Verifica si existe un cambio pendiente de enviar al grupo que indique el alta de un determinado documento</p>
     * @param psIdDoc Identificador del documento que se quiere verificar si hay o no alta
     * @return boolean true si existe alta pendiente de envio
     */
    public boolean existeAltaPendiente(String psIdDoc)
    {
    	//Sentencia a ejecutar
    	PreparedStatement loConsulta = null;
    	ResultSet loResultado = null;
    	String lsConsulta = null;    	
    	boolean lbExiste = false;
    	
		try
		{						
			moLog.info("Peticion para verificar si existe alta pendiente de envio del documento <"+psIdDoc+">");
			//consulta el doc_cambios
			lsConsulta = "SELECT CAM_ID FROM DOC_CAMBIOS WHERE CAM_IDDOC = ? AND CAM_TIPO = 'A'";
												  
			loConsulta = moConexion.prepareStatement(lsConsulta);
			
			// Indicamos el iddocumento
			loConsulta.setString(1, psIdDoc);
			
			//Se verifica si hay o no resultado			
			loResultado = loConsulta.executeQuery();
						
			if (loResultado.next())
			{
				lbExiste = true;				
			}
			else
			{
				lbExiste = false;
			}
			
			moLog.info("Resultado de verificar si existe alta pendiente de envio del doc<"+psIdDoc+"> es <"+lbExiste+">");
			
		}catch (Exception e)
		{
			moLog.error("Problemas al verificar si existe alta pendiente:"+e.getMessage());
		}
		finally
		{
			try 
			{
				loResultado.close();
				loResultado = null;
				loConsulta.close();
				loConsulta = null;			
			} catch (SQLException e) 
			{
				moLog.error("Problemas al cerrar el Prepared:"+e.getMessage());
			}
		}				

		return lbExiste;
		
    }//existeAltaPendiente    

    
    
    /**
     * <p>Cierra conexion de base de datos</p>
     */
    public void cerrarConexion()
    {
    	try 
    	{
			if (moConexion != null)
			{
	    		moConexion.close();
				moConexion = null;
			}
		} catch (SQLException e) 
		{
			moLog.error("Problemas al cerrar la conexion con la base de datos. URL de la base de datos<"+PFC_Constantes.BD_URL+">");
		}
    }//cerrarConexion
}
