package pfc.prototipo.jxta;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.SocketAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.util.StringTokenizer;

import net.jxta.document.AdvertisementFactory;
import net.jxta.id.IDFactory;
import net.jxta.peergroup.PeerGroupID;
import net.jxta.protocol.PipeAdvertisement;

public class PFC_JxtaUtilidades 
{

	/**
	 * <p>Genera un PipeAdvertisement</p>
	 * @param psNombre
	 * @param psId
	 * @param loIdGrupo
	 * @param psTipo
	 * @return
	 * @throws Exception
	 */
	public static PipeAdvertisement generarAdvertisement(String psNombre, String psId, PeerGroupID loIdGrupo, String psTipo) throws Exception 
	{
		PipeAdvertisement advertisement = (PipeAdvertisement)
		AdvertisementFactory.newAdvertisement(PipeAdvertisement.getAdvertisementType());
		advertisement.setType(psTipo);		
		advertisement.setName(psNombre);
		advertisement.setPipeID(IDFactory.fromURI(new URI(psId)));
		return advertisement;
	}//generarAdvertisement
	
	/**
	 * <p>Obtiene de la SocketAddress Origen el PeerID del nodo que ha enviado el mensaje</p>
	 * @param poDireccion SocketAddress Remota del nodo origen que envio el mensaje
	 * @return String con la PeerId del Nodo Origen
	 */
	public static String damePeerId(SocketAddress poDireccion)
	{
		String lsPeerId = "";
		
		String lsDireccion = poDireccion.toString();						
		String line = System.getProperty("line.separator"); 
		
		StringTokenizer loToken = new StringTokenizer(lsDireccion, line);
		
		//Recorremos direccion hasta obtener PeerID
		while (loToken.hasMoreElements())
		{
			String lsElemento = loToken.nextElement().toString();
			
			if (lsElemento.indexOf("PeerID") > 0)
			{
				//Se ha encontrado la informacion del PeerID
				lsPeerId = lsElemento.substring(lsElemento.indexOf("urn:"));
			}
			
		}//while
				
		return lsPeerId;
	}//damePeerId

	
	/**
	 * <p>Converite a ByteArray un Object</p>
	 * @param obj
	 * @return
	 */
	public static byte[] convertirObjectByteArray (Object obj) throws Exception
	{
		byte[] bytes = null;
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		
		ObjectOutputStream oos = new ObjectOutputStream(bos); 
		oos.writeObject(obj);
		oos.flush(); 
		oos.close(); 
		bos.close();
		bytes = bos.toByteArray ();
		return bytes;
	}//convertirByteArray
	
	/**
	 * <p>Convierte un ArrayByte a Object</p>
	 * @param bytes
	 * @return
	 */
	public static Object convertirByteArrayObject (byte[] bytes) throws Exception
	{
		Object obj = null;
		ByteArrayInputStream bis = new ByteArrayInputStream (bytes);
		ObjectInputStream ois = new ObjectInputStream (bis);
		obj = ois.readObject();
		return obj;
	}//convertirByteArrayObject	
	
	/**
	 * <p>Convierte de una fecha en string con formato yyyy-MM-dd HH:mm:ss.SSS a un objeto tipo java.sql.Date</p>
	 * @param psFecha
	 * @return
	 * @throws Exception
	 */
	public static Date obtenerFecha(String psFecha) throws Exception
	{
		SimpleDateFormat loFormato = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		return new Date( (loFormato.parse(psFecha)).getTime());
	}
}
