package pfc.prototipo.jxta.peer;

import java.io.DataInput;
import java.io.DataInputStream;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.sql.Date;

import org.apache.log4j.Logger;

import pfc.prototipo.PFC_Constantes;
import pfc.prototipo.PFC_ManejoFicheros;
import pfc.prototipo.bbdd.PFC_BDGestion;
import pfc.prototipo.jxta.PFC_JxtaUtilidades;
import pfc.prototipo.jxta.PFC_Mensaje;
import pfc.prototipo.rendimiento.PFC_Rendimiento;


import net.jxta.socket.JxtaServerSocket;

public class PFC_PGestorComunicacion  implements Runnable
{

	//Trazas
	private final static Logger moLog = Logger.getLogger(PFC_PGestorComunicacion.class);

	//Canal de comunicacion Multicast
	private JxtaServerSocket moSocket = null;
	
	// Indica cuando acaba el proceso
	private boolean mbSeguir = true;
    //Conexion con la base de datos
    private PFC_BDGestion moGestionBD = null;

	public PFC_PGestorComunicacion(JxtaServerSocket poSocket, PFC_BDGestion poGestionBD)
	{
		//Se inicializa canal de comunicacion
		this.moSocket = poSocket;
		moGestionBD = poGestionBD;
	}//PFC_SPGestorComunicacion()

	
	public void run() 
	{
		Socket loSocket = null;
		PFC_Mensaje loRespuesta = null;
		OutputStream loSalida = null; 
		DataOutput loDatosSalida = null; 
		byte[] loDatos = null;
		int liTamano;
		
		try
		{
			moLog.info("Se inicia proceso de escucha de mensajes por el canal UniCast");

			while (mbSeguir)
			{							
				moLog.info("Canal Unicast del Peer<"+PFC_Constantes.NODO_NOMBRE+"> se queda a la espera de recibir mensajes");
				System.out.println("*****************Canal Unicast del Peer<"+PFC_Constantes.NODO_NOMBRE+"> se queda a la espera de recibir mensajes");

				try
				{
					
					loSocket = moSocket.accept();
	
					moLog.info("Canal Unicast del Peer<"+PFC_Constantes.NODO_NOMBRE+"> ha recibido un mensaje..Pasa a su tratamiento");
					System.out.println("*******************Canal Unicast del Peer<"+PFC_Constantes.NODO_NOMBRE+"> ha recibido un mensaje..Pasa a su tratamiento");
									
					if (loSocket != null) 
					{
						// Se trata mensaje recibido
						loRespuesta = tratarMensaje(loSocket);
						
						// Si el mensaje devuelto es null no se tiene que contestar; si es un mensaje construido, se tiene que contestar al peticionario
						// Flujo de salida de datos
						if (loRespuesta != null)
						{
							loSalida = loSocket.getOutputStream();
							loDatosSalida = new DataOutputStream(loSalida);
									
							loDatos = PFC_JxtaUtilidades.convertirObjectByteArray(loRespuesta);			
							liTamano = loDatos.length;
							
							moLog.info("Se va a enviar mensaje contestacin con Tamao<"+liTamano+">");
							System.out.println("***********Se va a enviar mensaje contestacin con Tamao<"+liTamano+">");
							
							loDatosSalida.writeInt(liTamano);
							loSalida.write(loDatos);
							loSalida.flush();
							loSalida.close();
						}
					}
					else
					{
						moLog.info("Canal Unicast del rPeer<"+PFC_Constantes.NODO_NOMBRE+"> ha recibido un mensaje con socket NULL");
						System.out.println("*******************Canal Unicast del Peer<"+PFC_Constantes.NODO_NOMBRE+"> ha recibido un mensaje con socket NULL");					
					}
				}catch(Exception e)
				{
					moLog.error("Problemas al procesar un mensaje recibido por el canal Unicast:"+e);
				}
				finally
				{
					// Se cierra comunicacin
					if (loSocket != null)
						loSocket.close();
				}
			}//while		

		}catch(Exception e)
		{
			moLog.error("Problemas al recibir mensajes del tipo Unicast:"+e.getMessage());
		}
		
	}//run()
	
	private PFC_Mensaje tratarMensaje(Socket poSocket)
	{
		//Mensaje recibido
		PFC_Mensaje loMensaje = null;
		InputStream loInput = null; 
		DataInput loDatosIn = null; 
		int liTam = 0;
		byte[] loDatos = null;
		PFC_Mensaje loRespuesta = null;
		
		try
		{
			moLog.debug("Peticion de tratar un mensaje recibido por el canal Unicast: Socket<"+poSocket+">");
			// Se obtiene mensaje
			loInput = poSocket.getInputStream();
			loDatosIn = new DataInputStream(loInput);
			moLog.debug("Canales de entrada abiertos..");
			//Obtenemos tamanyo del mensaje
			liTam = loDatosIn.readInt();
			moLog.debug("Mensaje recibido de Tamanyo<"+liTam+">");
			//Se obtiene el objeto mensaje enviado
			loDatos = new byte[liTam];
			loDatosIn.readFully(loDatos);
			loMensaje = (PFC_Mensaje) PFC_JxtaUtilidades.convertirByteArrayObject(loDatos);
			moLog.info("Mensaje recibido de<"+loMensaje.getPeerOrigen()+"> y Tipo<"+loMensaje.getTipoMensaje()+">");
			System.out.println("**********Mensaje recibido de<"+loMensaje.getPeerOrigen()+"> y Tipo<"+loMensaje.getTipoMensaje()+">");

			// Se inicializa medidor del rendimiento
			PFC_Rendimiento loRendimiento = new PFC_Rendimiento("(Unicast)"+loMensaje.getTipoMensaje());					
			
			// En funciona tipo de mensaje se aplica tratamiento
			if (loMensaje.getTipoMensaje().equals(PFC_Mensaje.TIPO_PETICION_ALTA))
			{
				loRespuesta = gestionarPeticionAlta(loMensaje);
			}
			
			//Calculo del tiempo utilizado en dar de alta un documento.
			loRendimiento.tiempo();											
		}catch(Exception e)
		{
			moLog.error("Problemas al tratar el mensaje Unicast:"+e);
		}
		finally
		{
			try
			{
				loInput.close();
			}catch(Exception e)
			{
				moLog.error("Problemas al cerrar los datos de entrada:"+e.getMessage());
			}
		}
		
		return loRespuesta;
	}//tratarMensaje
	
	/**
	 * <p>Trata la peticion del envio de un documento. El mensaje recibido indica el id del documento a enviar</p>
	 * @param poMensaje
	 * @return
	 * @throws Exception
	 */
	private PFC_Mensaje gestionarPeticionAlta(PFC_Mensaje poMensaje) throws Exception
	{
		String lsIdDoc = poMensaje.getIdDocumento();
		
		moLog.info("Peticion para gestionar el envio del alta de documento<"+lsIdDoc+">");
		
		//Se construye mensaje respuesta 
		PFC_Mensaje loMensaje = new PFC_Mensaje(PFC_Mensaje.TIPO_RESPUESTA_ALTA);
		loMensaje.setPeerOrigen(PFC_Constantes.NODO_ID);
		
		Object lvsDocumento[] = moGestionBD.obtenerDocumento(lsIdDoc);
		// Se establece el contenido del documento
		String lsNomFisico = PFC_Constantes.componeNomDocFisico((String)lvsDocumento[0], PFC_Constantes.NODO_ID, (String)lvsDocumento[1]);
		loMensaje.setContenido(PFC_ManejoFicheros.getFicheroByteArray(lsNomFisico));
		// Se establece el nombre
		loMensaje.setNombreDocumento((String)lvsDocumento[1]);
		// Se establece la fecha
		loMensaje.setFechaDocumento(new Date((Long)lvsDocumento[2]));
		// Se establece identificador del documento
		loMensaje.setIdDocumento(lsIdDoc);
		// Se establece propietario
		loMensaje.setPropietario((String)lvsDocumento[3]);		
		
		moLog.info("Gestion del envio del alta de documento<"+lsIdDoc+"> REALIZADA");
		
		return loMensaje;
	}//gestionarPeticionAlta
	
	/**
	 * <p>Establece la variable que indica si tenemos que seguir con el proceso a false</p>
	 */
	public void terminarProceso()
	{
		this.mbSeguir = false;
	}
	
}
