package pfc.prototipo.jxta.peer;

import java.net.DatagramPacket;
import java.sql.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;

import pfc.prototipo.PFC_Constantes;
import pfc.prototipo.PFC_ManejoFicheros;
import pfc.prototipo.bbdd.PFC_BDGestion;
import pfc.prototipo.jxta.PFC_JxtaUtilidades;
import pfc.prototipo.jxta.PFC_Mensaje;
import pfc.prototipo.jxta.superpeer.PFC_SuperPeer;
import pfc.prototipo.rendimiento.PFC_Rendimiento;
import pfc.prototipo.xml.PFC_GestorXML;

import net.jxta.socket.JxtaMulticastSocket;

public class PFC_PGestorComunicacionMC  implements Runnable 
{
	//Trazas
	private final static Logger moLog = Logger.getLogger(PFC_PGestorComunicacionMC.class);

	//Canal de comunicacion Multicast
	private JxtaMulticastSocket moSocket = null;
	
	// Indica cuando acaba el proceso
	private boolean mbSeguir = true;
	
    //Conexion con la base de datos
    private PFC_BDGestion moGestionBD = null;

	
	public PFC_PGestorComunicacionMC(JxtaMulticastSocket poSocket, PFC_BDGestion poGestionBD)
	{
		moSocket = poSocket;
		moGestionBD = poGestionBD;
	}//PFC_PGestorComunicacionMC
	
	/**
	 * <p>Recibe mensaje y los procesa</p>
	 */
	public void run() 
	{
		byte[] loBuffer = null; 
				
		try
		{
			moLog.info("Se inicia proceso de escucha de mensajes por el canal MultiCast");

			while (mbSeguir)
			{							
				loBuffer = new byte[64*1024];
				DatagramPacket loDatos2 = new DatagramPacket(loBuffer, loBuffer.length);
				
				moLog.info("Canal Multicast del Peer<"+PFC_Constantes.NODO_NOMBRE+"> se queda a la espera de recibir mensajes");
				System.out.println("*****************Canal Multicast del Peer<"+PFC_Constantes.NODO_NOMBRE+"> se queda a la espera de recibir mensajes");
				
				moSocket.receive(loDatos2);
				
				moLog.info("Canal Multicast del Peer<"+PFC_Constantes.NODO_NOMBRE+"> ha recibido un mensaje..Pasa a su tratamiento");
				System.out.println("***********Canal Multicast del Peer<"+PFC_Constantes.NODO_NOMBRE+"> ha recibido un mensaje");
				
				
				//Tratamos el mensaje recibido
				this.tratarMensaje(loDatos2.getData());
				
				
			}//while
			
		}catch(Exception e)
		{
			moLog.error("Problemas al recibir mensajes del tipo Multicast:"+e.getMessage());
		}
		
	}//run

	/**
	 * <p>Centraliza todos los mensajes y en funcion del tipo realiza las tareas oportunas</p>
	 * @param poDatos
	 */
	private void tratarMensaje(byte[] poDatos)
	{
		//Mensaje recibido
		PFC_Mensaje loMensaje = null;
		boolean lbTratarOk = false;
		
		try
		{
			// Obtenemos el contenido del mensaje
			loMensaje = (PFC_Mensaje) PFC_JxtaUtilidades.convertirByteArrayObject(poDatos);		
			
			trataTipologiaMensaje(loMensaje);
			
		}catch(Exception e)		
		{
			moLog.error("Problemas al tratar el mensaje:"+poDatos);
		}
	}//tratarMensaje

	/**
	 * <p>Gestiona las diferentes tipologias de mensajes</p>
	 * @param poMensaje
	 * @throws Exception
	 */
	private void trataTipologiaMensaje(PFC_Mensaje poMensaje) throws Exception
	{
		boolean lbTratarOk = false;

		// Se obtiene el tipo de mensaje y Peer Origen
		String lsTipoMensaje = poMensaje.getTipoMensaje();
		String lsPeerOrigen = poMensaje.getPeerOrigen();
		
		// Se verifica si el mensaje recibido ha sido enviado por el mismo Peer. Si es as, se descarta
		if (!lsPeerOrigen.equals(PFC_Constantes.NODO_ID))
		{			
			moLog.info("Mensaje recibio de<"+lsPeerOrigen+"> del Tipo<"+lsTipoMensaje+">");
			System.out.println("***********Mensaje recibio de<"+lsPeerOrigen+"> del Tipo<"+lsTipoMensaje+">");

			// Se inicializa medidor del rendimiento
			PFC_Rendimiento loRendimiento = new PFC_Rendimiento("(Multicast)"+lsTipoMensaje);		
						
			if (lsTipoMensaje == null)
				throw new Exception("Tipo de mensaje no informado. NULL");
		
			if (lsTipoMensaje.equals(PFC_Mensaje.TIPO_NUEVO))
			{
				PFC_PeerComun.gestionarAltaDocumento(poMensaje, moGestionBD);
				lbTratarOk = true;
			}
			if (lsTipoMensaje.equals(PFC_Mensaje.TIPO_BAJA))
			{
				PFC_PeerComun.gestionarBajaDocumento(poMensaje, moGestionBD);
				lbTratarOk = true;
			}
			if (lsTipoMensaje.equals(PFC_Mensaje.TIPO_CAMBIO))
			{
				PFC_PeerComun.gestionarModificacionDocumento(poMensaje, moGestionBD);
				lbTratarOk = true;
			}
			if (lsTipoMensaje.equals(PFC_Mensaje.TIPO_ENVIO_ACTUALIZACIONES))
			{
				gestionarActualizaciones(poMensaje);
				lbTratarOk = true;
			}			
							
			if (!lbTratarOk)
			{
				moLog.info("Tpo de mensaje desconocido<"+lsTipoMensaje+">");
			}
			
			//Calculo del tiempo utilizado en dar de alta un documento.
			loRendimiento.tiempo();								
		}
		else
		{
			moLog.info("Se decarta mensaje ya que ha sido enviado por el mismo nodo.");
			System.out.println("**********Se decarta mensaje ya que ha sido enviado por el mismo nodo.");
		}
		
	}//trataTipologiaMensaje
	
	/**
	 * <p>Gestiona las actualizaciones enviadas en bloque por un Peer.</p>
	 * <p>Actualiza la base de datos con cada uno de los cambios realizados</p>
	 * @param poMensaje
	 * @throws Exception
	 */
	private void gestionarActualizaciones(PFC_Mensaje poMensaje) throws Exception
	{
		PFC_Mensaje loMensaje = null;
		
		moLog.info("Se inicia proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+">");
		// Se obtienen todos los cambios indicados por el peer origen			
		Vector loActualizaciones = poMensaje.getActualizaciones();
		int liCambios = (loActualizaciones != null ? loActualizaciones.size() : 0);
		moLog.info("Numero de cambios indicados<"+liCambios+">");
		
		for (int i=0; i < liCambios; i++)
		{
			loMensaje = (PFC_Mensaje) loActualizaciones.elementAt(i);
			
			// Se trata tipologia de actualizaciones
			trataTipologiaMensaje(loMensaje);
			
			
		}//for (int i=0; i < loActualizaciones.size(); i++)
		
		
		moLog.info("Proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+"> REALIZADO!");
		
	}//gestionarBajaDocumento
	
	
	
	/**
	 * <p>Establece la variable que indica si tenemos que seguir con el proceso a false</p>
	 */
	public void terminarProceso()
	{
		this.mbSeguir = false;
	}
	
}
