package pfc.prototipo.jxta.peer;

import java.io.DataInput;
import java.io.DataInputStream;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.DatagramPacket;
import java.net.URI;
import java.util.Enumeration;
import java.util.Vector;

import org.apache.log4j.Logger;

import pfc.prototipo.PFC_Constantes;
import pfc.prototipo.PFC_PanelConectarConexion;
import pfc.prototipo.PFC_PanelConectarConexion.PFC_ConectarAccion;

import pfc.prototipo.bbdd.PFC_BDGestion;
import pfc.prototipo.jxta.PFC_JxtaUtilidades;
import pfc.prototipo.jxta.PFC_Mensaje;
import pfc.prototipo.rendimiento.PFC_Rendimiento;
import net.jxta.credential.AuthenticationCredential;
import net.jxta.credential.Credential;
import net.jxta.discovery.DiscoveryService;
import net.jxta.document.Advertisement;
import net.jxta.document.StructuredDocument;
import net.jxta.id.ID;
import net.jxta.id.IDFactory;
import net.jxta.membership.Authenticator;
import net.jxta.membership.MembershipService;
import net.jxta.peer.PeerID;
import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupID;
import net.jxta.pipe.PipeService;
import net.jxta.platform.NetworkConfigurator;
import net.jxta.platform.NetworkManager;
import net.jxta.protocol.PeerGroupAdvertisement;
import net.jxta.protocol.PipeAdvertisement;
import net.jxta.socket.JxtaMulticastSocket;
import net.jxta.socket.JxtaServerSocket;
import net.jxta.socket.JxtaSocket;

public class PFC_Peer 
{
	//Trazas
	private final static Logger moLog = Logger.getLogger(PFC_Peer.class);

	//Red
	private NetworkManager moRed = null;
	//Servicio del Peer
	private DiscoveryService moSrvPeer = null;
	//Servicio de Grupo
	private DiscoveryService moSrvGrupoPeer = null;

	//Grupo de Peers
	private PeerGroup moGrupo = null;

	// Canal de comunicacion Multicast
	private JxtaMulticastSocket moSocketMC = null;

	//Gestion los mensajes de entrada por el canal Multicast
	private PFC_PGestorComunicacionMC  moGestioComunicacionMC = null;
	
	//Gestion los mensajes de entrada por el canal Unicast
	private PFC_PGestorComunicacion  moGestioComunicacion = null;

	// Canal de comunicaion Unicast
	private JxtaServerSocket moSocket = null;
	private PipeAdvertisement moAdvPipe = null;	

    //Conexion con la base de datos
    private PFC_BDGestion moGestionBD = null;

	
	public PFC_Peer(PFC_BDGestion poGestionBD, pfc.prototipo.PFC_PanelConectarConexion.PFC_ConectarAccion poPanel) throws Exception
	{		
		NetworkConfigurator loRedConfig;
		File loConfigLocal = null; 
		PeerGroup loRedGrupo = null;
		PeerGroup loTrabajoGrupo = null;
		Enumeration<Advertisement> loAnuncios = null;
		PeerGroupAdvertisement loAdGrupo = null;
		
		
		// Se inicializa objeto de conexion con base de datps
		moGestionBD = poGestionBD;
		
		// Indicador avance del proceso
		poPanel.cambiarProgreso(5);
		
		try 
		{
			moLog.info("Se inicia conexion a la red con el Peer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			//Iniciamos conexion a la red que se generara. En primer lugar se indica el tipo de Peer que se quiere. En segundo el nombre
			//de la red que se crea, en tercer lugar se genera un fichero de configuracion con el nombre del Peer
			moRed = new NetworkManager(NetworkManager.ConfigMode.EDGE,
					PFC_Constantes.GRUPO_TRABAJO_NOMBRE, new File(new File(".cache"), PFC_Constantes.NODO_NOMBRE).toURI());
			
		} catch (Exception e) 
		{
			moLog.error("No se puede crear la conexion con el Peer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">:"+e.getMessage());
			System.out.println("No se puede crear la conexion con el Peer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			throw new Exception(PFC_Constantes.ERROR_CONEXION_PEER);
		}		

		// Indicador avance del proceso
		poPanel.cambiarProgreso(15);
		
		//Hacemos persistente el ID de Peer
		moRed.setConfigPersistent(true);
		

		//Gestin de la configuracin
		try 
		{
			//Miro si existe configuracin local. En caso de existir, obtengo la configuracin del disco. En caso de no tenerla,
			//se crea y se guarda
			loRedConfig = moRed.getConfigurator();
		
			if (loRedConfig.exists() && !PFC_Constantes.CAMBIO_CONFIGURACION)
			{				
				moLog.info("Peer<"+PFC_Constantes.NODO_NOMBRE+"> con configuracin local. Se inicia proceso de carga");
				// Se obtiene configuracin local y se carga
				loConfigLocal = new File(loRedConfig.getHome(), "PlatformConfig");
				
				loRedConfig.load(loConfigLocal.toURI());
				moLog.info("Peer<"+ PFC_Constantes.NODO_NOMBRE+"> con configuracin local. Carga realizada!");
				
			}else
			{
				//No se dispone de configuracion local o se tiene que volver a crear por algun cambio en configuracion				
				moLog.info("Peer<"+PFC_Constantes.NODO_NOMBRE+"> sin configuracin local. Se crea nueva configuracion local");
							
				//Se establece nombre del peer
				loRedConfig.setName(PFC_Constantes.NODO_NOMBRE);
				loRedConfig.setPeerId(PFC_Constantes.NODO_ID);

				//Se indica cual sera el SuperPeer
				loRedConfig.clearRendezvousSeeds();
				loRedConfig.addSeedRendezvous(URI.create("tcp://" + PFC_Constantes.SUPERPEER_IP + ":" + PFC_Constantes.SUPERPEER_PUERTO));								
				//Se graba configuracion
				loRedConfig.save();
				
				// Se establece cualquier cambio aplicado
				moGestionBD.modificarCambio("N");
				PFC_Constantes.CAMBIO_CONFIGURACION = false;
				
				moLog.info("Peer<"+PFC_Constantes.NODO_NOMBRE+"> sin configuracin local. Configuracion local CREADA!");
			}
					
		} catch (Exception e) 
		{
			moLog.error("No se puede crear cargar la configuracion para realizar la conexion con el Peer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">:"+e.getMessage());
			System.out.println("No se puede crear cargar la configuracion para realizar la conexion con el Peer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			throw new Exception(PFC_Constantes.ERROR_CARGA_CONFIGURACION_PEER);			
		} 

		// Indicador avance del proceso
		poPanel.cambiarProgreso(25);		
		
		try 
		{
			//Se inicia red			
			loRedGrupo = moRed.startNetwork();			
			moLog.info("Conexion a la red con el Peer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> REALIZADA!");
			
			//Se obtiene servicio de grupo
			moSrvPeer = loRedGrupo.getDiscoveryService();
			
		} catch (Exception e) 
		{
			moLog.error("No se puede realizar Start en la conexion con el Peer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">:"+e.getMessage());
			System.out.println("No se puede realizar Start en la conexion con el Peer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			throw new Exception(PFC_Constantes.ERROR_INI_RED_PEER);			
		} 
		
		// Indicador avance del proceso
		poPanel.cambiarProgreso(35);
		
		//Una vez conectados y definido configuraciones propias, esperamos conexion con el mismo. 
		//Se configura la espera a 0 para indicar que esperaremos hasta obtener conexion
		System.out.println("*************** A la espera de conectarno con SuperPeer.........");
		moLog.info("Conectando con SuperPeer.......................");
		boolean lbConectado = moRed.waitForRendezvousConnection(PFC_Constantes.TIMEOUT_ESPERA_SUPERPEER);
		
		if (lbConectado)
		{
			System.out.println("*************** Conexion con SuperPeer realizada!!!!");
			moLog.info("Conexin con SuperPeer REALIZADA!");			
		}
		else
		{
			System.out.println("*************** NO se ha podido establecer conexion con SuperPeer!!!!");
			moLog.error("NO se ha podido establecer conexion con SuperPeer!!!!");
			this.pararServicio();
			throw new Exception(PFC_Constantes.ERROR_NO_CONEXION_SUPERPEER);
		}
		
		// Indicador avance del proceso
		poPanel.cambiarProgreso(45);		
		
		//Gestin del grupo 
		try 
		{
			moLog.info("Inicio proceso para adherirse al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			
			//En primer lugar se verifica si existe o no grupo creado. Para eso, vemos la cache(recursos locales)
			loAnuncios = moSrvPeer.getLocalAdvertisements(DiscoveryService.GROUP, "Name", PFC_Constantes.GRUPO_TRABAJO_NOMBRE);

			if ((loAnuncios != null) && loAnuncios.hasMoreElements()) 
			{
				moLog.debug("Grupo <"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> existe en Local!");
				//Como el grupo ya existe, se obtiene del anuncio el grupo para luego poder unirse al grupo
				loAdGrupo = (PeerGroupAdvertisement) loAnuncios.nextElement();
				loTrabajoGrupo = loRedGrupo.newGroup(loAdGrupo);
				//Se une al grupo
				unirseAlGrupo(loTrabajoGrupo);
				setGrupo(loTrabajoGrupo);
				//Una vez se ha unido al grupo, publicamos esa informacion
				moSrvGrupoPeer = loTrabajoGrupo.getDiscoveryService();
				moSrvGrupoPeer.publish(loTrabajoGrupo.getPeerAdvertisement());
				moSrvGrupoPeer.remotePublish(loTrabajoGrupo.getPeerAdvertisement());
				
			}
			else
			{
				moLog.debug("Grupo <"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> NO existe en Local!");
				//Se busca grupo de forma remota
				moSrvPeer.getRemoteAdvertisements(null, DiscoveryService.GROUP, "Name", PFC_Constantes.GRUPO_TRABAJO_NOMBRE, 1);
				
				//Verificamos que la busqueda de Grupo se ha realizado de forma correcta. 
				esperaMientrasGrupo(loRedGrupo);			
			}

			moLog.info("Proceso para adherirse al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> REALIZADO!");
			
		} catch (Exception e) 
		{
			moLog.error("No se puede adherir al Grupo con el Peer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">:"+e.getMessage());
			System.out.println("No se puede adherir al Grupo con el Peer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			throw new Exception(PFC_Constantes.ERROR_NO_CONEXION_GRUPO);		
		} 					
				
		// Indicador avance del proceso
		poPanel.cambiarProgreso(55);
		
	}//PFC_Peer
	
	/**
	 * <p>Proceso que permite unirse al grupo.</p>
	 * @param poGrupo
	 */
	private void esperaMientrasGrupo(PeerGroup poGrupo) throws Exception
	{
		int liItera = 2;
		long llTiempoEspera = 30 * 1 * 1000L;
		PeerGroup loTrabajoGrupo = null;
		Enumeration<Advertisement> loAnuncios = null;
		PeerGroupAdvertisement loAdGrupo = null;
		boolean lbAdhesion = false;
		
		moLog.info("Se inicia proceso de Espera para Unirse al Grupo");
		
		//Se itera esperando a la adhesion al grupo
		for (int i=0; i < liItera; i++)
		{
			moLog.debug("Intento<"+i+"> de <"+liItera+"> intentos para unirse al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");

			moLog.debug("Duermo por :" + (llTiempoEspera/60000)+" minutos a la espera de la adhesion al grupo");
			Thread.sleep(llTiempoEspera);
			liItera++;
			
			loAnuncios = moSrvPeer.getLocalAdvertisements(DiscoveryService.GROUP, "Name", PFC_Constantes.GRUPO_TRABAJO_NOMBRE);

			if ((loAnuncios != null) && loAnuncios.hasMoreElements()) 
			{				
				moLog.debug("Grupo <"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> existe en Local!");
				//Como el grupo ya existe, se obtiene del anuncio el grupo para luego poder unirse al grupo
				loAdGrupo = (PeerGroupAdvertisement) loAnuncios.nextElement();
				loTrabajoGrupo = poGrupo.newGroup(loAdGrupo);
				//Se une al grupo
				unirseAlGrupo(loTrabajoGrupo);
				setGrupo(loTrabajoGrupo);
				//Una vez se ha unido al grupo, publicamos esa informacion
				moSrvGrupoPeer = loTrabajoGrupo.getDiscoveryService();
				moSrvGrupoPeer.publish(loTrabajoGrupo.getPeerAdvertisement());
				moSrvGrupoPeer.remotePublish(loTrabajoGrupo.getPeerAdvertisement());
				lbAdhesion = true;
				break;
			}//if ((loAnuncios != null) && loAnuncios.hasMoreElements())
			
			
		}//for
		
		if (lbAdhesion)
		{
			moLog.info("Union al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> correcta!");
		}
		else
		{
			moLog.info("Union al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> incorrecta!");
			throw new Exception("No se ha podido realizar la union al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
		}
		
	}//esperaMientrasGrupo
	
	
	/**
	 * <p>Permite realizar la union a un grupo de nodos</p>
	 * @param grp
	 */
	private void unirseAlGrupo(PeerGroup grp) throws Exception
	{
		StructuredDocument creds = null;

		moLog.info("Se inicia proceso de union al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
		// credenciales para el Grupo
		AuthenticationCredential authCred =	new AuthenticationCredential( grp, null, creds );

		// MembershipService para el grupo
		MembershipService membership = grp.getMembershipService();

		// Se obtiene Authenticator para el Authentication creds
		Authenticator auth = membership.apply(authCred);

		// Si todo esta bien, se une al grupo
		if (auth.isReadyForJoin())
		{
			//Se une este peer al grupo
			Credential myCred = membership.join(auth);

			moLog.debug("Union al grupo correcta. Nombre del Grupo<" + grp.getPeerGroupName()+"> ID del Grupo<"+grp.getPeerGroupID()+">");
			
			// A partir de aqui se muestran los credenciales, es solo para mostrar info por log, no hace falta
			/*System.out.println("\nCredential: ");
			StructuredTextDocument doc = (StructuredTextDocument)
			myCred.getDocument(new MimeMediaType("text/plain"));

			StringWriter out = new StringWriter();
			doc.sendToWriter(out);
			System.out.println(out.toString());
			out.close();*/
		}
		else
		{
			moLog.info("No nos hemos podido unir al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			throw new Exception("No nos hemos podido unir al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
		}
		
		moLog.info("Proceso de union al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> REALIZADO!");
	}//unirseAlGrupo

	
	/**
	 * <p>Inicia los servicios del SuperPeer</p>
	 */
	public void ejecutaServicio()
	{
		PipeAdvertisement loAdvPipeMC = null;
		

		//Se crea canal de comunicacion Multicast y se inicia servicio de escucha
		try
		{
			moLog.debug("Se inicia proceso de creacin de canal de comunicacin Multicast");
			loAdvPipeMC = PFC_JxtaUtilidades.generarAdvertisement(PFC_Constantes.PIPE_MULTICAST_NOMBRE, PFC_Constantes.PIPE_MULTICAST_ID, getGrupo().getPeerGroupID(), PipeService.PropagateType);
			moSocketMC = new JxtaMulticastSocket(getGrupo(), loAdvPipeMC);
			moSocketMC.setSoTimeout(0);
			moLog.debug("Proceso de creacin de canal de comunicacin Multicast REALIZADO!");
			
			moGestioComunicacionMC = new PFC_PGestorComunicacionMC(moSocketMC, moGestionBD);
			Thread loThread = new Thread(moGestioComunicacionMC);
			loThread.start();
			
		}catch(Exception e)
		{
			moLog.error("No se puede iniciar canal de comunicacion Multicast:"+e.getMessage());
		}

		
		//Se crea canal de comunicacion Unicast y se inicia servicio de escucha
		try
		{
			moLog.debug("Se inicia proceso de creacin del Pipe del canal de comunicacin Unicast");
			moAdvPipe = PFC_JxtaUtilidades.generarAdvertisement(PFC_Constantes.PIPE_UNICAST_NOMBRE, PFC_Constantes.PIPE_UNICAST_ID, getGrupo().getPeerGroupID(), PipeService.UnicastType);
			moSocket = new JxtaServerSocket(getGrupo(), moAdvPipe);			
			moSocket.setSoTimeout(0);
			moLog.debug("Proceso de creacin de canal de comunicacin Unicast REALIZADO!");

			moGestioComunicacion = new PFC_PGestorComunicacion(moSocket, moGestionBD);
			Thread loThread = new Thread(moGestioComunicacion);
			loThread.start();
		
		}catch(Exception e)
		{
			moLog.error("No se puede iniciar canal de comunicacion Unicast:"+e.getMessage());
		}
			

	}//ejecutaServicio

	/**
	 * <p>Envia un mensaje al Grupo de Trabajo.</p>
	 */
	public void enviarMensajeGrupo(PFC_Mensaje poMensaje) throws Exception
	{
		DatagramPacket loDatos = null;
		int liTam = -1;
		
		moLog.info("Peticin de envo de mensaje al grupo desde el Peer<"+PFC_Constantes.NODO_NOMBRE+"> y el objeto mensaje<"+poMensaje+">");			
						
		//Se obtiene el mensaje en el formato correcto para poder ser enviado
		byte[] byteVal = PFC_JxtaUtilidades.convertirObjectByteArray(poMensaje);			
		liTam = byteVal.length;			
	    loDatos = new DatagramPacket(byteVal, liTam);	         		  
		moLog.debug("Se envia mensaje al grupo desde el Peer<"+PFC_Constantes.NODO_NOMBRE+"> con un tamao de<"+liTam+">");
		System.out.println("*************Se envia mensaje al grupo desde el Peer<"+PFC_Constantes.NODO_NOMBRE+"> con un tamao de<"+liTam+">");
		moSocketMC.send(loDatos);
		System.out.println("*********** Envo de mensaje al grupo desde el Peer<"+PFC_Constantes.NODO_NOMBRE+"> REALIZADO");			
		moLog.info("Envo de mensaje al grupo desde el Peer<"+PFC_Constantes.NODO_NOMBRE+"> REALIZADO");
			
	}//enviarMensajeGrupo
	
	public void enviarMensaje(PFC_Mensaje poMensaje, PeerID poDestino) throws Exception
	{
		PeerID loDestino = null;
		OutputStream loSalida = null; 
		InputStream loInput = null;
		DataOutput loDatosSalida = null;
		DataInput loDatosIn = null; 
		byte[] loDatos = null;
		int liTamano;
		JxtaSocket loSocket = null;
		PFC_Mensaje loMensaje = null;
		
		try
		{
			moLog.info("Peticion de envio de mensaje Unicast al destinatario<"+poDestino+">");
			// Se verifica el parametro poDestino ya que si su valor es null, significa que el destinatario es el SuperPeer
			loDestino = poDestino;
			if (poDestino == null)
			{
				moLog.info("Destinatario es SuperPeer");
				loDestino =  (PeerID)getURISuperPeer(1);
			}


			// Se abre socket de conexion
			moLog.debug("Se inicia apertura de Socket para enviar mensaje a destino<"+loDestino+">");
						
			loSocket = new JxtaSocket(getGrupo(),
				//Indica el Peer destinatario, null si no especifico
				loDestino,
				moAdvPipe,
				//Timeout de conexion 
				PFC_Constantes.TIMEOUT_SOCKET_UNICAST,
				// Conexion segura (confianza)
				true);
			moLog.debug("Socket ABIERTO!");

			// Flujo de salida de datos
			loSalida = loSocket.getOutputStream();
			loDatosSalida = new DataOutputStream(loSalida);
					
			loDatos = PFC_JxtaUtilidades.convertirObjectByteArray(poMensaje);			
			liTamano = loDatos.length;
			
			moLog.info("Se va a enviar mensaje con Tamno<"+liTamano+">");
			System.out.println("Se va a enviar mensaje con Tamno<"+liTamano+">");
			
			loDatosSalida.writeInt(liTamano);
			loSalida.write(loDatos);
			loSalida.flush();
			loSalida.close();
			moLog.info("Mensaje con Tamao<"+liTamano+"> ENVIADO");
			System.out.println("Mensaje con Tamao<"+liTamano+"> ENVIADO");

			// Se obtiene mensaje
			loInput = loSocket.getInputStream();
			loDatosIn = new DataInputStream(loInput);
			moLog.debug("Canales de entrada abiertos..");
			//Obtenemos tamanyo del mensaje
			liTamano = loDatosIn.readInt();
			moLog.debug("Mensaje recibido de Tamanyo<"+liTamano+">");
			//Se obtiene el objeto mensaje enviado
			loDatos = new byte[liTamano];
			loDatosIn.readFully(loDatos);
			loMensaje = (PFC_Mensaje) PFC_JxtaUtilidades.convertirByteArrayObject(loDatos);
			moLog.info("Mensaje recibido de<"+loMensaje.getPeerOrigen()+"> y Tipo<"+loMensaje.getTipoMensaje()+">");
			System.out.println("**********Mensaje recibido de<"+loMensaje.getPeerOrigen()+"> y Tipo<"+loMensaje.getTipoMensaje()+">");
			loInput.close();
			loSocket.close();
			
			// Se inicia tratamiento mensaje respuesta
			tratarRespuesta(loMensaje);
			
		} catch (Exception e) 
		{
			moLog.error("Problemas al enviar mensaje Unicast:"+e);
			throw new Exception("Problemas al enviar mensaje Unicast.");
		}
						
	}//enviaMensaje
	
	private void tratarRespuesta(PFC_Mensaje poMensaje) throws Exception
	{
		// Informacion de Altas recibidas.
		Vector loAltas = null;
		
		
		moLog.info("Peticion para tratar mensaje tipo<"+poMensaje.getTipoMensaje()+">");
		// Se inicializa medidor del rendimiento
		PFC_Rendimiento loRendimiento = new PFC_Rendimiento("(Unicast)"+poMensaje.getTipoMensaje());					
		
		// En funcion de la tipologia del mensaje se hace una u otra cosa
		if (poMensaje.getTipoMensaje().equals(PFC_Mensaje.TIPO_ACTUALIZACION))
		{
			moLog.info("Se inicia proceso de actualizacion de documentos. Numero de actualizaciones a realizar<"+(poMensaje.getActualizaciones() == null ? 0 : poMensaje.getActualizaciones().size())+">");
			System.out.println("***********Se inicia proceso de actualizacion de documentos. Numero de actualizaciones a realizar<"+(poMensaje.getActualizaciones() == null ? 0 : poMensaje.getActualizaciones().size())+">");
			
			// Se recorren todas las actualizaciones y se van haciendo las adaptaciones indicadas
			Vector loActualizaciones = poMensaje.getActualizaciones();
			if ( (loActualizaciones != null) && (loActualizaciones.size() > 0) )
			{
				
				for (int i=0; i < loActualizaciones.size(); i++)
				{
					if ( ((PFC_Mensaje)loActualizaciones.elementAt(i)).getTipoMensaje().equals(PFC_Mensaje.TIPO_BAJA))
					{
						PFC_PeerComun.gestionarBajaDocumento((PFC_Mensaje)loActualizaciones.elementAt(i) , moGestionBD);
					}
					if ( ((PFC_Mensaje)loActualizaciones.elementAt(i)).getTipoMensaje().equals(PFC_Mensaje.TIPO_AVISO_ALTA))
					{
						// Se obtiene la informacion del nuevo documento. Id documento y propietario
						String[] lvsDatos = new String[2];
						lvsDatos[0] = ((PFC_Mensaje)loActualizaciones.elementAt(i)).getIdDocumento();
						lvsDatos[1] = ((PFC_Mensaje)loActualizaciones.elementAt(i)).getPeerOrigen();
						moLog.info("Detectado nueva alta del documento con id<"+lvsDatos[0]+"> del propietario<"+lvsDatos[1]+">");
						//Se aade al vector de altas
						if (loAltas == null)
							loAltas = new Vector();
						
						loAltas.add(lvsDatos);
					}
					if ( ((PFC_Mensaje)loActualizaciones.elementAt(i)).getTipoMensaje().equals(PFC_Mensaje.TIPO_CAMBIO))
					{
						PFC_PeerComun.gestionarModificacionDocumento((PFC_Mensaje)loActualizaciones.elementAt(i) , moGestionBD);
					}
				}//for (int i=0; i < loActualizaciones.size(); i++)
				
				
				// Una vez se han procesado todas las actualizaciones que el Superpeer ha informado, se ejecutan aquellas que hayan podido ser de alta
				moLog.info("Numero de altas a procesar<"+(loAltas != null ? loAltas.size() : 0)+">");
				
				// Se verifica y realizan las altas que hayan podido ser informadas
				realizarAltas(loAltas);
				
			}//if (loActualizaciones != null)						
		}//if (poMensaje.getTipoMensaje().equals(PFC_Mensaje.TIPO_ACTUALIZACION))
		
		if (poMensaje.getTipoMensaje().equals(PFC_Mensaje.TIPO_RESPUESTA_ALTA))
		{
			moLog.info("Se inicia proceso de alta de un nuevo documento. ");			
			PFC_PeerComun.gestionarAltaDocumento(poMensaje, moGestionBD);
			moLog.info("Proceso de alta de un nuevo documento realizado!");
		}//if (poMensaje.getTipoMensaje().equals(PFC_Mensaje.TIPO_RESPUESTA_ALTA))
		
		//Calculo del tiempo utilizado en dar de alta un documento.
		loRendimiento.tiempo();											
		
		moLog.info("Peticion para tratar mensaje tipo<"+poMensaje.getTipoMensaje()+"> FINALIZADA!");
		
	}//tratarRespuesta

	/**
	 * <p>Trata todas las altas identificadas por el SuperPeer. Se envia mensaje a Peer creador del documento para que nos lo de.
	 * En caso de no estar disponible el peer, se pide el documento al Data Center (SuperPeer)</p>
	 * @param poAltas
	 * @throws Exception
	 */
	private void realizarAltas(Vector poAltas) throws Exception
	{
		
		moLog.info("Peticion para tratar altas de documentos. Numero de altas informadas<"+(poAltas != null ? poAltas.size() : 0)+">");
		
		if (poAltas != null)
		{
			// Para cada una de las altas indicadas se tiene que realizar una peticion al propietario del documento y este nos
			// tiene que enviar el documento
			for (int i = 0; i < poAltas.size(); i++)
			{				
				// Se obntiene la informacion del documento a pedir
				String[] lvsDoc = (String[])poAltas.elementAt(i);
				// Se construye un mensaje del tipo DAME_ALTA donde le indicaremos el iddocumento y sabremos el propietario al que enviarlo				
				PFC_Mensaje loMensaje = new PFC_Mensaje(PFC_Mensaje.TIPO_PETICION_ALTA);
				// Se establece el iddocumento
				loMensaje.setIdDocumento(lvsDoc[0]);
				// Establecemos el id del peer que envia el mensaje
				loMensaje.setPeerOrigen(PFC_Constantes.NODO_ID);
				
				// Se hace la peticion de envio de mensaje
				try
				{
					this.enviarMensaje(loMensaje, (PeerID)IDFactory.fromURI(new URI(lvsDoc[1])));
				}catch(Exception e)
				{
					// Al dar un error al realizar la peticion del documento al Peer creador, se realiza la peticion
					// del documento al Data Center (SuperPeer)
					this.enviarMensaje(loMensaje, null);
				}
				
			}//for (int i = 0; i < poAltas.size(); i++)
		}
		
		moLog.info("Peticion para tratar altas de documentos REALIZADA");
	}//realizarAltas
	
	/**
	 * <p>Obtiene el identificador unico del SuperPeer indicado</p>
	 * <p>Esta preparado para existir mas de un SuperPeer por lo que la variable de entrada indica la posicion del SuperPeer que queremos</p>
	 * @param piPosicionSuperPeer Posicion del SuperPeer que queremos. Si p.ej. = 1, de la posible lista de SuperPeer, obtiene la primera
	 * @return
	 */
	private ID getURISuperPeer(int piPosicionSuperPeer) throws Exception
	{
		//Identificador unico del SuperPeer seleccionado
		String lsURI = "";
		//Listado de SuperPeers disponibles
		Enumeration<ID> loSuperPeers = null;
		//Indice que nos indica porque posicion de la lista vamos
		int liIndice = 1;
		ID loSP = null;
		
		//Obtenemos los SuperPeer disponibles.
		loSuperPeers = moRed.getNetPeerGroup().getRendezVousService().getConnectedRendezVous();
		
		//En caso de no haber SuperPeer, error y abortamos. Sin SuperPeer no esta preparado para trabajar
		if (loSuperPeers == null)
		{
			System.out.println("Error - No disponemos de ningun SuperPeer.");
			System.exit(-1);
		}
		
		//Recorremos SuperPeers disponibles
		while (loSuperPeers.hasMoreElements())
		{
			//Obtenemos SuperPeer
			loSP = loSuperPeers.nextElement();
			
			if (liIndice == piPosicionSuperPeer)
			{
				lsURI =  loSP.toURI().toString();				
				
				break;
			}
		}
		
		if (lsURI.equals(""))
		{
			throw new Exception("No se puede obtener el identificador de SuperPeer");
		}
		
		return loSP;
		
	}//getURISuperPeer
	
	/**
	 * <p>Establece Grupo de Trabajo</p>
	 * @param poGrupo
	 */
	private void setGrupo(PeerGroup poGrupo)
	{
		moGrupo = poGrupo;
	}//setGrupo
	
	/**
	 * <p>Obtiene grupo de trabajo</p>
	 * @return
	 */
	public PeerGroup getGrupo()
	{
		return moGrupo;
	}//getGrupo
	
	/**
	 * <p>Para el servicio de red</p>
	 * @throws Throwable 
	 */
	public void pararServicio()
	{
		moLog.info("Peticion para parar conexion del Peer<"+PFC_Constantes.NODO_NOMBRE+"> del Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");		


		if (moGrupo != null)
		{			
			moLog.info("Se para Grupo.");
			moGrupo.stopApp();
			moGrupo.unref();
			moGrupo = null;
			moLog.info("Grupo PARADO!");
		}
		
		if (moGestioComunicacionMC != null)
		{
			moLog.info("Se para Thread comunicacion Multicast");
			moGestioComunicacionMC.terminarProceso();			
			moGestioComunicacionMC = null;
			moLog.info("Thread comunicacion Multicast PARADO!");
		}
		if (moGestioComunicacion != null)
		{
			moLog.info("Se para Thread comunicacion Uniicast");
			moGestioComunicacion.terminarProceso();
			moGestioComunicacion = null;
			moLog.info("Thread comunicacion Unicast PARADO!");
		}
		if (moSocket != null)
		{			
			try 
			{
				moLog.info("Se cierra Socket Unicast.");
				moSocket.close();
				moLog.info("Socket Unicast CERRADO.");
			} catch (IOException e) 
			{
				moLog.error("Problemas al cerrar el Socket Unicast.", e);
			}
			moSocket = null;
		}
		if (moSocketMC != null)
		{
			moLog.info("Se cierra Socket Multicast.");
			moSocketMC.close();
			moLog.info("Socket Multicast CERRADO.");
			moSocketMC = null;
		}
		if (moRed != null)
		{
			moLog.info("Se cierra NetworkManager");
			moRed.stopNetwork();		
			moRed = null;
			System.gc();
			System.runFinalization();
			System.gc();
			moLog.info("NetworkManager CERRADO!");									
		}
	
		
		
		moLog.info("Conexion del Peer<"+PFC_Constantes.NODO_NOMBRE+"> del Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> PARADA!");
	}//stopServicio

}
