package pfc.prototipo.jxta.peer;

import java.sql.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;

import pfc.prototipo.PFC_Constantes;
import pfc.prototipo.PFC_ManejoFicheros;
import pfc.prototipo.bbdd.PFC_BDGestion;
import pfc.prototipo.jxta.PFC_Mensaje;
import pfc.prototipo.xml.PFC_GestorXML;

public class PFC_PeerComun 
{
	//Trazas
	private final static Logger moLog = Logger.getLogger(PFC_PeerComun.class);

	/**
	 * <p>Gestiona la baja de un documento.</p>
	 * <p>Elimina referencia del Documento y lo elimina del disco</p>
	 * @param poMensaje
	 * @throws Exception
	 */
	public static void gestionarBajaDocumento(PFC_Mensaje poMensaje, PFC_BDGestion poGestionBD) throws Exception
	{
		
		moLog.info("Se inicia proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+">");
				
		// Se obtiene la informacion de la ubicacion y nombre del fichero
		Object[] loDatos = poGestionBD.obtenerDocumento(poMensaje.getIdDocumento());
		// Se establece el contenido del documento
		String lsNomFisico = PFC_Constantes.componeNomDocFisico((String)loDatos[0], poMensaje.getPeerOrigen(), (String)loDatos[1]);
		
		// Se da de baja la referencia del documento
		poGestionBD.eliminarDocumento(poMensaje.getIdDocumento());
		
		//Se elimina documento del disco		
		PFC_ManejoFicheros.borrarFichero(lsNomFisico);
		
		
		moLog.info("Proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+"> REALIZADO!");
		
	}//gestionarBajaDocumento
	
	/**
	 * <p>Gestiona la alta de un nuevo documento.</p>
	 * <p>Se tiene que guardar en disco el documento enviado</p>
	 * @param poMensaje
	 * @throws Exception
	 */
	public static void gestionarAltaDocumento(PFC_Mensaje poMensaje, PFC_BDGestion poGestionBD) throws Exception
	{
		byte[] lvbContenido = null;
		
		moLog.info("Se inicia proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+">");
		
		// Al tratarse de un mensaje del tipo nuevo significa que tenemos que obtener el fichero completo y guardarlo en disco
		// El nombre del documento estara compuesto por la ruta establecida por configuracion, el nombre del nodo origen y el nombre del documento indicado		
		String lsNombreDocumento = PFC_Constantes.componeNomDocFisico(PFC_Constantes.DIRECTORIO, poMensaje.getPropietario(), poMensaje.getNombreDocumento());
		
		PFC_ManejoFicheros.getByteArrayFile(lsNombreDocumento, poMensaje.getContenido());
		
		// Guarda la referencia del documento en base de datos		
		poGestionBD.guardarDocumento(poMensaje.getIdDocumento(), poMensaje.getPropietario(), 
				poMensaje.getNombreDocumento(), PFC_Constantes.DIRECTORIO, poMensaje.getFechaDocumento());
	}//gestionarAltaDocumento

	/**
	 * <p>Gestiona los cambios realizados en un documento.</p>
	 * <p>Actualiza el documento con cada uno de los cambios.</p>
	 * @param poMensaje
	 * @throws Exception
	 */
	public static void gestionarModificacionDocumento(PFC_Mensaje poMensaje, PFC_BDGestion poGestionBD) throws Exception
	{
		
		moLog.info("Se inicia proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+">");

		// Se obtiene la informacion de la ubicacion y nombre del fichero
		Object[] loDatos = poGestionBD.obtenerDocumento(poMensaje.getIdDocumento());
		// Se establece el contenido del documento
		String lsNomFisico = PFC_Constantes.componeNomDocFisico((String)loDatos[0], poMensaje.getPeerOrigen(), (String)loDatos[1]);
		// Se guardan los cambios en el disco.
		grabarCambiosRecibidos(lsNomFisico, poMensaje.getCambio());
		
		// Se actualiza la fecha del documento
		poGestionBD.modificarDocumento(poMensaje.getIdDocumento(), poMensaje.getPeerOrigen(), poMensaje.getFechaDocumento());
		
		moLog.info("Proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+"> REALIZADO!");
		
	}//gestionarBajaDocumento

	
	/**
	 * <p>Graba los cambios recibidos en el documento indicado</p>
	 * @param psNomFichero Nombre del fichero donde guardar los cambio
	 * @param poCambios Listado de cambios
	 * @throws Exception
	 */
	public static void grabarCambiosRecibidos(String psNomFichero, Vector poCambios) throws Exception
	{
			
		moLog.debug("Peticion para grabar en documentos cambios. Numero de cambios detectados<"+(poCambios == null ? 0 : poCambios.size())+">");

		for (int i = 0; i < poCambios.size(); i++)		
		{
			String[] lvsCambio = (String[])poCambios.get(i);
			System.out.println("Nodo<"+lvsCambio[0]+">Tipo de cambio<"+lvsCambio[1]+">Valor:\n"+lvsCambio[2]);

			if (lvsCambio[1].equals("A"))
			{
				PFC_GestorXML.crearNodo(lvsCambio[0], psNomFichero, lvsCambio[2]);
			}
			if (lvsCambio[1].equals("B"))
			{
				PFC_GestorXML.eliminarNodo(lvsCambio[0], psNomFichero);
			}
			if (lvsCambio[1].equals("M"))
			{
				PFC_GestorXML.modificarNodo(lvsCambio[0], psNomFichero, lvsCambio[2]);
			}			
		}//while
		moLog.debug("Grabado en documento cambios. Numero de cambios grabados<"+(poCambios == null ? 0 : poCambios.size())+">");
	}//grabarCambiosRecibidos
	
	
}
