package pfc.prototipo.jxta.superpeer;

import java.io.DataInput;
import java.io.DataInputStream;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.sql.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;

import pfc.prototipo.PFC_Constantes;
import pfc.prototipo.PFC_ManejoFicheros;
import pfc.prototipo.bbdd.PFC_BDGestion;
import pfc.prototipo.jxta.PFC_JxtaUtilidades;
import pfc.prototipo.jxta.PFC_Mensaje;
import pfc.prototipo.rendimiento.PFC_Rendimiento;

import net.jxta.socket.JxtaServerSocket;

public class PFC_SPGestorComunicacion implements Runnable
{

	//Trazas
	private final static Logger moLog = Logger.getLogger(PFC_SPGestorComunicacion.class);

	//Canal de comunicacion Multicast
	private JxtaServerSocket moSocket = null;	
	// Indica cuando acaba el proceso
	private boolean mbSeguir = true;
    //Conexion con la base de datos
    private PFC_BDGestion moGestionBD = null;

	public PFC_SPGestorComunicacion(JxtaServerSocket poSocket, PFC_BDGestion poGestionBD)
	{
		//Se inicializa canal de comunicacion
		this.moSocket = poSocket;
		moGestionBD = poGestionBD;
	}//PFC_SPGestorComunicacion()

	public void run() 
	{
		Socket loSocket = null;
		PFC_Mensaje loRespuesta = null;
		OutputStream loSalida = null; 
		DataOutput loDatosSalida = null; 
		byte[] loDatos = null;
		int liTamano;
		
		
		try
		{
			moLog.info("Se inicia proceso de escucha de mensajes por el canal UniCast");

			while (mbSeguir)
			{							
				moLog.info("Canal Unicast del SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> se queda a la espera de recibir mensajes");
				System.out.println("*****************Canal Unicast del SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> se queda a la espera de recibir mensajes");

				try
				{
					
					loSocket = moSocket.accept();
	
					moLog.info("Canal Unicast del SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> ha recibido un mensaje..Pasa a su tratamiento");
					System.out.println("*******************Canal Unicast del SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> ha recibido un mensaje..Pasa a su tratamiento");
									
					if (loSocket != null) 
					{
						// Se trata mensaje recibido
						loRespuesta = tratarMensaje(loSocket);
						
						// Si el mensaje devuelto es null no se tiene que contestar; si es un mensaje construido, se tiene que contestar al peticionario
						// Flujo de salida de datos
						if (loRespuesta != null)
						{
							loSalida = loSocket.getOutputStream();
							loDatosSalida = new DataOutputStream(loSalida);
									
							loDatos = PFC_JxtaUtilidades.convertirObjectByteArray(loRespuesta);			
							liTamano = loDatos.length;
							
							moLog.info("Se va a enviar mensaje contestacin con Tamao<"+liTamano+">");
							System.out.println("***********Se va a enviar mensaje contestacin con Tamao<"+liTamano+">");
							
							loDatosSalida.writeInt(liTamano);
							loSalida.write(loDatos);
							loSalida.flush();
							loSalida.close();
						}
					}
					else
					{
						moLog.info("Canal Unicast del SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> ha recibido un mensaje con socket NULL");
						System.out.println("*******************Canal Unicast del SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> ha recibido un mensaje con socket NULL");					
					}
				}catch(Exception e)
				{
					moLog.error("Problemas al procesar un mensaje recibido por el canal Unicast:"+e);
				}
				finally
				{
					// Se cierra comunicacin
					if (loSocket != null)
						loSocket.close();
				}
			}//while		

		}catch(Exception e)
		{
			moLog.error("Problemas al recibir mensajes del tipo Unicast:"+e.getMessage());
		}
		
	}//run()
	
	private PFC_Mensaje tratarMensaje(Socket poSocket)
	{
		//Mensaje recibido
		PFC_Mensaje loMensaje = null;
		InputStream loInput = null; 
		DataInput loDatosIn = null; 
		int liTam = 0;
		byte[] loDatos = null;
		PFC_Mensaje loRespuesta = null;
		
		try
		{
			moLog.debug("Peticion de tratar un mensaje recibido por el canal Unicast: Socket<"+poSocket+">");
			// Se obtiene mensaje
			loInput = poSocket.getInputStream();
			loDatosIn = new DataInputStream(loInput);
			moLog.debug("Canales de entrada abiertos..");
			//Obtenemos tamanyo del mensaje
			liTam = loDatosIn.readInt();
			moLog.debug("Mensaje recibido de Tamanyo<"+liTam+">");
			//Se obtiene el objeto mensaje enviado
			loDatos = new byte[liTam];
			loDatosIn.readFully(loDatos);
			loMensaje = (PFC_Mensaje) PFC_JxtaUtilidades.convertirByteArrayObject(loDatos);
			moLog.info("Mensaje recibido de<"+loMensaje.getPeerOrigen()+"> y Tipo<"+loMensaje.getTipoMensaje()+">");
			System.out.println("**********Mensaje recibido de<"+loMensaje.getPeerOrigen()+"> y Tipo<"+loMensaje.getTipoMensaje()+">");

			// Se inicializa medidor del rendimiento
			PFC_Rendimiento loRendimiento = new PFC_Rendimiento("(Unicast)"+loMensaje.getTipoMensaje());					
			
			// En funciona tipo de mensaje se aplica tratamiento
			if (loMensaje.getTipoMensaje().equals(PFC_Mensaje.TIPO_PETICION_ACTUALIZACION))
			{
				loRespuesta = gestionarPeticionActualizaciones(loMensaje);
			}
			// Peticion de documento. Funcion de Data Center
			if (loMensaje.getTipoMensaje().equals(PFC_Mensaje.TIPO_PETICION_ALTA))
			{
				loRespuesta = gestionarPeticionAlta(loMensaje);
			}
			
			//Calculo del tiempo utilizado en dar de alta un documento.
			loRendimiento.tiempo();								
		}catch(Exception e)
		{
			moLog.error("Problemas al tratar el mensaje Unicast:"+e);
		}
		finally
		{
			try
			{
				loInput.close();
			}catch(Exception e)
			{
				moLog.error("Problemas al cerrar los datos de entrada:"+e.getMessage());
			}
		}
		
		return loRespuesta;
	}//tratarMensaje
	
	private PFC_Mensaje gestionarPeticionActualizaciones(PFC_Mensaje poMensaje) throws Exception
	{
		// Documentos censados por SP que no son propiedad del nodo peticionario
		Hashtable loDocsSuperPeer = null;
		Hashtable loDocsPeer = null;
		String lsIdDoc = null;
		//Mensaje respuesta con las actualizaciones a realizar
		PFC_Mensaje loRespuesta = new PFC_Mensaje(PFC_Constantes.NODO_ID, PFC_Mensaje.TIPO_ACTUALIZACION);
		PFC_Mensaje loMensaje = null;
		
		moLog.info("Se inicia proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+">");
		
		// Se obtienen los documentos que el SuperPeer tiene censados que NO sean propiedad del Peer peticionario
		loDocsSuperPeer = moGestionBD.obtenerMisNoDocumentos(poMensaje.getPeerOrigen());
		
		// Se obtienen los documentos del Peer peticionario que ha enviado conforme NO son suyos y busca actualizaciones
		loDocsPeer = poMensaje.getDocumentos();
		
		System.out.println("***********SuperPeer tiene<"+(loDocsSuperPeer != null ? loDocsSuperPeer.keySet().size() : 0)+">" +
				"documentos que no son del peticionario. Y el peticionario" +
				" tiene<"+(loDocsPeer != null ? loDocsPeer.keySet().size() : 0)+">" +
				"documentos no suyos");
		
		// En primer lugar se recorre la lista de Documentos obtenida del peer y se verifica si hay alguno que NO existe en la lista del SuperPeer
		// esto significara que se ha dado de baja y se guardaria una Baja
		Enumeration loKeys = loDocsPeer.keys();
		while (loKeys.hasMoreElements())
		{
			lsIdDoc = (String)loKeys.nextElement();
			
			if (loDocsSuperPeer.get(lsIdDoc) == null)
			{
				moLog.debug("Se detecta documento a dar de baja. Documento<"+lsIdDoc+">");
				String[] lvsDatos = (String[])loDocsPeer.get(lsIdDoc);
				// Se crea un sub-mensaje del tipo Baja
				loMensaje = new PFC_Mensaje(PFC_Mensaje.TIPO_BAJA);
				loMensaje.setIdDocumento(lsIdDoc);
				loMensaje.setPeerOrigen(lvsDatos[0]);
				loMensaje.setNombreDocumento(lvsDatos[1]);
				loRespuesta.addActualizacion(loMensaje);
			}
			
		}//while
		
		// Ahora se recorre la lista de documentos del SuperPeer para verificar si hay cambios o altas
		loKeys = null;
		loKeys = loDocsSuperPeer.keys();
		while (loKeys.hasMoreElements())
		{
			lsIdDoc = (String)loKeys.nextElement();
			String[] lvsDatos = (String[])loDocsSuperPeer.get(lsIdDoc);
			if (loDocsPeer.get(lsIdDoc) == null)
			{
				moLog.debug("Se detecta documento a dar de alta. Documento<"+lsIdDoc+">");
				
				// Se crea un sub-mensaje del tipo Baja
				loMensaje = new PFC_Mensaje(PFC_Mensaje.TIPO_AVISO_ALTA);
				loMensaje.setIdDocumento(lsIdDoc);
				loMensaje.setPeerOrigen(lvsDatos[0]);
				loMensaje.setNombreDocumento(lvsDatos[1]);
				loMensaje.setFechaDocumento(PFC_JxtaUtilidades.obtenerFecha(lvsDatos[2]));
				loRespuesta.addActualizacion(loMensaje);
				moLog.debug("Mensaje aviso de alta creado.");
			}
			else
			{
				// En este punto el documento si que existe en el Peer peticionario. Se verifica si ha habido cambios respecto a la fecha actualizacion que tiene el peer
				// Se obtiene fecha actualizacion que tiene SuperPeer y fecha actualizacion que tiene peer. Si la fecha SuperPeer es superior a la del peer significa que ha habido cambios
				String lsFechaSuperPeer = ((String[])loDocsSuperPeer.get(lsIdDoc))[2];
				String lsFechaPeer = ((String[])loDocsPeer.get(lsIdDoc))[2];
				if (hayQueActualizar(lsFechaSuperPeer, lsFechaPeer))
				{
					moLog.info("Se ha detectado una actualizacion del documento<"+lsIdDoc+">");
					
					// Se obtiene actualizaciones disponibles
					Vector loActualizaciones = moGestionBD.obtenerCambiosDocumento(lsIdDoc, PFC_Constantes.NOTIFICACION_MODIFICACION,  PFC_JxtaUtilidades.obtenerFecha(lsFechaPeer));
					moLog.info("Numero de actualizaciones<"+(loActualizaciones == null ? 0 : loActualizaciones.size())+"> para el documento<"+lsIdDoc+">");
					loMensaje = new PFC_Mensaje(PFC_Mensaje.TIPO_CAMBIO);
					// Se establece el nombre del document
					loMensaje.setNombreDocumento(lvsDatos[1]);
					// Propietario del documento
					loMensaje.setPeerOrigen(lvsDatos[0]);
					// Se establece el identificador del document
					loMensaje.setIdDocumento(lsIdDoc);		
					//Se establece la fecha del documento
					loMensaje.setFechaDocumento(PFC_JxtaUtilidades.obtenerFecha(lvsDatos[2]));						
					// Se establecen los cambios realizados
					loMensaje.setCambio(loActualizaciones); 
					loRespuesta.addActualizacion(loMensaje);
					moLog.debug("Mensaje aviso de modificacion creado.");					
				}
				
			}
			
		}//while
		
		
		moLog.info("Proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+"> REALIZADO!");
		
		return loRespuesta;
		
	}//gestionarPeticionActualizaciones
	
	/**
	 * <p>Indica si hay actualizaciones pendientes para un determinado peer</p>
	 * <p>Compara la fecha de actualizacion que tiene el SuperPeer con la fecha de actualizacion que tiene el Peer. Si la fecha de actualizacion
	 * del SuperPeer es syperior a la del Peer significa que hay actualizaciones pendientes de enviar.</p>
	 * @param psFechaSuperPeer Fecha actualizacion que tiene SuperPeer
	 * @param psFechaPeer Fecha actualizacion que tiene Peer
	 * @return
	 * @throws Exception
	 */
	private boolean hayQueActualizar(String psFechaSuperPeer, String psFechaPeer) throws Exception
	{
		//Se verifica si la fecha del SuperPeer es mayor a la del peer lo que significara que hay cambios pendientes de enviar al peer
		System.out.println("***********Se verifica si Peer<"+psFechaPeer+"> tiene la misma actualizacion del doc que el Superpeer<"+psFechaSuperPeer+">");
				
		if ( PFC_JxtaUtilidades.obtenerFecha(psFechaSuperPeer).getTime() > PFC_JxtaUtilidades.obtenerFecha(psFechaPeer).getTime()) 
		{			
			return true;
		}
		
		
		return false;
	}

	/**
	 * <p>Trata la peticion del envio de un documento. El mensaje recibido indica el id del documento a enviar</p>
	 * @param poMensaje
	 * @return
	 * @throws Exception
	 */
	private PFC_Mensaje gestionarPeticionAlta(PFC_Mensaje poMensaje) throws Exception
	{
		String lsIdDoc = poMensaje.getIdDocumento();
		
		moLog.info("DataCenter.Peticion para gestionar el envio del alta de documento<"+lsIdDoc+">");
		
		//Se construye mensaje respuesta 
		PFC_Mensaje loMensaje = new PFC_Mensaje(PFC_Mensaje.TIPO_RESPUESTA_ALTA);
		loMensaje.setPeerOrigen(PFC_Constantes.NODO_ID);
		
		Object lvsDocumento[] = moGestionBD.obtenerDocumento(lsIdDoc);
		// Se establece el contenido del documento
		String lsNomFisico = PFC_Constantes.componeNomDocFisico((String)lvsDocumento[0], (String)lvsDocumento[3], (String)lvsDocumento[1]);
		loMensaje.setContenido(PFC_ManejoFicheros.getFicheroByteArray(lsNomFisico));
		// Se establece el nombre
		loMensaje.setNombreDocumento((String)lvsDocumento[1]);
		// Se establece la fecha
		loMensaje.setFechaDocumento(new Date((Long)lvsDocumento[2]));
		// Se establece identificador del documento
		loMensaje.setIdDocumento(lsIdDoc);
		// Se establece propietario
		loMensaje.setPropietario((String)lvsDocumento[3]);
		
		moLog.info("DataCenter.Gestion del envio del alta de documento<"+lsIdDoc+"> REALIZADA");
		
		return loMensaje;
	}//gestionarPeticionAlta
	
	/**
	 * <p>Establece la variable que indica si tenemos que seguir con el proceso a false</p>
	 */
	public void terminarProceso()
	{
		this.mbSeguir = false;
	}
	
}//PFC_SPGestorComunicacion
