package pfc.prototipo.jxta.superpeer;

import java.io.IOException;
import java.net.DatagramPacket;
import java.sql.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;

import pfc.prototipo.PFC_Constantes;
import pfc.prototipo.PFC_ManejoFicheros;
import pfc.prototipo.bbdd.PFC_BDGestion;
import pfc.prototipo.jxta.PFC_JxtaUtilidades;
import pfc.prototipo.jxta.PFC_Mensaje;
import pfc.prototipo.rendimiento.PFC_Rendimiento;
import pfc.prototipo.xml.PFC_GestorXML;
import net.jxta.peergroup.PeerGroup;
import net.jxta.pipe.PipeService;
import net.jxta.protocol.PipeAdvertisement;
import net.jxta.socket.JxtaMulticastSocket;

public class PFC_SPGestorComunicacionMC implements Runnable 
{
	//Trazas
	private final static Logger moLog = Logger.getLogger(PFC_SPGestorComunicacionMC.class);

	//Canal de comunicacion Multicast
	private JxtaMulticastSocket moSocket = null;
	
	// Indica cuando acaba el proceso
	private boolean mbSeguir = true;
	
    //Conexion con la base de datos
    private PFC_BDGestion moGestionBD = null;

    public PFC_SPGestorComunicacionMC(JxtaMulticastSocket poSocket, PFC_BDGestion poGestionBD)
	{
		moLog.debug("Se instancia Thread de Gestion de Comunicaciones Multicast para SuperPeer<"+PFC_Constantes.NODO_NOMBRE+">");
		moSocket = poSocket;
		moGestionBD = poGestionBD;
	}
	
	/**
	 * <p>Recibe mensaje y los procesa</p>
	 */
	public void run() 
	{
		byte[] loBuffer = null; 
		DatagramPacket loDatos2 = null;
		
		try
		{
			moLog.info("Se inicia proceso de escucha de mensajes por el canal MultiCast");

			while (mbSeguir)
			{							
				loBuffer = new byte[64*1024];
				loDatos2 = new DatagramPacket(loBuffer, loBuffer.length);
				
				moLog.info("Canal Multicast del SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> se queda a la espera de recibir mensajes");
				System.out.println("*****************Canal Multicast del SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> se queda a la espera de recibir mensajes");
				
				moSocket.receive(loDatos2);
				
				moLog.info("Canal Multicast del SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> ha recibido un mensaje..Pasa a su tratamiento");
				System.out.println("***********Canal Multicast del SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> ha recibido un mensaje de<"+loBuffer.length+">");
				
				
				//Tratamos el mensaje recibido
				this.tratarMensaje(loDatos2.getData());
				
			}//while
			
		}catch(Exception e)
		{
			moLog.error("Problemas al recibir mensajes del tipo Multicast:"+e.getMessage());
		}
		
	}//run()
	
	/**
	 * <p>Centraliza todos los mensajes y en funcion del tipo realiza las tareas oportunas</p>
	 * @param poDatos
	 */
	private void tratarMensaje(byte[] poDatos)
	{
		//Mensaje recibido
		PFC_Mensaje loMensaje = null;
		boolean lbTratarOk = false;
		
		try
		{
			// Obtenemos el contenido del mensaje
			loMensaje = (PFC_Mensaje) PFC_JxtaUtilidades.convertirByteArrayObject(poDatos);		
			
			trataTipologiaMensaje(loMensaje);
			
		}catch(Exception e)		
		{
			moLog.error("Problemas al tratar el mensaje:"+e);
		}
	}//tratarMensaje
	
	/**
	 * <p>Gestiona las diferentes tipologias de mensajes</p>
	 * @param poMensaje
	 * @throws Exception
	 */
	private void trataTipologiaMensaje(PFC_Mensaje poMensaje) throws Exception
	{
		boolean lbTratarOk = false;
		
		// Se obtiene el tipo de mensaje
		String lsTipoMensaje = poMensaje.getTipoMensaje();
		
		moLog.info("Mensaje recibio de<"+poMensaje.getPeerOrigen()+"> del Tipo<"+lsTipoMensaje+">");
		System.out.println("***********Mensaje recibio de<"+poMensaje.getPeerOrigen()+"> del Tipo<"+lsTipoMensaje+">");

		// Se inicializa medidor del rendimiento
		PFC_Rendimiento loRendimiento = new PFC_Rendimiento("(Multicast)"+lsTipoMensaje);		
		
		// Aplicar tratamiento al mensaje recibido			
		if (lsTipoMensaje == null)
			throw new Exception("Tipo de mensaje no informado. NULL");
		
		if (lsTipoMensaje.equals(PFC_Mensaje.TIPO_NUEVO))
		{
			gestionarAltaDocumento(poMensaje);
			lbTratarOk = true;
		}
		if (lsTipoMensaje.equals(PFC_Mensaje.TIPO_BAJA))
		{
			gestionarBajaDocumento(poMensaje);
			lbTratarOk = true;
		}
		if (lsTipoMensaje.equals(PFC_Mensaje.TIPO_CAMBIO))
		{
			gestionarModificacionDocumento(poMensaje);
			lbTratarOk = true;
		}						
		if (lsTipoMensaje.equals(PFC_Mensaje.TIPO_ENVIO_ACTUALIZACIONES))
		{
			gestionarActualizaciones(poMensaje);
			lbTratarOk = true;
		}			
		
		if (!lbTratarOk)
		{
			moLog.info("Tpo de mensaje desconocido<"+lsTipoMensaje+">");
		}
		
		
		//Calculo del tiempo utilizado en dar de alta un documento.
		loRendimiento.tiempo();					
	}//trataTipologiaMensaje
	
	/**
	 * <p>Gestiona la alta de un nuevo documento.</p>
	 * <p>Guarda referencia del documento en base de datos.</p>
	 * @param poMensaje
	 * @throws Exception
	 */
	private void gestionarAltaDocumento(PFC_Mensaje poMensaje) throws Exception
	{
		
		moLog.info("Se inicia proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+">");
				
		// Como el SuperPeer tambien actua de Data Center, se guardar copia fisica del documento cuando
		// se trata de una alta
		String lsNombreDocumento = PFC_Constantes.componeNomDocFisico(PFC_Constantes.DIRECTORIO, poMensaje.getPropietario(), poMensaje.getNombreDocumento());
		
		PFC_ManejoFicheros.getByteArrayFile(lsNombreDocumento, poMensaje.getContenido());
		
		// El SuperPeer lo unico que tiene que hacer es guardarse referencia del documento
		moGestionBD.guardarDocumentoSP(poMensaje.getIdDocumento(), poMensaje.getPropietario(), 
				poMensaje.getNombreDocumento(), poMensaje.getFechaDocumento(), PFC_Constantes.DIRECTORIO);
			
		moLog.info("Proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+"> REALIZADO!");
		
	}//gestionarAltaDocumento
	
	/**
	 * <p>Gestiona la baja de un documento.</p>
	 * <p>Elimina referencia del Documento</p>
	 * @param poMensaje
	 * @throws Exception
	 */
	private void gestionarBajaDocumento(PFC_Mensaje poMensaje) throws Exception
	{
		
		moLog.info("Se inicia proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+">");
				
		// Se obtiene la informacion de la ubicacion y nombre del fichero
		Object[] loDatos = moGestionBD.obtenerDocumento(poMensaje.getIdDocumento());
		
		// El SuperPeer lo unico que tiene que hacer es dar de baja la referencia del documento y dar de baja cualquier modificacion
		//que pueda tener
		moGestionBD.eliminarDocumento(poMensaje.getIdDocumento());
		moGestionBD.eliminarCambios(poMensaje.getIdDocumento());

		// Al actura como Data Center, se elimina fichero de disco		
		String lsNomFisico = PFC_Constantes.componeNomDocFisico((String)loDatos[0], poMensaje.getPeerOrigen(), (String)loDatos[1]);
		PFC_ManejoFicheros.borrarFichero(lsNomFisico);

		
		moLog.info("Proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+"> REALIZADO!");
		
	}//gestionarBajaDocumento

	/**
	 * <p>Gestiona los cambios realizados en un documento.</p>
	 * <p>Actualiza la base de datos con cada uno de los cambios realizados</p>
	 * @param poMensaje
	 * @throws Exception
	 */
	private void gestionarModificacionDocumento(PFC_Mensaje poMensaje) throws Exception
	{
		
		moLog.info("Se inicia proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+">");
					
		// Se guardan los cambios en base de datos
		grabarCambiosRecibidos(poMensaje);
		
		// Se actualiza la fecha del documento
		moGestionBD.modificarDocumento(poMensaje.getIdDocumento(), poMensaje.getPeerOrigen(), poMensaje.getFechaDocumento());
		
		moLog.info("Proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+"> REALIZADO!");
		
	}//gestionarBajaDocumento

	
	/**
	 * <p>Graba los cambios recibidos en la base de datos para poder notificarlos</p>
	 * @param poMensaje Mensaje recibido
	 * @throws Exception
	 */
	public  void grabarCambiosRecibidos(PFC_Mensaje poMensaje) throws Exception
	{
		Vector loCambios = poMensaje.getCambio();	
		
		moLog.debug("Peticion para grabar en documentos cambios. Numero de cambios detectados<"+(loCambios == null ? 0 : loCambios.size())+">");
		for (int i = 0; i < loCambios.size(); i++)		
		{
			String[] lvsCambio = (String[])loCambios.get(i);
			System.out.println("Nodo<"+lvsCambio[0]+">Tipo de cambio<"+lvsCambio[1]+">Orden<"+lvsCambio[3]+"> Valor:\n"+lvsCambio[2]);

			// Se crea registro en base de datos con los cambios
			moGestionBD.altaNotificacion(poMensaje.getIdDocumento(), PFC_Constantes.NOTIFICACION_MODIFICACION, 
					poMensaje.getFechaDocumento(), lvsCambio[0], lvsCambio[1], lvsCambio[2], lvsCambio[3]);
		}//while
		moLog.debug("Grabado en DOC_CAMBIOS cambios. Numero de cambios grabados<"+(loCambios == null ? 0 : loCambios.size())+">");
	}//grabarCambiosRecibidos	
	
	/**
	 * <p>Gestiona las actualizaciones enviadas en bloque por un Peer.</p>
	 * <p>Actualiza la base de datos con cada uno de los cambios realizados</p>
	 * @param poMensaje
	 * @throws Exception
	 */
	private void gestionarActualizaciones(PFC_Mensaje poMensaje) throws Exception
	{
		PFC_Mensaje loMensaje = null;
		
		moLog.info("Se inicia proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+">");
		// Se obtienen todos los cambios indicados por el peer origen			
		Vector loActualizaciones = poMensaje.getActualizaciones();
		int liCambios = (loActualizaciones != null ? loActualizaciones.size() : 0);
		moLog.info("Numero de cambios indicados<"+liCambios+">");
		
		for (int i=0; i < liCambios; i++)
		{
			loMensaje = (PFC_Mensaje) loActualizaciones.elementAt(i);
			
			// Se trata tipologia de actualizaciones
			trataTipologiaMensaje(loMensaje);
			
			
		}//for (int i=0; i < loActualizaciones.size(); i++)
		
		
		moLog.info("Proceso de tratamiento mensaje tipo<"+poMensaje.getTipoMensaje()+"> recibido desde<"+poMensaje.getPeerOrigen()+"> REALIZADO!");
		
	}//gestionarBajaDocumento
	
	
	/**
	 * <p>Establece la variable que indica si tenemos que seguir con el proceso a false</p>
	 */
	public void terminarProceso()
	{
		this.mbSeguir = false;
	}
	
}
