package pfc.prototipo.jxta.superpeer;

import java.io.DataInput;
import java.io.DataInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringWriter;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.MessageFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.StringTokenizer;

import javax.security.cert.CertificateException;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import pfc.prototipo.PFC_Constantes;
import pfc.prototipo.bbdd.PFC_BDGestion;
import pfc.prototipo.jxta.PFC_JxtaUtilidades;
import pfc.prototipo.jxta.peer.PFC_PGestorComunicacion;
import pfc.prototipo.jxta.peer.PFC_PGestorComunicacionMC;

import net.jxta.credential.AuthenticationCredential;
import net.jxta.credential.Credential;
import net.jxta.discovery.DiscoveryEvent;
import net.jxta.discovery.DiscoveryListener;
import net.jxta.discovery.DiscoveryService;
import net.jxta.document.Advertisement;
import net.jxta.document.AdvertisementFactory;
import net.jxta.document.MimeMediaType;
import net.jxta.document.StructuredDocument;
import net.jxta.document.StructuredTextDocument;
import net.jxta.endpoint.Message;
import net.jxta.endpoint.StringMessageElement;
import net.jxta.exception.PeerGroupException;
import net.jxta.id.ID;
import net.jxta.id.IDFactory;
import net.jxta.membership.Authenticator;
import net.jxta.membership.MembershipService;
import net.jxta.peer.PeerID;
import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupID;
import net.jxta.pipe.InputPipe;
import net.jxta.pipe.OutputPipe;
import net.jxta.pipe.OutputPipeEvent;
import net.jxta.pipe.OutputPipeListener;
import net.jxta.pipe.PipeMsgEvent;
import net.jxta.pipe.PipeMsgListener;
import net.jxta.pipe.PipeService;
import net.jxta.platform.NetworkConfigurator;
import net.jxta.platform.NetworkManager;
import net.jxta.protocol.ModuleImplAdvertisement;
import net.jxta.protocol.PeerGroupAdvertisement;
import net.jxta.protocol.PipeAdvertisement;
import net.jxta.rendezvous.RendezVousService;
import net.jxta.rendezvous.RendezvousEvent;
import net.jxta.rendezvous.RendezvousListener;
import net.jxta.socket.JxtaMulticastSocket;
import net.jxta.socket.JxtaServerSocket;

public class PFC_SuperPeer
{
	//Trazas
	private final static Logger moLog = Logger.getLogger(PFC_SuperPeer.class);

	//Red
	private NetworkManager moRed = null;

	//Servicio de SuperPeer
	private DiscoveryService moSrvSuperPeer = null;
	//Servicio de Grupo
	private DiscoveryService moSrvGrupoPeer = null;
	//Grupo de Peers
	private PeerGroup moGrupo = null;

	//Gestion de la base de datos
	private PFC_BDGestion moGestionBD = null;

	// Canal de comunicacion Multicast
	private JxtaMulticastSocket moSocketMC = null;
	
	// Gestiona mensajes de entrada canal Multicast
	private PFC_SPGestorComunicacionMC moGestioComunicacionMC = null;

	// Gestiona mensajes de entrada canal Multicast
	private PFC_SPGestorComunicacion moGestioComunicacion = null;
	
	// Canal de comunicaion Unicast
	private JxtaServerSocket moSocket = null;
	
	/**
	 * <p>Inicializa configuracion y crea conexion al grupo</p>
	 */
	public PFC_SuperPeer()
	{
		NetworkConfigurator loRedConfig;
		File loConfigLocal = null; 
		PeerGroup loRedGrupo = null;
		PeerGroup loTrabajoGrupo = null;
		PeerGroupAdvertisement loAdGrupo = null;
		Enumeration<Advertisement> loAnuncios = null;
					
		//Indica si existe configuracion
		boolean lbExisteConfig = false;
		
		try
		{
			//Se crea conexion con base de datos.
			moGestionBD = new PFC_BDGestion();
			
			//Se verifica si existe una configuracion creada. En caso de no existir, se genera de nuevo.
			//Si no existe es porque la aplicacion se utiliza por primera vez y es cuando se tiene que generar los identificadores unicos
			//del superpeer y de los canales de comunicacion necesarios
			lbExisteConfig = moGestionBD.existeTabla(PFC_Constantes.TABLA_CONFIGURACION);
			
			if (lbExisteConfig)
			{
				moLog.info("Se carga configuracion.");
				PFC_Constantes.cargarConfiguracion(moGestionBD);
			}
			else
			{
				moLog.info("Aplicacion en proceso de inicializacion. Se carga configuracion por primera vez");
				//La primera vez que se inicia, se establece que el directorio es el path actual mas el establecido por defecto
				PFC_Constantes.DIRECTORIO = new File(".").getCanonicalPath() + PFC_Constantes.DIRECTORIO;
				moLog.info("Se establece el directorio de trabajo inicial<"+PFC_Constantes.DIRECTORIO+">");
				PFC_Constantes.crearConfiguracion(moGestionBD, true);
			}

		}catch(Exception e)
		{
			moLog.error("No se ha podido iniciar la solucion:"+e.getLocalizedMessage());
			System.out.println("No se ha podido iniciar la solucion");
			if (moGestionBD != null)
			{
				moGestionBD.cerrarConexion();
			}
			System.exit(-1);
		}

		// Inicializacion de conexion a la red
		
		try 
		{
			moLog.info("Se inicia conexion a la red con el SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			//Iniciamos conexion a la red que se generara. En primer lugar se indica el tipo de SuperPeer que se quiere. En segundo el nombre
			//de la red que se crea, en tercer lugar se genera un fichero de configuracion con el nombre del SuperPeer
			moRed = new NetworkManager(NetworkManager.ConfigMode.RENDEZVOUS_RELAY,
					PFC_Constantes.GRUPO_TRABAJO_NOMBRE, new File(new File(".cache"), PFC_Constantes.NODO_NOMBRE).toURI());
						
		} catch (Exception e) 
		{
			moLog.error("No se puede crear la conexion con el SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">:"+e.getMessage());
			System.out.println("No se puede crear la conexion con el SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			System.exit(-1);
		}		
		
		//Hacemos persistente el ID de Peer
		moRed.setConfigPersistent(true);
				
		//Gestin de la configuracin
		try 
		{
			//Miro si existe configuracin local. En caso de existir, obtengo la configuracin del disco. En caso de no tenerla,
			//se crea y se guarda
			loRedConfig = moRed.getConfigurator();
		
			if (loRedConfig.exists())
			{
				moLog.info("Peer<"+PFC_Constantes.NODO_NOMBRE+"> con configuracin local. Se inicia proceso de carga");
				// Se obtiene configuracin local y se carga
				loConfigLocal = new File(loRedConfig.getHome(), "PlatformConfig");
				
				loRedConfig.load(loConfigLocal.toURI());
				moLog.info("Peer<"+ PFC_Constantes.NODO_NOMBRE+"> con configuracin local. Carga realizada!");
				
			}else
			{
				//No se dispone de configuracion local, se crea una nueva y se guarda				
				moLog.info("Peer<"+PFC_Constantes.NODO_NOMBRE+"> sin configuracin local. Se crea nueva configuracion local");				
				//Se establece nombre del peer				
				loRedConfig.setName(PFC_Constantes.NODO_NOMBRE);
				loRedConfig.setPeerId(PFC_Constantes.NODO_ID);
				System.out.println("*********** Se establece puerto<"+PFC_Constantes.SUPERPEER_PUERTO+">");
				loRedConfig.setTcpPort(new Integer(PFC_Constantes.SUPERPEER_PUERTO).intValue());
				loRedConfig.setTcpEnabled(true);
				loRedConfig.setUseMulticast(true);				
	
				//Se graba configuracion
				loRedConfig.save();
				moLog.info("Peer<"+PFC_Constantes.NODO_NOMBRE+"> sin configuracin local. Configuracion local CREADA!");
			}
					
		
		} catch (Exception e) 
		{
			moLog.error("No se puede crear cargar la configuracion para realizar la conexion con el SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">:"+e.getMessage());
			System.out.println("No se puede crear cargar la configuracion para realizar la conexion con el SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			System.exit(-1);			
		} 
		
		
		//Inicio de la Red (comunicacin)		
		try 
		{
			//Inicio de las comunicaciones
			loRedGrupo = moRed.startNetwork();			
			moLog.info("Conexion a la red con el SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> REALIZADA!");
			
			//Se obtiene servicio de grupo
			moSrvSuperPeer = loRedGrupo.getDiscoveryService();
			
		} catch (Exception e) 
		{
			moLog.error("No se puede realizar Start en la conexion con el SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">:"+e.getMessage());
			System.out.println("No se puede realizar Start en la conexion con el SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			System.exit(-1);			
		} 
		
		
		//Gestin del grupo 
		try 
		{
			moLog.info("Inicio proceso para adherirse al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			
			//En primer lugar se verifica si existe o no grupo creado. Para eso, vemos la cache(recursos locales)
			loAnuncios = moSrvSuperPeer.getLocalAdvertisements(DiscoveryService.GROUP, "Name", PFC_Constantes.GRUPO_TRABAJO_NOMBRE);
		
			if ((loAnuncios != null) && loAnuncios.hasMoreElements()) 
			{
				moLog.debug("Grupo <"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> existe en Local!");
				//Como el grupo ya existe, se obtiene del anuncio el grupo para luego poder unirse al grupo
				loAdGrupo = (PeerGroupAdvertisement) loAnuncios.nextElement();
				loTrabajoGrupo = loRedGrupo.newGroup(loAdGrupo);
				//Se une al grupo
				unirseAlGrupo(loTrabajoGrupo);
				setGrupo(loTrabajoGrupo);
			}
			else
			{
				moLog.debug("Grupo <"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> NO existe en Local!");
				//Se crea el grupo
				loTrabajoGrupo = generarGrupo(loRedGrupo);
				
				//Se publica el SuperPeer
				loTrabajoGrupo.getRendezVousService().startRendezVous();
				//Se une al grupo
				unirseAlGrupo(loTrabajoGrupo);
				setGrupo(loTrabajoGrupo);
			}
			
			//Una vez se ha unido al grupo, publicamos esa informacion
			moSrvGrupoPeer = loTrabajoGrupo.getDiscoveryService();
			moSrvGrupoPeer.publish(loTrabajoGrupo.getPeerAdvertisement());
			moSrvGrupoPeer.remotePublish(loTrabajoGrupo.getPeerAdvertisement());
			
			moLog.info("Proceso para adherirse al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> REALIZADO!");
			
		} catch (Exception e) 
		{
			moLog.error("No se puede adherir al Grupo con el SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">:"+e.getMessage());
			System.out.println("No se puede adherir al Grupo con el SuperPeer<"+PFC_Constantes.NODO_NOMBRE+"> y Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			System.exit(-1);			
		} 					
	
	}//PFC_SuperPeer
	
	/**
	 * <p>Inicio ejecucion del SuperPeer</p>
	 * @param args
	 */
	public static void main(String args[]) 
	{
		int liPuerto = -1;
		
		//Lo primero que se realiza es inicialiar el sistema de trazas. Se carga fichero de propiedades donde se indica toda la gestin del sistema de trazas
		try
		{
			//Se carga configuracion de trazas
			PropertyConfigurator.configure("log4j.properties");

		}catch(Exception e)
		{
			System.out.println("ERROR: Problemas al inicializar el sistema de trazas. Verifique su instalacin para poder utilizar la solucin.");
			System.exit(-1);
		}

		//Si se ha introducido el parametro de entrada, se verifica que sea un entero
		if ( (args != null) && (args.length != 0) )
		{			
			//Se verifica que el parametro introducido sea un numerico
			try
			{
				liPuerto =  new Integer(args[0]).intValue();
			}catch(Exception e)
			{
				//No se trata de un numero. Parametro introducido pero formato erroneo
				moLog.error("Formato del parmetro errneo. Debe ser un numrico entero.");
			}
		}
		
		
		// Si se ha introducido un numero de puerto numerico se establece como puerto de trabajo
		if (liPuerto != -1)
		{
			moLog.info("Se establece como puerto de trabajo<"+liPuerto+">");
			System.out.println("************Se establece como puerto de trabajo<"+liPuerto+">");
			PFC_Constantes.SUPERPEER_PUERTO = ""+liPuerto;
		}
		
		// Se establece nombre del SuperPeer
		PFC_Constantes.NODO_NOMBRE = PFC_Constantes.SUPERPEER_NOMBRE_PREFIJO;
		moLog.debug("Se inicia el proceso de SuperPeer con nombre asignado<"+PFC_Constantes.NODO_NOMBRE+">");
		
		
		PFC_SuperPeer loSuper = new PFC_SuperPeer();
		loSuper.ejecutaServicio();
		
	}//main
	
	
	/**
	 * <p>Inicia los servicios del SuperPeer</p>
	 */
	private void ejecutaServicio()
	{
		PipeAdvertisement loAdvPipeMC = null;
		PipeAdvertisement loAdvPipe = null;
		
		moLog.info("Se inicia Servicio del SuperPeer con nombre<"+PFC_Constantes.NODO_NOMBRE+">");

		
		try
		{
			moLog.debug("Se inicia proceso de creacin de canal de comunicacin Multicast");
			loAdvPipeMC = PFC_JxtaUtilidades.generarAdvertisement(PFC_Constantes.PIPE_MULTICAST_NOMBRE, PFC_Constantes.PIPE_MULTICAST_ID, getGrupo().getPeerGroupID(), PipeService.PropagateType);
			moSocketMC = new JxtaMulticastSocket(getGrupo(), loAdvPipeMC);
			moSocketMC.setSoTimeout(0);
			moLog.debug("Proceso de creacin de canal de comunicacin Multicast REALIZADO!");
			
			moGestioComunicacionMC = new PFC_SPGestorComunicacionMC(moSocketMC, moGestionBD);
			Thread loThread = new Thread(moGestioComunicacionMC);
			loThread.start();
			
		}catch(Exception e)
		{
			moLog.error("No se puede iniciar canal de comunicacion Multicast:"+e.getMessage());
		}
		

		//Se crea canal de comunicacion Unicast y se inicia servicio de escucha
		try
		{
			moLog.debug("Se inicia proceso de creacin del Pipe del canal de comunicacin Unicast");
			loAdvPipe = PFC_JxtaUtilidades.generarAdvertisement(PFC_Constantes.PIPE_UNICAST_NOMBRE, PFC_Constantes.PIPE_UNICAST_ID, getGrupo().getPeerGroupID(), PipeService.UnicastType);
			moSocket = new JxtaServerSocket(getGrupo(), loAdvPipe);
			moSocket.setSoTimeout(0);
			moLog.debug("Proceso de creacin de canal de comunicacin Unicast REALIZADO!");

			moGestioComunicacion = new PFC_SPGestorComunicacion(moSocket, moGestionBD);
			Thread loThread = new Thread(moGestioComunicacion);
			loThread.start();
		
		}catch(Exception e)
		{
			moLog.error("No se puede iniciar canal de comunicacion Unicast:"+e.getMessage());
		}
		
		
	}//ejecutaServicio
	

	/**
	 * <p>Crea Grupo de Trabajo</p>
	 * @param poGrupo
	 * @return
	 * @throws Exception 
	 */
	private PeerGroup generarGrupo(PeerGroup poGrupo) throws Exception 
	{
		ModuleImplAdvertisement loIAGrupo = null;
		PeerGroup loGrupo = null;
		PeerGroupAdvertisement loAnuncioGrupo = null;
	
		moLog.info("Se inicia proceso de creacin del Grupo de trabajo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
		loIAGrupo = poGrupo.getAllPurposePeerGroupImplAdvertisement();	
		loGrupo = poGrupo.newGroup(null, loIAGrupo, PFC_Constantes.GRUPO_TRABAJO_NOMBRE,PFC_Constantes.GRUPO_TRABAJO_DESC);
		loAnuncioGrupo = loGrupo.getPeerGroupAdvertisement();
		// Se publica anuncio de grupo
		moSrvSuperPeer.publish(loAnuncioGrupo);
		moSrvSuperPeer.remotePublish(loAnuncioGrupo);

		moLog.info("Creacin del Grupo de trabajo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> REALIZADA!");
			
		return loGrupo;
	}//generarGrupo

	/**
	 * <p>Permite realizar la union a un grupo de nodos</p>
	 * @param grp
	 */
	private void unirseAlGrupo(PeerGroup grp) throws Exception
	{
		StructuredDocument creds = null;

		moLog.info("Se inicia proceso de union al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
		// credenciales para el Grupo
		AuthenticationCredential authCred =	new AuthenticationCredential( grp, null, creds );

		// MembershipService para el grupo
		MembershipService membership = grp.getMembershipService();

		// Se obtiene Authenticator para el Authentication creds
		Authenticator auth = membership.apply(authCred);

		// Si todo esta bien, se une al grupo
		if (auth.isReadyForJoin())
		{
			//Se une este peer al grupo
			Credential myCred = membership.join(auth);

			moLog.debug("Union al grupo correcta. Nombre del Grupo<" + grp.getPeerGroupName()+"> ID del Grupo<"+grp.getPeerGroupID()+">");
			
			// A partir de aqui se muestran los credenciales, es solo para mostrar info por log, no hace falta
			/*System.out.println("\nCredential: ");
			StructuredTextDocument doc = (StructuredTextDocument)
			myCred.getDocument(new MimeMediaType("text/plain"));

			StringWriter out = new StringWriter();
			doc.sendToWriter(out);
			System.out.println(out.toString());
			out.close();*/
		}
		else
		{
			moLog.info("No nos hemos podido unir al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
			throw new Exception("No nos hemos podido unir al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+">");
		}
		
		moLog.info("Proceso de union al Grupo<"+PFC_Constantes.GRUPO_TRABAJO_NOMBRE+"> REALIZADO!");
	}//unirseAlGrupo

	/**
	 * <p>Establece Grupo de Trabajo</p>
	 * @param poGrupo
	 */
	private void setGrupo(PeerGroup poGrupo)
	{
		moGrupo = poGrupo;
	}//setGrupo
	
	/**
	 * <p>Obtiene grupo de trabajo</p>
	 * @return
	 */
	public PeerGroup getGrupo()
	{
		return moGrupo;
	}//getGrupo
}
