package pfc.prototipo.xml;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.StringReader;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.jdom.xpath.XPath;
import org.xml.sax.InputSource;


public class PFC_GestorXML 
{
	//Trazas
	private final static Logger moLog = Logger.getLogger(PFC_GestorXML.class);

	private boolean nodeTypeDiff = true; 
    private boolean nodeValueDiff = true;	
	
   
    
	/**
	 * <p>Genera un xml a partir de un String</p>
	 * @param psContenido String con el contenido del XML
	 * @param psRuta Ruta completa donde se quiere guardar el documento xml
	 * @throws Exception 
	 */
	public static void crearXML(String psContenido, String psRuta) throws Exception
	{
		Document loDoc = null;
		Format loFormat = null;
		XMLOutputter loXmlOut = null; 
		FileOutputStream loFichero = null; 
		
		try
		{
			//Se crea el documento XML en disco a partir del contenido pasado en el String
			moLog.info("Peticion para crear XMl con la siguiente ruta<"+psRuta+">");
			loDoc = obtenerDocument(psContenido);
			
			//Formateamos y le damos salida a disco
			loFormat = Format.getPrettyFormat(); 
			loXmlOut = new XMLOutputter(loFormat);
			loFichero = new FileOutputStream(psRuta); 
			loXmlOut.output(loDoc, loFichero);
        	loFichero.flush();
        	loFichero.close();
        	moLog.info("XMl con la siguiente ruta<"+psRuta+"> CREADO!");
			
		}catch(Exception e)
		{
			moLog.error("Problemas al generar el XML a partir del String de Contenido:"+e.getMessage());
			throw new Exception("No se ha podido crear el documento XML.");
		}
		
	}//crearXML
	
	/**
	 * <p>Carga el XMl en un String</p>
	 * @param psRuta Ruta completa donde se quiere guardar el documento xml
	 * @throws Exception 
	 */
	public static String cargarXML(String psRuta) throws Exception
	{
		SAXBuilder loBuilder = null;
		Document loDoc = null;
		XMLOutputter loXmlOut = null; 
		String lsContenido = null;
		
		try
		{
			//Se crea el documento XML en disco a partir del contenido pasado en el String
			moLog.info("Peticion para cargar XMl de la siguiente ruta<"+psRuta+">");
			loBuilder = new SAXBuilder();
			loDoc = loBuilder.build(new FileInputStream(psRuta));			
			loXmlOut = new XMLOutputter(Format.getPrettyFormat());
			lsContenido =  loXmlOut.outputString(loDoc);
			
			moLog.info("XMl de la siguiente ruta<"+psRuta+"> CARGADO");
			
        	return lsContenido;
        	
		}catch(Exception e)
		{
			moLog.error("Problemas al cargar XML de la ruta<"+psRuta+":"+e.getMessage());
			throw new Exception("No se ha podido cargar el documento XML.");
		}
		
	}//crearXML
	
	/**
	 * <p>Obtiene un Document a partir de un String con el contenido</p>
	 * @param psContenido Contenido del XML
	 * @return Document
	 * @throws Exception
	 */
	private static Document obtenerDocument(String psContenido) throws Exception
	{
		SAXBuilder loBuilder = null;
		StringReader lsContenido = null;
		InputSource lsEntrada = null;

		try
		{
			//Se crea el documento XML en disco a partir del contenido pasado en el String
			moLog.info("Peticion para crear crea un Document");
			loBuilder = new SAXBuilder();
			lsContenido = new StringReader(psContenido);
			lsEntrada = new InputSource(lsContenido);
			loBuilder.setFastReconfigure(true);
			moLog.info("Peticion de crear un Document REALIZADA!");
			
			return loBuilder.build(lsEntrada);
			
        	
			
		}catch(Exception e)
		{
			moLog.error("Problemas al crear un Document:"+e.getMessage());
			throw new Exception("Problemas al crear un Document");
		}
		
	}//obtenerDocument
	
	/**
	 * <p>Elimina un nodo</p>
	 * @param psNodo Path del nodo a eliminar
	 * @param psFichero Ruta del fichero donde borrar el nodo
	 */
	public static void eliminarNodo(String psNodo, String psFichero)
	{
		SAXBuilder loBuilder = null;
		Document loDoc = null;
		Element loNodo = null;
		String lsNodoaBorrar = null;
		
		try
		{
			moLog.info("Peticin para eliminar el nodo<"+psNodo+"> del documento<"+psFichero+">");
			
			//Se obtiene contenido del fichero xml
			loBuilder = new SAXBuilder();
			loDoc = loBuilder.build(new FileInputStream(psFichero));			
			
			moLog.debug("Contenido del fichero<"+psFichero+"> cargada!");
			
			//Se obtiene el nodo indicado
			loNodo = (Element)XPath.selectSingleNode(loDoc, psNodo);  
			
			moLog.debug("Nodo<"+psNodo+"> = <"+loNodo+">");
			
			// Se verifica si es nodo raiz. En tal caso no se deja borrar
			if (loNodo.isRootElement())
			{
				moLog.info("Se quiere eleminiar nodo Raiz. No se permite");
				throw new Exception("Se quiere eleminiar nodo Raiz. No se permite");
			}
			
			// Se obtiene nombre del nodo a Borrar
			lsNodoaBorrar = loNodo.getName();
			
			// Se elimina nodo			
			moLog.debug("Se elimina nodo<"+lsNodoaBorrar+">");			
			loNodo.getParentElement().getChildren(lsNodoaBorrar).remove(getPosicion(psNodo));
			
			// Se guarda el cambio en el fichero
        	volcarFichero(loDoc, psFichero);

			moLog.info("Peticin para eliminar el nodo<"+psNodo+"> del documento<"+psFichero+"> REALIZADA");
        	
		}catch(Exception e)
		{
			moLog.error("Problemas al borrar el nodo<"+psNodo+"> del fichero<"+psFichero+">:"+e);
		}
		
	}//eliminarNodo
	
	private static int getPosicion(String psNodo)
	{
		int liInicio = psNodo.lastIndexOf("[");
		int liFin = psNodo.lastIndexOf("]");
		
		return new Integer(psNodo.substring(liInicio+1, liFin)).intValue()-1;
	}
	
	/**
	 * <p>Crea un nodo en el path indicado con el contenido especificado</p>
	 * @param psNodo Path del nodo a eliminar
	 * @param psFichero Ruta del fichero donde borrar el nodo
	 * @param psContenido Contenido del nodo a crear
	 */
	public static void crearNodo(String psNodo, String psFichero, String psContenido)
	{
		SAXBuilder loBuilder = null;
		Document loDoc = null;
		Document loDocMod = null;
		Element loNodo = null;
		String lsPathPadre = null;
		Element loNodoPadre = null;

		
		try
		{
			moLog.info("Peticin para crear el nodo<"+psNodo+"> en el documento<"+psFichero+">Contenido:\n"+psContenido);
			
			//Se obtiene contenido del fichero xml
			loBuilder = new SAXBuilder();
			loDoc = loBuilder.build(new FileInputStream(psFichero));			
			
			moLog.debug("Contenido del fichero<"+psFichero+"> cargada!");
			
			//Se obtiene el nodo indicado  
			loDocMod = obtenerDocument(psContenido);
			
			//Se obtiene el nodo raiz que es realmente el nodo que queremos crear
			loNodo = loDocMod.detachRootElement();
			lsPathPadre = getPadre(psNodo);
			
			// Se obtiene el path padre para aadir el nuevo nodo
			loNodoPadre = (Element)XPath.selectSingleNode(loDoc, lsPathPadre);
			loNodoPadre.addContent(loNodo);
			
			// Volcamos la informacion al fichero
			volcarFichero(loDoc, psFichero);
			
			moLog.info("Peticin para crear el nodo<"+psNodo+"> del documento<"+psFichero+"> REALIZADA");
        	
		}catch(Exception e)
		{
			moLog.error("Problemas al crear el nodo<"+psNodo+"> del fichero<"+psFichero+">:"+e.getMessage());
		}
		
	}//crearNodo	
	
	/**
	 * <p>Modifica el contenido de un nodo</p>
	 * @param psNodo
	 * @param psFichero
	 * @param psContenido
	 */
	public static void modificarNodo(String psNodo, String psFichero, String psContenido)
	{
		
		SAXBuilder loBuilder = null;
		Document loDoc = null;
		Document loDocMod = null;
		Element loNodo = null;
		Element loNodoNuevo = null;
		String lsNodoaBorrar = null;
		
		try
		{
			moLog.info("Peticin para modificar el nodo<"+psNodo+"> del documento<"+psFichero+">");
			
			//Se obtiene contenido del fichero xml
			loBuilder = new SAXBuilder();
			loDoc = loBuilder.build(new FileInputStream(psFichero));			
			
			moLog.debug("Contenido del fichero<"+psFichero+"> cargada!");
			
			//Se obtiene el nodo indicado
			loNodo = (Element)XPath.selectSingleNode(loDoc, psNodo);  
		
			// Se obtiene el nuevo contenido del nodo
			loDocMod = obtenerDocument(psContenido);
			loNodoNuevo = loDocMod.detachRootElement();
			
			loNodo.removeContent();			
			loNodo.setText(loNodoNuevo.getText());
			
			List <Attribute> loAtributos = loNodo.getAttributes();
			int liNum = loAtributos.size();
			
			// Limpieza de atributos			
			for(int i = 0; i < liNum; i++)
			{					
				loNodo.removeAttribute(loAtributos.get(0));				
			}
			 			
			// Gestion de los atributos
			for(Attribute loAtr: (List <Attribute>) loNodoNuevo.getAttributes())
			{
				Attribute lo1 = (Attribute) loAtr.clone();
				lo1.detach();				
				loNodo.setAttribute(lo1);				
			}
			
			
			// Gestion de los hijos
			List <Element> loHijos = loNodoNuevo.getChildren();			
			for (int i = 0; i < loHijos.size(); i++)
			{
				Element loHijo = (Element) loHijos.get(i).clone();
				loHijo.detach();
				loNodo.addContent(loHijo);
			}
			
			// Se guarda el cambio en el fichero
        	volcarFichero(loDoc, psFichero);

			moLog.info("Peticin para eliminar el nodo<"+psNodo+"> del documento<"+psFichero+"> REALIZADA");
        	
		}catch(Exception e)
		{
			moLog.error("Problemas al modificar el nodo<"+psNodo+"> del fichero<"+psFichero+">:"+e);
		}
		
		
		
		
	}//modificarNodo
	
	/**
	 * <p>Vuelca el contenido de un XML (Document) en un fichero</p>
	 * @param poDoc Documento con el contenido del fichero xml
	 * @param psFichero Ruta y nombre del fichero
	 */
	private static void volcarFichero(Document poDoc, String psFichero)
	{
		Format loFormat = null;
		XMLOutputter loXmlOut = null; 
		FileOutputStream loFichero = null; 
		
		try
		{
			//Formateamos y le damos salida a disco
			loFormat = Format.getPrettyFormat(); 
			loXmlOut = new XMLOutputter(loFormat);
			loFichero = new FileOutputStream(psFichero); 
			loXmlOut.output(poDoc, loFichero);
	    	loFichero.flush();
	    	loFichero.close();
		}catch(Exception e)
		{
			moLog.error("No se puede volvar a disco el contenido de un xml:"+e.getMessage());
		}
	}//volcarFichero
	
	/**
	 * <p>Obtiene las diferencias entre dos XML.</p>
	 * <p>Las diferencias las censa en un Hashtable indicando el tipo: A, B, M</p>
	 * @param psFichero Ruta del fichero origen
	 * @param psContenido Contenido completo del fichero modificado
	 * @return
	 */
	public static Vector detectarDiferencias(String psFichero, String psContenido)
	{
		SAXBuilder loBuilder = null;
		Document loDocMod = null;
		Document loDoc = null;
		Element loRaizDoc = null;
		Element loRaizDocMod = null;
		Vector loModificaciones = null;
		int liNumCambio = 1;
		
		try
		{
			moLog.info("Peticion de busqueda de cambios para el fichero<"+psFichero+">");			
			
			//Se inicializa contenedor de cambios
			loModificaciones = new Vector();
			
			// En primer lugar creamos un XML logico con el nuevo contenido del fichero
			loDocMod = obtenerDocument(psContenido);
			
			// Ahora obtenemos el fichero xml original
			loBuilder = new SAXBuilder();
			loDoc = loBuilder.build(new FileInputStream(psFichero));			
			
			//Se obtiene el nodo raiz de cada uno de los documentos
			loRaizDoc = loDoc.getRootElement();
			loRaizDocMod = loDocMod.getRootElement();
			
			//Se verifica que el elemento raiz sea el mismo. En caso contrario indicamos al usuario que no puede modificar el nodo raiz
			if (loRaizDoc.getName() != loRaizDocMod.getName())
			{
				moLog.info("Detectada modificacin en el nombre nodo raiz del documento. No se tiene que permitir modificar este elemento");
				// Se pone a null las modificaciones para indicar que se ha intentado cambiar nombre nodo raiz
				loModificaciones = null;
				return loModificaciones;
			}
			
			// Se verifica si ha habido cambio en los atributos. En tal caso, se envia como modificacion todo el documento			
			if (detectaCambioAtributos(loRaizDoc.getAttributes(), loRaizDocMod.getAttributes()))
			{
				moLog.info("Detectada modificacin en el nodo raiz del documento por cambio de atributos. La modificacin contempla todo el documento");				
				//Se aade modificacion
				loModificaciones.add(new String[]{getPath(loRaizDoc), "M",psContenido, "1"});
				return loModificaciones;				
			}
			
			//Se verifica si ha habido cambio en el texto. En tal caso, se envia como modificacion todo el documento			
			if (!loRaizDoc.getText().trim().equals(loRaizDocMod.getText().trim()))
			{
				moLog.info("Detectada modificacin en el texto nodo raiz del documento. La modificacin contempla todo el documento");				
				//Se aade modificacion
				loModificaciones.add(new String[]{getPath(loRaizDoc), "M",psContenido, "1"});
				return loModificaciones;
			}
			
			
			
			// Se verifican si ha habido cambios en los hijos
			verificarHijos(loRaizDoc.getChildren(), loDoc, loRaizDocMod.getChildren(), loDocMod, loModificaciones, getPath(loRaizDocMod), liNumCambio);
			
			moLog.info("Peticion de busqueda de cambios para el fichero<"+psFichero+"> REALIZADA");			
			
			
		}catch(Exception e)
		{
			moLog.error("No se puede volcar a disco el contenido de un xml:"+e.getMessage());
		}
		
		return loModificaciones;
		
	}//detectarDiferencias
	

	/**
	 * <p>Obtiene el contenido de un Element en un String</p>
	 * @param poElemento Elemento a convertir
	 * @return String con el contenido del Element
	 */
	private static String getStringElement(Element poElemento)
	{
		XMLOutputter loXmlOut = null; 
		String lsContenido = null;
		
		try
		{
			//Se convierte a String el contenido completo de un Elemento
			moLog.info("Peticion para cconvertir a String un Element");
			loXmlOut = new XMLOutputter(Format.getPrettyFormat());
			lsContenido =  loXmlOut.outputString(poElemento);
			moLog.info("Peticion para convertir a String un Element REALIZADA");
        	
		}catch(Exception e)
		{
			moLog.error("Problemas al convertir un Element a String:"+e.getMessage());			
		}
		return lsContenido;		
	}//getStringElement
	
	/**
	 * <p>Verifica si hay cambio en los nodos hijos</p>
	 * @param poHijosDoc
	 * @param poDoc
	 * @param poHijosDocCam
	 * @param poDocCam
	 * @param poCambios
	 */
	private static void verificarHijos(List<Element> poHijosDoc, Document poDoc, List<Element> poHijosDocCam, Document poDocCam, Vector poCambios, String psPathPadre, int piNumCambio)
	{
		String lsPath = null;
		Hashtable loTratados = null;
		
		try
		{
			moLog.debug("Se inicia verificacin de cambio en los hijos. Numero de hijos destino<"+poHijosDocCam.size()+"> numero de hijos origen<"+poHijosDoc.size()+">");
			
			
			loTratados = new Hashtable();
			
			// Se inicia proceso recorriendo los hijos
			for (Element loHijoDocCam:  poHijosDocCam)
			{
				lsPath = psPathPadre + "/" + loHijoDocCam.getName();
				
				// Se verifica si ese path ya se ha tratado con anterioridad. En caso afirmativa, se pasa al siguiente hijo
				if (!loTratados.containsKey(lsPath))
				{										
					// Se actualiza lista de tratados
					loTratados.put(lsPath, 1);
					
					// Se obtienen todos los hijos con ese path
					List <Element> loHijosCam = XPath.selectNodes(poDocCam, lsPath);
					List <Element> loHijosOrigen = XPath.selectNodes(poDoc, lsPath);
										
					
					for(int i = 0; i < loHijosCam.size(); i++)
					{						
						Element loSubHijoDocCam = loHijosCam.get(i);
						
						// Se verifica si el hijo existe en el documento origen en el mismo path
						String lsPathHijo = lsPath + "["+(i+1)+"]";
						
						Element loSubHijoDoc = (Element)XPath.selectSingleNode(poDoc, lsPathHijo);						
						if (loSubHijoDoc == null)
						{
							//No existe en origen, por lo que se tiene que marcar como Alta
							poCambios.add(new String[]{lsPathHijo, "A",getStringElement(loSubHijoDocCam), ""+piNumCambio});
							piNumCambio++;
							continue;
						}
						
						// El nodo hijo si que existe en origen
						if (detectaCambioAtributos(loSubHijoDocCam.getAttributes(), loSubHijoDoc.getAttributes()))
						{
							// No tiene los mismos atributos por lo que se marca como cambio
							poCambios.add(new String[]{lsPathHijo, "M",getStringElement(loSubHijoDocCam), ""+piNumCambio});
							piNumCambio++;
							continue;					
						}

						//Se verifica si se ha cambiado el texto
						if (!loSubHijoDocCam.getText().trim().equals(loSubHijoDoc.getText().trim()))
						{
							poCambios.add(new String[]{lsPathHijo, "M",getStringElement(loSubHijoDocCam), ""+piNumCambio});
							piNumCambio++;
							continue;
						}


						// Se verifica si tiene hijos
						if (loSubHijoDocCam.getChildren().size() == 0)
						{
							// No tiene hijos. Se verifica si origen tiene hijos, en tal caso se tendran que marcar todos como baja
							if (loSubHijoDoc.getChildren().size() != 0)
							{
								// El origen tiene hijos por lo que tenemos que marcarlos como baja
								List<Element> loHijosTmp = loSubHijoDoc.getChildren();

								int liNum = 1;
								Hashtable loControl = new Hashtable();
								for(Element loHijoTmp: loHijosTmp)
								{
									String lsPathTmp =  lsPathHijo + "/" + loHijoTmp.getName();
									if (!loControl.contains(lsPathTmp))
									{
										loControl.put(lsPathTmp, "1");
										List <Element> loHijosPath = XPath.selectNodes(poDoc, lsPathTmp);									
										int liNumReg = loHijosPath.size();
										for (int a = 0; a < loHijosPath.size(); a++)
										{
											lsPathTmp =  lsPathTmp+"["+liNumReg+"]";
											poCambios.add(new String[]{lsPathTmp, "B","", ""+piNumCambio});
											piNumCambio++;
											liNumReg--;											
										}
									
									
									}
								}	
								continue;
							}
						}
						else
						{
							//Si que tiene hijos, por lo que se inicia el proceso de comprobacion de los hijos
							verificarHijos(loSubHijoDoc.getChildren(), poDoc, loSubHijoDocCam.getChildren(), poDocCam, poCambios, lsPathHijo, piNumCambio);
						}
						
					}//for(int i = 0; i < loHijosCam.size(); i++)
					
					
					// Se verifica si en origen habia mas elementos que en cambiado. En tal caso, se dan de baja
					int liElementos = loHijosOrigen == null ? 0 : loHijosOrigen.size();
					int liNumBajas = liElementos - loHijosCam.size();
					int liRegistro = loHijosCam.size() + liNumBajas;					
					
					if (liNumBajas > 0)
					{
						for (int i = 0; i < liNumBajas; i++)
						{
							String lsPathHijo = lsPath + "["+(liRegistro)+"]";						
							poCambios.add(new String[]{lsPathHijo, "B","", ""+piNumCambio});
							piNumCambio++;
							liRegistro--;
						}
					}
					
				}//if (!loTratados.containsKey(lsPath))
			}//for(int i = 0; i < poHijosDocCam.size(); i++)
			
			
			// Se inicia proceso recorriendo los hijos del origen para detectar bajas
			for (Element loHijoDoc:  poHijosDoc)
			{
				lsPath = psPathPadre + "/" + loHijoDoc.getName();
				
				// Se verifica si ese path ya se ha tratado con anterioridad. En caso afirmativa, se pasa al siguiente hijo
				if (!loTratados.containsKey(lsPath))
				{										
					// Se actualiza lista de tratados
					loTratados.put(lsPath, 1);

					// Se obtienen todos los hijos con ese path
					List <Element> loHijosOrigen = XPath.selectNodes(poDoc, lsPath);
					
					int liRegistro = loHijosOrigen.size();
					for(int i = 0; i < loHijosOrigen.size(); i++)
					{						
						Element loSubHijoDocCam = loHijosOrigen.get(i);
						
						// Se verifica si el hijo existe en el documento origen en el mismo path
						String lsPathHijo = lsPath + "["+(liRegistro)+"]";						
						poCambios.add(new String[]{lsPathHijo, "B","", ""+piNumCambio});
						piNumCambio++;
						liRegistro--;
					}//for(int i = 0; i < loHijosOrigen.size(); i++)
				}
			}//for (Element loHijoDoc:  poHijosDoc)
			
			

			moLog.debug("Se finaliza verificacin de cambio en los hijos");
		}catch(Exception e)
		{
			moLog.error("Problemas al verificar si ha habido cambios en los hijos:"+e.getMessage());
			e.printStackTrace();
		}
		
	}//verificarHijos
	
	
	/**
	 * <p>Detecta cambio en los atributos. Se realizan las siguientes comprobaciones:</p>
	 * <p>. Que el nmero de atributos sea el mismo</p>
	 * <p>. Si tiene el mismo nmero de atributos, que tengan el mismo nombre y valor</p>
	 * @param poAtrDoc Lista de atributos del documento original
	 * @param poAtrDocMod Lista de atributos del documento modificado
	 * @return
	 */
	private static boolean detectaCambioAtributos(List poAtrDoc, List poAtrDocMod)
	{
		Attribute loAtDoc = null;
		Attribute loAtDocMod = null;
		
		try
		{
			moLog.debug("Peticion para detectar cambios en atributos");
			//En primer lugar se verifica si el numero de atributos es el mismo; en caso negativo cambio
			if (poAtrDoc.size() != poAtrDocMod.size())
			{
				moLog.debug("Detectados cambio de numero de atributos");
				return true;
			}
			
			// Verificamos que el nombre de los atributos sean iguales y sus valores
			for (int i=0; i < poAtrDoc.size(); i++)
			{
				//Obtenemos atributo del documento original
				loAtDoc = (Attribute)poAtrDoc.get(i);
				//Obtenemos atributo del documento modificado
				loAtDocMod = (Attribute)poAtrDocMod.get(i);
				
				if ( (!loAtDoc.getName().equals(loAtDocMod.getName())) || (!loAtDoc.getValue().equals(loAtDocMod.getValue())) )
				{
					moLog.debug("Detectados cambio de nombre o valor de atributos");
					return true;
				}
				
			}//for
			
			moLog.debug("NO se han detectado cambios en atributos");
			
		}catch(Exception e)
		{
			moLog.error("No se puede verificar si hay cambios en los atributos de un nodo:"+e.getMessage());
		}
		
		return false;
	}//detectaCambioAtributos
	 
	/**
	 * <p>Obtiene el path completo del nodo</p>
	 */
    private static String getPath( Element poNode ) throws Exception 
    { 
        StringBuilder loPath = null; 
 
        loPath = new StringBuilder();
        
        do 
        {            
        	loPath.insert(0, poNode.getName() ); 
        	loPath.insert( 0, "/" ); 
        } 
        while( ( poNode = poNode.getParentElement() ) != null ); 
 
        return loPath.toString(); 
    }//getPath 	
	
    /**
     * <p>Obtiene el path del padre.</p>
     * @param psPath Ruta de un nodo
     * @return String con el path del padre
     */
    private static String getPadre(String psPath)
    {
    	return psPath.substring(0, (psPath.lastIndexOf("/")));  
    }//getPadre
    
}
